@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef sndcard_h
 #define sndcard_h
@@ -98,6 +98,9 @@ typedef struct _MSFilter * (*MSSndCardCreateWriterFunc)(struct _MSSndCard *obj);
 typedef struct _MSSndCard * (*MSSndCardDuplicateFunc)(struct _MSSndCard *obj);
 typedef void (*MSSndCardSetUsageHintFunc)(struct _MSSndCard *obj, bool_t is_going_to_be_used);
 typedef void (*MSSndCardUnloadFunc)(MSSndCardManager *obj);
+typedef void (*MSSndCardAudioSessionFunc)(struct _MSSndCard *obj, bool_t actived);
+typedef void (*MSSndCardCallKitFunc)(struct _MSSndCard *obj, bool_t enabled);
+typedef void (*MSSndCardAudioRouteFunc)(struct _MSSndCard *obj);
 
 
 struct _MSSndCardDesc{
@@ -115,6 +118,9 @@ struct _MSSndCardDesc{
 	MSSndCardDuplicateFunc duplicate;
 	MSSndCardUnloadFunc unload;
 	MSSndCardSetUsageHintFunc usage_hint;
+	MSSndCardAudioSessionFunc audio_session_activated;
+	MSSndCardCallKitFunc callkit_enabled;
+	MSSndCardAudioRouteFunc audio_route_changed;
 };
 
 /**
@@ -123,6 +129,24 @@ struct _MSSndCardDesc{
  */
 typedef struct _MSSndCardDesc MSSndCardDesc;
 
+/**
+ * Structure for sound card object.
+ * @var MSSndCard
+ */
+typedef struct _MSSndCard MSSndCard;
+
+enum _MSSndCardStreamType{
+	MS_SND_CARD_STREAM_VOICE,
+	MS_SND_CARD_STREAM_RING,
+	MS_SND_CARD_STREAM_MEDIA,
+	MS_SND_CARD_STREAM_DTMF,
+};
+
+/**
+ * Structure for sound card stream type.
+ */
+typedef enum _MSSndCardStreamType MSSndCardStreamType;
+
 #define MS_SND_CARD_CAP_DISABLED (0) /**<This soundcard is disabled.*/
 #define MS_SND_CARD_CAP_CAPTURE (1) /**<This sound card can capture sound */
 #define MS_SND_CARD_CAP_PLAYBACK (1<<1) /**<This sound card can playback sound */
@@ -138,14 +162,9 @@ struct _MSSndCard{
 	void *data;
 	int preferred_sample_rate;
 	int latency;
+	MSSndCardStreamType streamType;
 };
 
-/**
- * Structure for sound card object.
- * @var MSSndCard
- */
-typedef struct _MSSndCard MSSndCard;
-
 #ifdef __cplusplus
 extern "C"{
 #endif
@@ -258,6 +277,15 @@ MS2_PUBLIC const MSList * ms_snd_card_manager_get_list(MSSndCardManager *m);
  *
  */
 MS2_PUBLIC void ms_snd_card_manager_add_card(MSSndCardManager *m, MSSndCard *c);
+
+/**
+ * Prepend a sound card object in a sound card manager's list.
+ *
+ * @param m    A sound card manager containing sound cards.
+ * @param c    A sound card object.
+ *
+ */
+MS2_PUBLIC void ms_snd_card_manager_prepend_card(MSSndCardManager *m, MSSndCard *c);
 	
 /**
  * Set the sound card manager of a sound card.
@@ -290,6 +318,8 @@ MS2_PUBLIC void ms_snd_card_manager_register_desc(MSSndCardManager *m, MSSndCard
 **/
 MS2_PUBLIC void ms_snd_card_manager_reload(MSSndCardManager *m);
 
+/* This function is available for testing only, this should not be used in a real application! */
+MS2_PUBLIC void ms_snd_card_manager_bypass_soundcard_detection(bool_t value);
 
 /** @} */
 
@@ -519,6 +549,44 @@ MS2_PUBLIC int ms_snd_card_set_preferred_sample_rate(MSSndCard *obj,int rate);
 **/
 MS2_PUBLIC void ms_snd_card_set_usage_hint(MSSndCard *obj, bool_t is_going_to_be_used);
 
+/**
+ * Used by application to notify whether audio access is allowed for the process.
+ * On most platform this function is useless, but in an iOS application using Callkit, the system decides when audio (through the AVAudioSession singleton) is open or closed.
+ * Such application needs to explicitely notify mediastreamer2 with ms_snd_card_notify_audio_session_activated() about the state of the audio session.
+ *
+ * @param obj      A sound card object.
+ * @param actived    TRUE if audio session is activated, FALSE otherwise.
+ */
+MS2_PUBLIC void ms_snd_card_notify_audio_session_activated(MSSndCard *obj, bool_t activated);
+
+/**
+ * Used by application to notify whether audio route is changed. On most platform this function is useless.
+ * But  an iOS application needs to explicitely notify mediastreamer2 with ms_snd_card_notify_audio_route_changed() about the changment of audio route to ajust the sample rate for playback/record.
+ *
+ * @param obj      A sound card object.
+*/
+MS2_PUBLIC void ms_snd_card_notify_audio_route_changed(MSSndCard *obj);
+
+/**
+ * Used by application to tell the MSSndCard if rely on notifications of activation of audio session.
+ * When yesno is set to FALSE, the MSSndCard will not rely on notifications of activation of audio session, and will assume that audio is always usable.
+ * If set to TRUE, the mediastreamer2 will require explicit calls to ms_snd_card_notify_audio_session_activated().
+ *
+ * @param obj      A sound card object.
+ * @param yesno    TRUE if app notifies is activated, FALSE otherwise. The default value is FALSE.
+ */
+MS2_PUBLIC void ms_snd_card_app_notifies_activation(MSSndCard *obj, bool_t yesno);
+
+/**
+ * Sets the stream type for this soundcard, default is VOICE
+**/
+MS2_PUBLIC void ms_snd_card_set_stream_type(MSSndCard *obj, MSSndCardStreamType type);
+
+/**
+ * Gets the stream type for this soundcard, default is VOICE
+**/
+MS2_PUBLIC MSSndCardStreamType ms_snd_card_get_stream_type(MSSndCard *obj);
+
 /**
  * Create a alsa card with user supplied pcm name and mixer name.
  * @param pcmdev The pcm device name following alsa conventions (ex: plughw:0)
