@@ -1,21 +1,21 @@
 /*
-mediastreamer2 android_mediacodec.cpp
-Copyright (C) 2015 Belledonne Communications SARL
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "mediastreamer2/mscommon.h"
 #include "mediastreamer2/msjava.h"
@@ -82,6 +82,9 @@ struct AMediaFormat {
 	jmethodID setInteger;
 	jmethodID getInteger;
 	jmethodID setString;
+	jmethodID containsKey;
+	jmethodID toString;
+	std::string description;
 };
 
 int handle_java_exception() {
@@ -247,14 +250,11 @@ media_status_t AMediaCodec_configure(AMediaCodec *codec, const AMediaFormat* for
 	return (handle_java_exception() == -1) ? AMEDIA_ERROR_BASE : AMEDIA_OK;
 }
 
-media_status_t AMediaCodec_delete(AMediaCodec *codec) {
+void AMediaCodec_delete(AMediaCodec *codec) {
 	JNIEnv *env = ms_get_jni_env();
-
 	env->CallVoidMethod(codec->jcodec, codec->release);
 	env->DeleteGlobalRef(codec->jcodec);
 	ms_free(codec);
-
-	return (handle_java_exception() == -1) ? AMEDIA_ERROR_BASE : AMEDIA_OK;
 }
 
 media_status_t AMediaCodec_start(AMediaCodec *codec) {
@@ -398,16 +398,30 @@ media_status_t AMediaCodec_releaseOutputBuffer(AMediaCodec *codec, size_t idx, b
 	return (handle_java_exception() == -1) ? AMEDIA_ERROR_BASE : AMEDIA_OK;
 }
 
-void AMediaCodec_reset(AMediaCodec *codec) {
+media_status_t AMediaCodec_reset(AMediaCodec *codec) {
 	JNIEnv *env = ms_get_jni_env();
 	env->CallVoidMethod(codec->jcodec, codec->reset);
-	handle_java_exception();
+	if (handle_java_exception() != 0) return AMEDIA_ERROR_UNKNOWN;
+	return AMEDIA_OK;
 }
 
-void AMediaCodec_setParams(AMediaCodec *codec, const char *params) {
+static void putToBundle(JNIEnv *env, AMediaCodec *codec, jobject jbundle, AMediaFormat *fmt, const char *key){
+	int32_t value = 0;
+	
+	if (AMediaFormat_getInt32(fmt, key, &value)){
+		jstring jkey = env->NewStringUTF(key);
+		env->CallVoidMethod(jbundle, codec->putIntId, jkey, value);
+		handle_java_exception();
+		env->DeleteLocalRef(jkey);
+		ms_message("AMediaCodec_setParams() %s key transfered to Bundle with value %i", key, value);
+	}
+}
+
+void AMediaCodec_setParams(AMediaCodec *codec, const AMediaFormat * fmt) {
 	JNIEnv *env = ms_get_jni_env();
 	jobject jbundle = NULL;
 	jclass BundleClass = NULL;
+	
 
 	/* We can't stock jclass information due to JNIEnv difference between different threads */
 	if (!_loadClass(env, "android/os/Bundle", &BundleClass)) {
@@ -415,12 +429,11 @@ void AMediaCodec_setParams(AMediaCodec *codec, const char *params) {
 		handle_java_exception();
 		return;
 	}
-
-	jstring msg = env->NewStringUTF(params);
 	jbundle = env->NewObject(BundleClass, codec->_init_BundleClass);
-	env->CallVoidMethod(jbundle, codec->putIntId, msg, 0);
-	handle_java_exception();
-	env->DeleteLocalRef(msg);
+
+	putToBundle(env, codec, jbundle, (AMediaFormat *)fmt, "request-sync");
+	putToBundle(env, codec, jbundle, (AMediaFormat *)fmt, "video-bitrate");
+	putToBundle(env, codec, jbundle, (AMediaFormat *)fmt, "frame-rate");
 
 	env->CallVoidMethod(codec->jcodec, codec->setParameters, jbundle);
 	handle_java_exception();
@@ -528,6 +541,28 @@ bool_t AMediaImage_isAvailable(void) {
 	return ms_get_android_sdk_version() >= 22;
 }
 
+bool_t AMediaCodec_checkCodecAvailability(const char *mime) {
+	bool_t res = TRUE;
+	AMediaCodec *encoder = NULL, *decoder = NULL;
+
+	encoder = AMediaCodec_createEncoderByType(mime);
+	if (encoder) {
+		AMediaCodec_delete(encoder);
+	} else {
+		ms_warning("MediaCodec: '%s' format not supported for encoding", mime);
+		res = FALSE;
+	}
+	decoder = AMediaCodec_createDecoderByType(mime);
+	if (decoder) {
+		AMediaCodec_delete(decoder);
+	} else {
+		ms_warning("MediaCodec: '%s' format not supported for decoding", mime);
+		res = FALSE;
+	}
+	if (res) ms_message("MediaCodec: '%s' format supported", mime);
+	return res;
+}
+
 
 ////////////////////////////////////////////////////
 //                                                //
@@ -553,6 +588,8 @@ bool AMediaFormat_loadMethodID(AMediaFormat * format) {
 	success &= _getMethodID(env, mediaFormatClass, "setInteger", "(Ljava/lang/String;I)V", &(format->setInteger));
 	success &= _getMethodID(env, mediaFormatClass, "getInteger", "(Ljava/lang/String;)I", &(format->getInteger));
 	success &= _getMethodID(env, mediaFormatClass, "setString", "(Ljava/lang/String;Ljava/lang/String;)V", &(format->setString));
+	success &= _getMethodID(env, mediaFormatClass, "containsKey", "(Ljava/lang/String;)Z", &(format->containsKey));
+	success &= _getMethodID(env, mediaFormatClass, "toString", "()Ljava/lang/String;", &(format->toString));
 	if(!success) {
 		ms_error("%s(): one method or field could not be found", __FUNCTION__);
 		goto error;
@@ -598,21 +635,37 @@ media_status_t AMediaFormat_delete(AMediaFormat* format) {
 	return AMEDIA_OK;
 }
 
+const char *AMediaFormat_toString(AMediaFormat *format) {
+	JNIEnv *env = ms_get_jni_env();
+	jstring jdescription = static_cast<jstring>(env->CallObjectMethod(format->jformat, format->toString));
+	if (handle_java_exception() != 0) return nullptr;
+	const char *description = env->GetStringUTFChars(jdescription, nullptr);
+	format->description = description;
+	env->ReleaseStringUTFChars(jdescription, description);
+	env->DeleteLocalRef(jdescription);
+	return format->description.c_str();
+}
+
 bool AMediaFormat_getInt32(AMediaFormat *format, const char *name, int32_t *out){
 	JNIEnv *env = ms_get_jni_env();
+	bool hasValue;
 
 	if (!format) {
-		ms_error("Format nul");
+		ms_error("Format null");
 		return false;
 	}
+	
 
 	jstring jkey = env->NewStringUTF(name);
-	jint jout = env->CallIntMethod(format->jformat, format->getInteger, jkey);
-	*out = jout;
+	hasValue = env->CallBooleanMethod(format->jformat, format->containsKey, jkey);
+	
+	if (hasValue){
+		jint jout = env->CallIntMethod(format->jformat, format->getInteger, jkey);
+		*out = jout;
+	}
 	env->DeleteLocalRef(jkey);
 	handle_java_exception();
-
-	return true;
+	return hasValue;
 }
 
 void AMediaFormat_setInt32(AMediaFormat *format, const char* name, int32_t value) {
