@@ -1,21 +1,20 @@
 /*
- * androidsound.cpp -Android Media plugin for Linphone, based on C++ sound apis.
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- * Copyright (C) 2009-2012  Belledonne Communications, Grenoble, France
+ * This file is part of mediastreamer2.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <mediastreamer2/msfilter.h>
@@ -23,6 +22,7 @@
 #include <mediastreamer2/msticker.h>
 #include <mediastreamer2/mssndcard.h>
 #include <mediastreamer2/devices.h>
+#include <mediastreamer2/android_utils.h>
 
 #include "AudioTrack.h"
 #include "AudioRecord.h"
@@ -30,8 +30,6 @@
 
 #include <jni.h>
 
-#include "hardware_echo_canceller.h"
-
 #define NATIVE_USE_HARDWARE_RATE 1
 //#define TRACE_SND_WRITE_TIMINGS
 
@@ -167,7 +165,6 @@ struct AndroidSndReadData{
 	int64_t read_samples;
 	audio_io_handle_t iohandle;
 	jobject aec;
-	double av_skew;
 	bool started;
 	bool builtin_aec;
 };
@@ -185,12 +182,24 @@ struct AndroidSndWriteData{
 		ms_mutex_destroy(&mutex);
 		ms_bufferizer_uninit(&bf);
 	}
+	void updateStreamTypeFromMsSndCard() {
+		MSSndCardStreamType type = ms_snd_card_get_stream_type(soundCard);
+		stype = AUDIO_STREAM_VOICE_CALL;
+		if (type == MS_SND_CARD_STREAM_RING) {
+			stype = AUDIO_STREAM_RING;
+		} else if (type == MS_SND_CARD_STREAM_MEDIA) {
+			stype = AUDIO_STREAM_MUSIC;
+		} else if (type == MS_SND_CARD_STREAM_DTMF) {
+			stype = AUDIO_STREAM_DTMF;
+		}
+	}
 	void setCard(AndroidNativeSndCardData *card){
 		mCard=card;
 #ifdef NATIVE_USE_HARDWARE_RATE
 		rate=card->mPlayRate;
 #endif
 	}
+	MSSndCard *soundCard;
 	AndroidNativeSndCardData *mCard;
 	audio_stream_type_t stype;
 	int rate;
@@ -214,22 +223,11 @@ static MSFilter *android_snd_card_create_reader(MSSndCard *card){
 static MSFilter *android_snd_card_create_writer(MSSndCard *card){
 	MSFilter *f=ms_android_snd_write_new(ms_snd_card_get_factory(card));
 	(static_cast<AndroidSndWriteData*>(f->data))->setCard((AndroidNativeSndCardData*)card->data);
+	(static_cast<AndroidSndWriteData*>(f->data))->soundCard = card;
 	return f;
 }
 
-static int get_sdk_version(){
-	static int sdk_version = 0;
-	if (sdk_version==0){
-		/* Get Android SDK version. */
-		JNIEnv *jni_env = ms_get_jni_env();
-		jclass version_class = jni_env->FindClass("android/os/Build$VERSION");
-		jfieldID fid = jni_env->GetStaticFieldID(version_class, "SDK_INT", "I");
-		sdk_version = jni_env->GetStaticIntField(version_class, fid);
-		ms_message("SDK version [%i] detected", sdk_version);
-		jni_env->DeleteLocalRef(version_class);
-	}
-	return sdk_version;
-}
+
 
 static void android_snd_card_detect(MSSndCardManager *m) {
 	bool audio_record_loaded = false;
@@ -241,13 +239,13 @@ static void android_snd_card_detect(MSSndCardManager *m) {
 	SoundDeviceDescription *d = NULL;
 	MSDevicesInfo *devices = NULL;
 
-	if (get_sdk_version() > 19) {
+	if (ms2_android_get_sdk_version() > 19) {
 		/*it is actually working well on android 5 on Nexus 4 but crashes on Samsung S5, due to, maybe
 		 * calling convention of C++ method being different. Arguments received by AudioTrack constructor do not match the arguments
 		 * sent by the caller (TransferType maps to uid argument!).
 		 * Until we find a rational explanation to this, the native module is disabled on Android 5.
 		**/
-		ms_message("Native android sound support not tested on SDK [%i], disabled.",get_sdk_version());
+		ms_message("Native android sound support not tested on SDK [%i], disabled.",ms2_android_get_sdk_version());
 		return;
 	}
 	
@@ -350,11 +348,11 @@ static void android_snd_read_cb(int event, void* user, void *p_info){
 		 *  1) understand why AudioRecord thread doesn't detach.
 		 *  2) disable logs just for this thread (using a TLS)
 		 */
-		int loglevel=ortp_get_log_level_mask(ORTP_LOG_DOMAIN);
-		ortp_set_log_level_mask(NULL, ORTP_ERROR|ORTP_FATAL);
+		int loglevel=bctbx_get_log_level_mask(BCTBX_LOG_DOMAIN);
+		bctbx_set_log_level_mask(NULL, BCTBX_LOG_ERROR|BCTBX_LOG_FATAL);
 		ad->mTickerSynchronizer = ms_ticker_synchronizer_new();
 		ms_ticker_set_synchronizer(obj->ticker, ad->mTickerSynchronizer);
-		ortp_set_log_level_mask(ORTP_LOG_DOMAIN, loglevel);
+		bctbx_set_log_level_mask(BCTBX_LOG_DOMAIN, loglevel);
 	}
 	if (event==AudioRecord::EVENT_MORE_DATA){
 		AudioRecord::Buffer info;
@@ -387,7 +385,7 @@ static void android_snd_read_activate_hardware_aec(MSFilter *obj){
 		return;
 	}
 	
-	ad->aec = enable_hardware_echo_canceller(env, sessionId);
+	ad->aec = ms_android_enable_hardware_echo_canceller(env, sessionId);
 }
 
 static void android_snd_read_preprocess(MSFilter *obj){
@@ -436,7 +434,7 @@ static void android_snd_read_postprocess(MSFilter *obj){
 		ad->rec->stop();
 		if (ad->aec){
 			JNIEnv *env=ms_get_jni_env();
-			delete_hardware_echo_canceller(env, ad->aec);
+			ms_android_delete_hardware_echo_canceller(env, ad->aec);
 			ad->aec = NULL;
 		}
 		ad->rec=0;
@@ -471,8 +469,6 @@ static void android_snd_read_process(MSFilter *obj){
 		//ms_message("android_snd_read_process: Outputing %i bytes",msgdsize(om));
 		ms_queue_put(obj->outputs[0],om);
 		ad->nbufs++;
-		if (ad->nbufs % 100 == 0)
-			ms_message("sound/wall clock skew is average=%g ms", ad->av_skew);
 	}
 	ms_mutex_unlock(&ad->mutex);
 }
@@ -689,7 +685,7 @@ static void android_snd_write_cb(int event, void *user, void * p_info){
 static int channel_mask_for_audio_track(int nchannels) {
 	int channel_mask;
 	channel_mask = audio_channel_out_mask_from_count(nchannels);
-	if (get_sdk_version() < 14) {
+	if (ms2_android_get_sdk_version() < 14) {
 		ms_message("Android version older than ICS, apply audio channel hack for AudioTrack");
 		if ((channel_mask & AUDIO_CHANNEL_OUT_MONO) == AUDIO_CHANNEL_OUT_MONO) {
 			channel_mask = 0x4;
@@ -709,6 +705,7 @@ static void android_snd_write_preprocess(MSFilter *obj){
 	ad->mCard->enableVoipMode();
 	ad->nFramesRequested=0;
 	
+	ad->updateStreamTypeFromMsSndCard();
 	if (AudioTrack::getMinFrameCount(&play_buf_size,ad->stype,ad->rate)==0){
 		ms_message("AudioTrack: min frame count is %i",play_buf_size);
 	}else{
