@@ -1,22 +1,20 @@
 /*
- * msandroid.cpp -Android Media plugin for Linphone-
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
+ * This file is part of mediastreamer2.
  *
- * Copyright (C) 2009  Belledonne Communications, Grenoble, France
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mediastreamer2/mssndcard.h"
@@ -24,6 +22,7 @@
 #include "mediastreamer2/msticker.h"
 #include "mediastreamer2/msjava.h"
 #include "mediastreamer2/devices.h"
+#include "mediastreamer2/android_utils.h"
 #include <jni.h>
 
 #include <sys/time.h>
@@ -32,7 +31,6 @@
 #include "mediastreamer2/zrtp.h"
 #include <cpu-features.h>
 
-#include "hardware_echo_canceller.h"
 
 #define USE_HARDWARE_RATE 1
 
@@ -326,7 +324,6 @@ static void* msandroid_read_cb(msandroid_sound_read_data* d) {
 		goto end;
 	}
 	//start recording
-	ms_message("Start recording");
 	jni_env->CallVoidMethod(d->audio_record,record_id);
 
 	// int read (byte[] audioData, int offsetInBytes, int sizeInBytes)
@@ -474,7 +471,7 @@ static void sound_read_preprocess(MSFilter *f){
 		if (sessionId==-1) {
 			return;
 		}
-		d->aec = enable_hardware_echo_canceller(env, sessionId);
+		d->aec = ms_android_enable_hardware_echo_canceller(env, sessionId);
 	}
 }
 
@@ -486,7 +483,6 @@ static void sound_read_postprocess(MSFilter *f){
 	JNIEnv *jni_env = ms_get_jni_env();
 
 	ms_ticker_set_synchronizer(f->ticker, NULL);
-	d->read_samples=0;
 
 	//stop recording
 	stop_id = jni_env->GetMethodID(d->audio_record_class,"stop", "()V");
@@ -513,11 +509,16 @@ static void sound_read_postprocess(MSFilter *f){
 		jni_env->CallVoidMethod(d->audio_record,release_id);
 	}
 	if (d->aec) {
-		delete_hardware_echo_canceller(jni_env, d->aec);
+		ms_android_delete_hardware_echo_canceller(jni_env, d->aec);
 		d->aec = NULL;
 	}
 	goto end;
 	end: {
+		if (d->ticker_synchronizer) {
+			ms_ticker_synchronizer_destroy(d->ticker_synchronizer);
+			d->ticker_synchronizer = NULL;
+		}
+		d->read_samples=0;
 		if (d->audio_record) jni_env->DeleteGlobalRef(d->audio_record);
 		jni_env->DeleteGlobalRef(d->audio_record_class);
 		if (d->read_buff) jni_env->DeleteGlobalRef(d->read_buff);
