@@ -1,36 +1,27 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-This is the video capture filter for Android.
-It uses one of the JNI wrappers to access Android video capture API.
-See:
-	org.linphone.mediastream.video.capture.AndroidVideoApi9JniWrapper
-	org.linphone.mediastream.video.capture.AndroidVideoApi8JniWrapper
-	org.linphone.mediastream.video.capture.AndroidVideoApi5JniWrapper
-
- * Copyright (C) 2010  Belledonne Communications, Grenoble, France
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-extern "C" {
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/mswebcam.h"
 #include "mediastreamer2/msjava.h"
 #include "mediastreamer2/msticker.h"
-}
 
 #include <jni.h>
 #include <math.h>
@@ -54,12 +45,13 @@ struct AndroidWebcamConfig {
 
 struct AndroidReaderContext {
 	AndroidReaderContext(MSFilter *f, MSWebCam *cam):filter(f), webcam(cam),frame(0),fps(5){
-		ms_message("Creating AndroidReaderContext for Android VIDEO capture filter");
+		ms_message("[Legacy Capture] Creating AndroidReaderContext for Android VIDEO capture filter");
 		ms_mutex_init(&mutex,NULL);
 		androidCamera = 0;
 		previewWindow = 0;
 		rotation = rotationSavedDuringVSize = UNDEFINED_ROTATION;
 		allocator = ms_yuv_buf_allocator_new();
+		snprintf(fps_context, sizeof(fps_context), "Captured mean fps=%%f");
 	};
 
 	~AndroidReaderContext(){
@@ -101,6 +93,9 @@ static AndroidReaderContext *getContext(MSFilter *f);
 static int video_capture_set_fps(MSFilter *f, void *arg){
 	AndroidReaderContext* d = (AndroidReaderContext*) f->data;
 	d->fps=*((float*)arg);
+	snprintf(d->fps_context, sizeof(d->fps_context), "Captured mean fps=%%f, expected=%f", d->fps);
+	ms_video_init_framerate_controller(&d->fpsControl, d->fps);
+	ms_video_init_average_fps(&d->averageFps, d->fps_context);
 	return 0;
 }
 
@@ -109,7 +104,7 @@ static int video_capture_set_autofocus(MSFilter *f, void* data){
 	AndroidReaderContext* d = (AndroidReaderContext*) f->data;
 	jmethodID method = env->GetStaticMethodID(d->helperClass,"activateAutoFocus", "(Ljava/lang/Object;)V");
 	env->CallStaticObjectMethod(d->helperClass, method, d->androidCamera);
-	
+
 	return 0;
 }
 
@@ -141,17 +136,17 @@ static int video_capture_set_vsize(MSFilter *f, void* data){
 
 	if (!resArray) {
 		ms_mutex_unlock(&d->mutex);
-		ms_error("Failed to retrieve camera '%d' supported resolutions\n", ((AndroidWebcamConfig*)d->webcam->data)->id);
+		ms_error("[Legacy Capture] Failed to retrieve camera '%d' supported resolutions\n", ((AndroidWebcamConfig*)d->webcam->data)->id);
 		return -1;
 	}
 
 	// handle result :
 	//   - 0 : width
-    //   - 1 : height
-    //   - 2 : useDownscaling
+	//   - 1 : height
+	//   - 2 : useDownscaling
 	jint res[3];
 	env->GetIntArrayRegion((jintArray)resArray, 0, 3, res);
-	ms_message("Camera selected resolution is: %dx%d (requested: %dx%d) with downscaling?%d\n", res[0], res[1], d->requestedSize.width, d->requestedSize.height, res[2]);
+	ms_message("[Legacy Capture] Camera selected resolution is: %dx%d (requested: %dx%d) with downscaling?%d\n", res[0], res[1], d->requestedSize.width, d->requestedSize.height, res[2]);
 	d->hwCapableSize.width =  res[0];
 	d->hwCapableSize.height = res[1];
 	d->useDownscaling = res[2];
@@ -162,25 +157,32 @@ static int video_capture_set_vsize(MSFilter *f, void* data){
 
 	// if hw supplies a smaller resolution, modify requested size accordingly
 	if ((hwSize * downscale * downscale) < rqSize) {
-		ms_message("Camera cannot produce requested resolution %dx%d, will supply smaller one: %dx%d\n",
+		ms_message("[Legacy Capture] Camera cannot produce requested resolution %dx%d, will supply smaller one: %dx%d\n",
 			d->requestedSize.width, d->requestedSize.height, (int) (res[0] * downscale), (int) (res[1]*downscale));
 		d->usedSize.width = (int) (d->hwCapableSize.width * downscale);
 		d->usedSize.height = (int) (d->hwCapableSize.height * downscale);
 	} else if ((hwSize * downscale * downscale) > rqSize) {
-		ms_message("Camera cannot produce requested resolution %dx%d, will capture a bigger one (%dx%d) and crop it to match encoder requested resolution\n",
-			d->requestedSize.width, d->requestedSize.height, (int)(res[0] * downscale), (int)(res[1] * downscale));
-		d->usedSize.width = d->requestedSize.width;
-		d->usedSize.height = d->requestedSize.height;
+		if (d->requestedSize.width > d->hwCapableSize.width || d->requestedSize.height > d->hwCapableSize.height) {
+			ms_message("[Legacy Capture] Camera cannot produce requested resolution %dx%d, will capture a bigger one (%dx%d)\n",
+				d->requestedSize.width, d->requestedSize.height, (int)(res[0] * downscale), (int)(res[1] * downscale));
+			d->usedSize.width = d->requestedSize.width = d->hwCapableSize.width;
+			d->usedSize.height = d->requestedSize.height = d->hwCapableSize.height;
+		} else {
+			ms_message("[Legacy Capture] Camera cannot produce requested resolution %dx%d, will capture a bigger one (%dx%d) and crop it to match encoder requested resolution\n",
+				d->requestedSize.width, d->requestedSize.height, (int)(res[0] * downscale), (int)(res[1] * downscale));
+			d->usedSize.width = d->requestedSize.width;
+			d->usedSize.height = d->requestedSize.height;
+		}
 	} else {
 		d->usedSize.width = d->requestedSize.width;
 		d->usedSize.height = d->requestedSize.height;
 	}
-	
+
 	// is phone held |_ to cam orientation ?
 	if (d->rotation == UNDEFINED_ROTATION || compute_image_rotation_correction(d, d->rotation) % 180 != 0) {
 		if (d->rotation == UNDEFINED_ROTATION) {
-			ms_error("To produce a correct image, Mediastreamer MUST be aware of device's orientation BEFORE calling 'configure_video_source'\n"); 
-			ms_warning("Capture filter do not know yet about device's orientation.\n"
+			ms_error("[Legacy Capture] To produce a correct image, Mediastreamer MUST be aware of device's orientation BEFORE calling 'configure_video_source'\n");
+			ms_warning("[Legacy Capture] Capture filter do not know yet about device's orientation.\n"
 				"Current assumption: device is held perpendicular to its webcam (ie: portrait mode for a phone)\n");
 			d->rotationSavedDuringVSize = 0;
 		} else {
@@ -194,12 +196,16 @@ static int video_capture_set_vsize(MSFilter *f, void* data){
 			int t = d->usedSize.width;
 			d->usedSize.width = d->usedSize.height;
 			d->usedSize.height = t;
-			ms_message("Swapped resolution width and height to : %dx%d\n", d->usedSize.width, d->usedSize.height);
+			ms_message("[Legacy Capture] Swapped resolution width and height to : %dx%d\n", d->usedSize.width, d->usedSize.height);
 		}
 	} else {
 		d->rotationSavedDuringVSize = d->rotation;
 	}
 
+	if (d->usedSize.width != 0 && d->usedSize.height != 0) {
+		ms_filter_notify(f, MS_CAMERA_PREVIEW_SIZE_CHANGED, &d->usedSize);
+	}
+
 	ms_mutex_unlock(&d->mutex);
 	return 0;
 }
@@ -218,7 +224,7 @@ static int video_capture_get_pix_fmt(MSFilter *f, void *data){
 // Java will give us a pointer to capture preview surface.
 static int video_set_native_preview_window(MSFilter *f, void *arg) {
 	AndroidReaderContext* d = (AndroidReaderContext*) f->data;
-	
+
 	ms_mutex_lock(&d->mutex);
 
 	jobject w = (jobject)*((unsigned long*)arg);
@@ -234,9 +240,9 @@ static int video_set_native_preview_window(MSFilter *f, void *arg) {
 
 	if (d->androidCamera) {
 		if (d->previewWindow == 0) {
-			ms_message("Preview capture window set for the 1st time (win: %p rotation:%d)\n", w, d->rotation);
+			ms_message("[Legacy Capture] Preview capture window set for the 1st time (win: %p rotation:%d)\n", w, d->rotation);
 		} else {
-			ms_message("Preview capture window changed (oldwin: %p newwin: %p rotation:%d)\n", d->previewWindow, w, d->rotation);
+			ms_message("[Legacy Capture] Preview capture window changed (oldwin: %p newwin: %p rotation:%d)\n", d->previewWindow, w, d->rotation);
 
 			env->CallStaticVoidMethod(d->helperClass,
 						env->GetStaticMethodID(d->helperClass,"stopRecording", "(Ljava/lang/Object;)V"),
@@ -248,15 +254,19 @@ static int video_set_native_preview_window(MSFilter *f, void *arg) {
 						((AndroidWebcamConfig*)d->webcam->data)->id,
 						d->hwCapableSize.width,
 						d->hwCapableSize.height,
-						(jint)d->fps,
+						(jint)30,
 						(d->rotation != UNDEFINED_ROTATION) ? d->rotation:0,
 						(jlong)d));
 		}
 		// if previewWindow AND camera are valid => set preview window
 		if (w && d->androidCamera)
 			env->CallStaticVoidMethod(d->helperClass, method, d->androidCamera, w);
+
+		if (d->usedSize.width != 0 && d->usedSize.height != 0) {
+			ms_filter_notify(f, MS_CAMERA_PREVIEW_SIZE_CHANGED, &d->usedSize);
+		}
 	} else {
-		ms_message("Preview capture window set but camera not created yet; remembering it for later use\n");
+		ms_message("[Legacy Capture] Preview capture window set but camera not created yet; remembering it for later use\n");
 	}
 	d->previewWindow = w;
 
@@ -278,12 +288,11 @@ static int video_set_device_rotation(MSFilter* f, void* arg) {
 }
 
 void video_capture_preprocess(MSFilter *f){
-	ms_message("Preprocessing of Android VIDEO capture filter");
+	ms_message("[Legacy Capture] Preprocessing of Android VIDEO capture filter");
 
 	AndroidReaderContext *d = getContext(f);
 	ms_mutex_lock(&d->mutex);
 
-	snprintf(d->fps_context, sizeof(d->fps_context), "Captured mean fps=%%f, expected=%f", d->fps);
 	ms_video_init_framerate_controller(&d->fpsControl, d->fps);
 	ms_video_init_average_fps(&d->averageFps, d->fps_context);
 
@@ -291,12 +300,12 @@ void video_capture_preprocess(MSFilter *f){
 
 	jmethodID method = env->GetStaticMethodID(d->helperClass,"startRecording", "(IIIIIJ)Ljava/lang/Object;");
 
-	ms_message("Starting Android camera '%d' (rotation:%d)", ((AndroidWebcamConfig*)d->webcam->data)->id, d->rotation);
+	ms_message("[Legacy Capture] Starting Android camera '%d' (rotation:%d)", ((AndroidWebcamConfig*)d->webcam->data)->id, d->rotation);
 	jobject cam = env->CallStaticObjectMethod(d->helperClass, method,
 			((AndroidWebcamConfig*)d->webcam->data)->id,
 			d->hwCapableSize.width,
 			d->hwCapableSize.height,
-			(jint)d->fps,
+			(jint)30,
 			d->rotationSavedDuringVSize,
 			(jlong)d);
 	d->androidCamera = env->NewGlobalRef(cam);
@@ -305,7 +314,12 @@ void video_capture_preprocess(MSFilter *f){
 		method = env->GetStaticMethodID(d->helperClass,"setPreviewDisplaySurface", "(Ljava/lang/Object;Ljava/lang/Object;)V");
 		env->CallStaticVoidMethod(d->helperClass, method, d->androidCamera, d->previewWindow);
 	}
-	ms_message("Preprocessing of Android VIDEO capture filter done");
+	ms_message("[Legacy Capture] Preprocessing of Android VIDEO capture filter done");
+
+	if (d->frame) {
+		freemsg(d->frame);
+		d->frame = NULL;
+	}
 	ms_mutex_unlock(&d->mutex);
 }
 
@@ -321,6 +335,7 @@ static void video_capture_process(MSFilter *f){
 	}
 
 	ms_video_update_average_fps(&d->averageFps, f->ticker->time);
+	mblk_set_timestamp_info(d->frame, f->ticker->time * 90);
 
 	ms_queue_put(f->outputs[0],d->frame);
 	d->frame = 0;
@@ -328,12 +343,12 @@ static void video_capture_process(MSFilter *f){
 }
 
 static void video_capture_postprocess(MSFilter *f){
-	ms_message("Postprocessing of Android VIDEO capture filter");
+	ms_message("[Legacy Capture] Postprocessing of Android VIDEO capture filter");
 	AndroidReaderContext* d = getContext(f);
 	JNIEnv *env = ms_get_jni_env();
-	
+
 	ms_mutex_lock(&d->mutex);
-	
+
 	if (d->androidCamera) {
 		jmethodID method = env->GetStaticMethodID(d->helperClass,"stopRecording", "(Ljava/lang/Object;)V");
 
@@ -342,23 +357,23 @@ static void video_capture_postprocess(MSFilter *f){
 	}
 	d->androidCamera = 0;
 	d->previewWindow = 0;
-	if (d->frame){
+	if (d->frame) {
 		freemsg(d->frame);
-		d->frame=NULL;
+		d->frame = NULL;
 	}
 	ms_mutex_unlock(&d->mutex);
 }
 
 static void video_capture_init(MSFilter *f) {
 	AndroidReaderContext* d = new AndroidReaderContext(f, 0);
-	ms_message("Init of Android VIDEO capture filter (%p)", d);
+	ms_message("[Legacy Capture] Init of Android VIDEO capture filter (%p)", d);
 	JNIEnv *env = ms_get_jni_env();
 	d->helperClass = getHelperClassGlobalRef(env);
 	f->data = d;
 }
 
 static void video_capture_uninit(MSFilter *f) {
-	ms_message("Uninit of Android VIDEO capture filter");
+	ms_message("[Legacy Capture] Uninit of Android VIDEO capture filter");
 	AndroidReaderContext* d = getContext(f);
 	JNIEnv *env = ms_get_jni_env();
 	env->DeleteGlobalRef(d->helperClass);
@@ -399,15 +414,15 @@ MS_FILTER_DESC_EXPORT(ms_video_capture_desc)
 /* Webcam methods */
 static void video_capture_detect(MSWebCamManager *obj);
 static void video_capture_cam_init(MSWebCam *cam){
-	ms_message("Android VIDEO capture filter cam init");
+	ms_message("[Legacy Capture] Android VIDEO capture filter cam init");
 }
 
 static MSFilter *video_capture_create_reader(MSWebCam *obj){
-	ms_message("Instanciating Android VIDEO capture MS filter");
+	ms_message("[Legacy Capture] Instanciating Android VIDEO capture MS filter");
 
 	MSFilter* lFilter = ms_factory_create_filter_from_desc(ms_web_cam_get_factory(obj), &ms_video_capture_desc);
 	getContext(lFilter)->webcam = obj;
-	
+
 	return lFilter;
 }
 
@@ -419,24 +434,33 @@ MSWebCamDesc ms_android_video_capture_desc={
 		NULL
 };
 
-static void video_capture_detect(MSWebCamManager *obj){
-	ms_message("Detecting Android VIDEO cards");
+void android_video_capture_detect_cameras_legacy(MSWebCamManager *obj) {
+	ms_message("[Legacy Capture] Detecting Android VIDEO cards");
+
 	JNIEnv *env = ms_get_jni_env();
 	jclass helperClass = getHelperClassGlobalRef(env);
-	
-	if (helperClass==NULL) return;
 
-	// create 3 int arrays - assuming 2 webcams at most
-	jintArray indexes = (jintArray)env->NewIntArray(2);
-	jintArray frontFacing = (jintArray)env->NewIntArray(2);
-	jintArray orientation = (jintArray)env->NewIntArray(2);
+	if (helperClass == NULL) return;
+
+	jmethodID countMethod = env->GetStaticMethodID(helperClass,"detectCamerasCount", "()I");
+	int count = env->CallStaticIntMethod(helperClass, countMethod);
+	ms_message("[Legacy Capture] %d cards detected", count);
+
+	jintArray indexes = (jintArray)env->NewIntArray(count);
+	jintArray frontFacing = (jintArray)env->NewIntArray(count);
+	jintArray orientation = (jintArray)env->NewIntArray(count);
 
 	jmethodID method = env->GetStaticMethodID(helperClass,"detectCameras", "([I[I[I)I");
+	env->CallStaticIntMethod(helperClass, method, indexes, frontFacing, orientation);
 
-	int count = env->CallStaticIntMethod(helperClass, method, indexes, frontFacing, orientation);
+	bool frontFacingFound = false;
+	bool backFacingFound = false;
+	for (int i = 0; i < count; i++) {
+		if (frontFacingFound && backFacingFound) {
+			ms_message("[Legacy Capture] Found one front facing camera and one back facing camera, no need for more.");
+			break;
+		}
 
-	ms_message("%d cards detected", count);
-	for(int i=0; i<count; i++) {
 		MSWebCam *cam = ms_web_cam_new(&ms_android_video_capture_desc);
 		AndroidWebcamConfig* c = new AndroidWebcamConfig();
 		env->GetIntArrayRegion(indexes, i, 1, &c->id);
@@ -444,21 +468,43 @@ static void video_capture_detect(MSWebCamManager *obj){
 		env->GetIntArrayRegion(orientation, i, 1, &c->orientation);
 		cam->data = c;
 		cam->name = ms_strdup("Android video name");
-		char* idstring = (char*) ms_malloc(15);
-		snprintf(idstring, 15, "Android%d", c->id);
-		cam->id = idstring;
-		ms_web_cam_manager_add_cam(obj,cam);
-		ms_message("camera created: id=%d frontFacing=%d orientation=%d [msid:%s]\n", c->id, c->frontFacing, c->orientation, idstring);
+
+		const char *facing = "Back";
+		if (c->frontFacing) {
+			facing = "Front";
+			frontFacingFound = true;
+		} else {
+			backFacingFound = true;
+		}
+
+		char *idstring = bctbx_strdup_printf("%sFacingCamera%d", facing, c->id);
+		cam->id = ms_strdup(idstring);
+		bctbx_free(idstring);
+
+		if (ms_get_android_sdk_version() >= 26) {
+			ms_warning("[Legacy Capture] Compatibility mode detected, prepending camera to prevent NoWebcam to be the first one");
+			ms_web_cam_manager_prepend_cam(obj, cam); // Otherwise NoWebcam will be the first one...
+		} else {
+			ms_web_cam_manager_add_cam(obj, cam);
+		}
+		ms_message("[Legacy Capture] camera created: id=%d frontFacing=%d orientation=%d [msid:%s]\n", c->id, c->frontFacing, c->orientation, cam->id);
 	}
 	env->DeleteLocalRef(indexes);
 	env->DeleteLocalRef(frontFacing);
 	env->DeleteLocalRef(orientation);
 
 	env->DeleteGlobalRef(helperClass);
-	ms_message("Detection of Android VIDEO cards done");
+	ms_message("[Legacy Capture] Detection of Android VIDEO cards done");
 }
 
+static void video_capture_detect(MSWebCamManager *obj) {
+	if (ms_get_android_sdk_version() >= 26) {
+		ms_warning("[Legacy Capture] Android >= 8.0 detected, disabling legacy capture filter");
+		return;
+	}
 
+	android_video_capture_detect_cameras_legacy(obj);
+}
 
 /************************ JNI methods                  ************************/
 #ifdef __cplusplus
@@ -468,9 +514,9 @@ extern "C" {
 JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoApi5JniWrapper_putImage(JNIEnv*  env,
 		jclass  thiz,jlong nativePtr,jbyteArray frame) {
 	AndroidReaderContext* d = (AndroidReaderContext*) nativePtr;
-	
+
 	ms_mutex_lock(&d->mutex);
-	
+
 	if (!d->androidCamera){
 		ms_mutex_unlock(&d->mutex);
 		return;
@@ -482,28 +528,35 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoA
 	}
 
 	if (d->rotation != UNDEFINED_ROTATION && d->rotationSavedDuringVSize != d->rotation) {
-		ms_warning("Rotation has changed (new value: %d) since vsize was run (old value: %d)."
+		ms_warning("[Legacy Capture] Rotation has changed (new value: %d) since vsize was run (old value: %d)."
 					"Will produce inverted images. Use set_device_orientation() then update call.\n",
 			d->rotation, d->rotationSavedDuringVSize);
 	}
 
 	int image_rotation_correction = compute_image_rotation_correction(d, d->rotationSavedDuringVSize);
 
+	MSVideoSize used_size = d->usedSize;
+	MSVideoSize capable_size = d->hwCapableSize;
+	MSVideoSize requested_size = d->requestedSize;
+	int use_downscale = d->useDownscaling;
+
+	ms_mutex_unlock(&d->mutex);
+
 	jboolean isCopied;
 	jbyte* jinternal_buff = env->GetByteArrayElements(frame, &isCopied);
 	if (isCopied) {
-		ms_warning("The video frame received from Java has been copied");
+		ms_warning("[Legacy Capture] The video frame received from Java has been copied");
 	}
 
 	int y_cropping_offset=0, cbcr_cropping_offset=0;
 	MSVideoSize targetSize;
-	d->useDownscaling?targetSize.width=d->requestedSize.width*2:targetSize.width=d->requestedSize.width;
-	d->useDownscaling?targetSize.height=d->requestedSize.height*2:targetSize.height=d->requestedSize.height;
+	use_downscale?targetSize.width=requested_size.width*2:targetSize.width=requested_size.width;
+	use_downscale?targetSize.height=requested_size.height*2:targetSize.height=requested_size.height;
 
-	compute_cropping_offsets(d->hwCapableSize, targetSize, &y_cropping_offset, &cbcr_cropping_offset);
+	compute_cropping_offsets(capable_size, targetSize, &y_cropping_offset, &cbcr_cropping_offset);
 
-	int width = d->hwCapableSize.width;
-	int height = d->hwCapableSize.height;
+	int width = capable_size.width;
+	int height = capable_size.height;
 
 	uint8_t* y_src = (uint8_t*)(jinternal_buff + y_cropping_offset);
 	uint8_t* cbcr_src = (uint8_t*) (jinternal_buff + width * height + cbcr_cropping_offset);
@@ -517,17 +570,21 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_video_capture_AndroidVideoA
  	mblk_t* yuv_block = copy_ycbcrbiplanar_to_true_yuv_with_rotation_and_down_scale_by_2(d->allocator, y_src
 														, cbcr_src
 														, image_rotation_correction
-														, d->usedSize.width
-														, d->usedSize.height
-														, d->hwCapableSize.width
-														, d->hwCapableSize.width
+														, used_size.width
+														, used_size.height
+														, capable_size.width
+														, capable_size.width
 														, false
-														, d->useDownscaling);
+														, use_downscale);
+
+	ms_mutex_lock(&d->mutex);
+
 	if (yuv_block) {
 		if (d->frame)
 			freemsg(d->frame);
 		d->frame = yuv_block;
 	}
+
 	ms_mutex_unlock(&d->mutex);
 
 	// JNI_ABORT free the buffer without copying back the possible changes
@@ -543,10 +600,10 @@ static int compute_image_rotation_correction(AndroidReaderContext* d, int rotati
 
 	int result;
 	if (conf->frontFacing) {
-		ms_debug("%s: %d + %d\n", __FUNCTION__, ((AndroidWebcamConfig*)d->webcam->data)->orientation, rotation);
+		ms_debug("[Legacy Capture] %s: %d + %d\n", __FUNCTION__, ((AndroidWebcamConfig*)d->webcam->data)->orientation, rotation);
 	 	result = ((AndroidWebcamConfig*)d->webcam->data)->orientation + rotation;
 	} else {
-		ms_debug("%s: %d - %d\n", __FUNCTION__, ((AndroidWebcamConfig*)d->webcam->data)->orientation, rotation);
+		ms_debug("[Legacy Capture] %s: %d - %d\n", __FUNCTION__, ((AndroidWebcamConfig*)d->webcam->data)->orientation, rotation);
 	 	result = ((AndroidWebcamConfig*)d->webcam->data)->orientation - rotation;
 	}
 	while(result < 0)
@@ -572,15 +629,15 @@ static void compute_cropping_offsets(MSVideoSize hwSize, MSVideoSize outputSize,
 
 
 static jclass getHelperClassGlobalRef(JNIEnv *env) {
-	ms_message("getHelperClassGlobalRef (env: %p)", env);
+	ms_message("[Legacy Capture] getHelperClassGlobalRef (env: %p)", env);
 	const char* className;
 	// FindClass only returns local references.
-	
+
 	// Find the current Android SDK version
 	jclass version = env->FindClass(VersionPath);
 	jmethodID method = env->GetStaticMethodID(version,"sdk", "()I");
 	android_sdk_version = env->CallStaticIntMethod(version, method);
-	ms_message("Android SDK version found is %i", android_sdk_version);
+	ms_message("[Legacy Capture] Android SDK version found is %i", android_sdk_version);
 	env->DeleteLocalRef(version);
 
 	if (android_sdk_version >= 9) {
@@ -592,7 +649,7 @@ static jclass getHelperClassGlobalRef(JNIEnv *env) {
 	}
 	jclass c = env->FindClass(className);
 	if (c == 0) {
-		ms_error("Could not load class '%s' (%d)", className, android_sdk_version);
+		ms_error("[Legacy Capture] Could not load class '%s' (%d)", className, android_sdk_version);
 		return NULL;
 	} else {
 		jclass globalRef = reinterpret_cast<jclass>(env->NewGlobalRef(c));
