@@ -1,21 +1,20 @@
 /*
- * devices.c - sound device information table (latency, presence of AEC etc...)
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
- * Copyright (C) 2009-2012  Belledonne Communications, Grenoble, France
+ * This file is part of mediastreamer2.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -30,6 +29,23 @@
 
 
 #ifdef __ANDROID__
+
+int ms2_android_get_sdk_version(void){
+	static int sdk_version = 0;
+	if (sdk_version==0){
+		/* Get Android SDK version. */
+		JNIEnv *jni_env = ms_get_jni_env();
+		jclass version_class = (*jni_env)->FindClass(jni_env, "android/os/Build$VERSION");
+		jfieldID fid = (*jni_env)->GetStaticFieldID(jni_env, version_class, "SDK_INT", "I");
+		sdk_version = (*jni_env)->GetStaticIntField(jni_env, version_class, fid);
+		ms_message("SDK version [%i] detected", sdk_version);
+		(*jni_env)->DeleteLocalRef(jni_env, version_class);
+	}
+	return sdk_version;
+}
+
+
+
 /*
  * 1st column: list of triplet frequency, gain, width
  * 2nd column: mic gain in db
@@ -67,6 +83,7 @@ static SoundDeviceDescription devices[]={
 	{	"motorola",				"XT907",				"",				0,	500 },
 	{	"motorola",				"DROIX X2",				"",				0,	320 },
 	{	"motorola",				"MotoG3",				"msm8916",		DEVICE_HAS_BUILTIN_AEC_CRAPPY,	100 }, /*The MotoG3 audio capture hangs for several seconds when switching to speaker mode*/
+	{	"motorola",				"Moto G (4)",				"msm8952",			0, 0, 44100 }, /*the soundcard doesn't work well at 48khz, it apparently output 44100 ! */
 	{	"motorola",				"Nexus 6",				"msm8084",		DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0}, /* Nexus 6*/
 	{	"samsung",				"GT-S5360",				"bcm21553",		0,	250 }, /*<Galaxy Y*/
 	{	"samsung",				"GT-S5360L",			"",				0,	250 }, /*<Galaxy Y*/
@@ -172,6 +189,7 @@ static SoundDeviceDescription devices[]={
 	{	"LENOVO",				"Lenovo A6000",			"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0},
 	{	"LENOVO",				"Lenovo A616",			"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0},
 	{	"LENOVO",				"Lenovo YT3-850F",		"msm8909",			DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0},
+	{	"LENOVO", 				"Lenovo TB-X704F",		"msm8953",			DEVICE_MCH265_LIMIT_DEQUEUE_OF_OUTPUT_BUFFERS, 0},
 
 	{	"Enspert",				"IGGY",					""		,		0,	320 ,0}, /*Wiko iggy*/
 	{	"Yota Devices Limited", "YD201",				"msm8974",		DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 48000 }, /* Yotaphone 2 */
@@ -185,6 +203,8 @@ static SoundDeviceDescription devices[]={
 
 	{	"Alcatel",				"4027D",				"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
 	{	"Alcatel",				"7045Y", 				"", 			DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
+	{	"TCL",					"6070Y",			"msm8952",	DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 16000},
+	
 
 	{	"HUAWEI",				"HUAWEI P7-L10",		"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
 	{	"HUAWEI",				"HUAWEI P7-L11",		"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
@@ -195,13 +215,21 @@ static SoundDeviceDescription devices[]={
 	{	"HUAWEI",				"HUAWEI P6-U06",		"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
 	{	"HUAWEI",				"Hol-U19",				"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
 	{	"HUAWEI",				"302HW",				"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
+	{	"HUAWEI",				"HUAWEI GRA-L09",		"hi3635",		DEVICE_MCH264ENC_NO_PIX_FMT_CONV, 0 },
+	{	"HUAWEI",				"HUAWEI VNS-L31",		"hi6250",		DEVICE_MCH264ENC_NO_PIX_FMT_CONV, 0 },
+	{	"HUAWEI",				"ALE-L21",				"hi6210sft",	DEVICE_MCH264ENC_NO_PIX_FMT_CONV, 0 },
+	{	"Huawei",				"AOSP on angler",		"msm8994",		DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 16000},
 
 	{	"WIKO", 				"HIGHWAY 4G",			"tegra",		DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
 	{	"Symphony",				"HIGHWAY 4G",			"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
-
+	{	"rockchip",				"rk3288_box",			"rk3288",	DEVICE_USE_ANDROID_CAMCORDER, 0, 16000},
+	{	"rockchip",				"X9-LX",				"rk3288",		DEVICE_USE_ANDROID_CAMCORDER, 0, 16000},
 	{	NULL, 					NULL,					NULL,			0, 	0,	0}
 };
 
+/*On android >= 8 we assume that AEC is builtin and usable with opensles api*/
+SoundDeviceDescription genericSoundDeviceDescriptorAboveAndroid8 = { "Generic", "Generic", "Generic", DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 0};
+
 SoundDeviceDescription genericSoundDeviceDescriptor={"Generic", "Generic", "Generic", 0, 250, 0, 0};
 
 JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerAndroidContext_addSoundDeviceDescription(JNIEnv* env, jobject thiz, jstring jmanufacturer, jstring jmodel, jstring jplatform, jint flags, jint delay, jint rate) {
@@ -356,7 +384,14 @@ SoundDeviceDescription* ms_devices_info_get_sound_device_description(MSDevicesIn
 	d = ms_devices_info_lookup_device(devices_info, manufacturer, model, platform);
 	if (!d) {
 		ms_message("No information available for [%s/%s/%s],", manufacturer, model, platform);
+#ifdef __ANDROID__
+		if (ms2_android_get_sdk_version() >= 26){
+			d = &genericSoundDeviceDescriptorAboveAndroid8;
+			ms_message("Using android >= 8 sound device descriptor.");
+		}else d = &genericSoundDeviceDescriptor;
+#else
 		d = &genericSoundDeviceDescriptor;
+#endif
 	} else {
 		ms_message("Found information for [%s/%s/%s] from internal table", manufacturer, model, platform);
 		exact_match = TRUE;
@@ -378,3 +413,4 @@ SoundDeviceDescription* ms_devices_info_get_sound_device_description(MSDevicesIn
 				manufacturer, model, platform, (d->flags & DEVICE_HAS_BUILTIN_AEC) ? "yes" : "no", d->delay);
 	return d;
 }
+
