@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2010-2011  Belledonne Communications (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /*
 A prior version of this file was developed by Hiroki Mori and was contributed to the project
@@ -114,7 +114,8 @@ typedef struct AURead{
 	AUCommon common;
 	queue_t rq;
 	MSTickerSynchronizer *ticker_synchronizer;
-	uint64_t read_frames;
+	uint64_t timestamp;
+	bool_t first_process;
 }AURead;
 
 typedef struct AUWrite{
@@ -407,6 +408,10 @@ static OSStatus readRenderProc(void *inRefCon,
 	rm->b_wptr+=lreadAudioBufferList.mBuffers[0].mDataByteSize;
 	
 	ms_mutex_lock(&d->common.mutex);
+	if (inTimeStamp->mFlags & kAudioTimeStampSampleTimeValid) {
+		d->timestamp = inTimeStamp->mSampleTime;
+	}
+
 	putq(&d->rq,rm);
 	ms_mutex_unlock(&d->common.mutex);
 
@@ -599,15 +604,6 @@ static void audio_unit_close(AUCommon *d){
 	d->au=NULL;
 }
 
-
-static mblk_t *au_read_get(AURead *d){
-	mblk_t *m;
-	ms_mutex_lock(&d->common.mutex);
-	m=getq(&d->rq);
-	ms_mutex_unlock(&d->common.mutex);
-	return m;
-}
-
 static void au_write_put(AUWrite *d, mblk_t *m){
 	ms_mutex_lock(&d->common.mutex);
 	ms_flow_controlled_bufferizer_put(d->buffer,m);
@@ -636,16 +632,29 @@ static void au_read_preprocess(MSFilter *f){
 	AURead *d = (AURead *) f->data;
 	ms_ticker_set_synchronizer(f->ticker, d->ticker_synchronizer);
 	audio_unit_open(&d->common,TRUE);
+	d->first_process = TRUE;
 }
 
 static void au_read_process(MSFilter *f){
 	AURead *d = (AURead *) f->data;
 	mblk_t *m;
-	while((m=au_read_get(d))!=NULL){
-		d->read_frames += (msgdsize(m) / 2) / d->common.nchannels;
-		ms_queue_put(f->outputs[0],m);
+	
+	ms_mutex_lock(&d->common.mutex);
+	if (d->first_process) {
+		/* The read queue must be flushed on first
+		 * process() call because it contains samples
+		 * that has been produced since the ticker
+		 * was attached. Do not skiped this data would
+		 * make the clock skew estimation to be wrong. */
+		d->first_process = FALSE;
+		flushq(&d->rq, 0);
+	} else {
+		while((m=getq(&d->rq))!=NULL){
+			ms_queue_put(f->outputs[0],m);
+			ms_ticker_synchronizer_update(d->ticker_synchronizer, d->timestamp, d->common.rate);
+		}
 	}
-	ms_ticker_synchronizer_update(d->ticker_synchronizer, d->read_frames, d->common.rate);
+	ms_mutex_unlock(&d->common.mutex);
 }
 
 static void au_read_postprocess(MSFilter *f){
@@ -730,11 +739,92 @@ static int get_nchannels(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int ms_macsnd_get_volume(MSFilter *f, void *arg, bool isCapture){
+    AUCommon *d = (AUCommon *) f->data;
+    float * pvolume = (float *)arg;
+    AudioObjectPropertyAddress volumeAddr =
+    {
+        kAudioHardwareServiceDeviceProperty_VirtualMasterVolume/*kAudioDevicePropertyVolumeScalar*/,
+        isCapture ? kAudioDevicePropertyScopeInput : kAudioDevicePropertyScopeOutput,
+        kAudioObjectPropertyElementMaster
+    };
+    
+    UInt32 size = sizeof(*pvolume);
+    
+    OSStatus err = AudioObjectGetPropertyData(d->dev, &volumeAddr, 0, NULL, &size, pvolume);
+    
+    if (err != noErr) {
+        ms_warning("MacSound: cannot get capture volume from #[%d]. Err = %d", d->dev, err);
+        *pvolume = 0.0f;
+        return -1;
+    }
+    
+    return 0;
+}
+
+
+static int ms_macsnd_capture_get_volume(MSFilter *f, void *arg)
+{
+    return ms_macsnd_get_volume(f, arg, true);
+}
+
+static int ms_macsnd_playback_get_volume(MSFilter *f, void *arg)
+{
+    return ms_macsnd_get_volume(f, arg, false);
+}
+
+
+static int ms_macsnd_set_volume(MSFilter *f, void *arg, bool isCapture)
+{
+    AUCommon *d = (AUCommon *) f->data;
+    float * pvolume = (float *)arg;
+    Boolean isWritable;
+    AudioObjectPropertyAddress  volumeAddr =
+    {
+        kAudioHardwareServiceDeviceProperty_VirtualMasterVolume,
+        isCapture ? kAudioDevicePropertyScopeInput : kAudioDevicePropertyScopeOutput,
+        kAudioObjectPropertyElementMaster
+    };
+    
+    OSStatus err = AudioObjectIsPropertySettable(d->dev, &volumeAddr, &isWritable);
+    
+    if ( err == noErr ) {
+	if ( isWritable ) {
+            err = AudioObjectSetPropertyData(d->dev, &volumeAddr, 0, NULL, sizeof(float), pvolume);
+	    if (err != noErr) {
+		ms_error("MacSnd: Could not set volume of device #[%d]. Err = %d", d->dev, err);
+                return -1;
+            }
+        }
+	else {
+	    ms_error("MacSnd: volume of device #[%d] is not settable.", d->dev);
+            return -2;
+        }
+    }
+    else {
+	ms_error("MacSnd: Could not set volume of device #[%d]. Err = %d (step 2)", d->dev, err);
+        return -3;
+    }
+    return 0;
+}
+
+static int ms_macsnd_capture_set_volume(MSFilter *f, void *arg)
+{
+    return ms_macsnd_set_volume(f, arg, true);
+}
+
+static int ms_macsnd_playback_set_volume(MSFilter *f, void *arg)
+{
+    return ms_macsnd_set_volume(f, arg, false);
+}
+
 static MSFilterMethod au_read_methods[]={
 	{	MS_FILTER_SET_SAMPLE_RATE	, set_rate	},
 	{	MS_FILTER_GET_SAMPLE_RATE	, get_rate },
 	{	MS_FILTER_SET_NCHANNELS		, read_set_nchannels	},
 	{	MS_FILTER_GET_NCHANNELS		, get_nchannels	},
+	{	MS_AUDIO_CAPTURE_SET_VOLUME_GAIN, ms_macsnd_capture_set_volume	},
+	{	MS_AUDIO_CAPTURE_GET_VOLUME_GAIN, ms_macsnd_capture_get_volume	},
 	{	0				, NULL		}
 };
 
@@ -743,6 +833,8 @@ static MSFilterMethod au_write_methods[]={
 	{	MS_FILTER_GET_SAMPLE_RATE	, get_rate },
 	{	MS_FILTER_SET_NCHANNELS		, write_set_nchannels	},
 	{	MS_FILTER_GET_NCHANNELS		, get_nchannels	},
+	{	MS_AUDIO_PLAYBACK_SET_VOLUME_GAIN, ms_macsnd_playback_set_volume	},
+	{	MS_AUDIO_PLAYBACK_GET_VOLUME_GAIN, ms_macsnd_playback_get_volume	},
 	{	0				, NULL		}
 };
 
