@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2010  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/mssndcard.h"
@@ -105,8 +105,10 @@ static void init_pulse_context(void){
 static void uninit_pulse_context(void){
 	the_pa_ref--;
 	if (the_pa_ref == 0){
+		pa_threaded_mainloop_lock(the_pa_loop);
 		pa_context_disconnect(the_pa_context);
 		pa_context_unref(the_pa_context);
+		pa_threaded_mainloop_unlock(the_pa_loop);
 		pa_threaded_mainloop_stop(the_pa_loop);
 		pa_threaded_mainloop_free(the_pa_loop);
 		the_pa_context = NULL;
@@ -255,7 +257,7 @@ static void pulse_card_detect(MSSndCardManager *m){
 	/* connect to pulse server */
 	init_pulse_context();
 	if(!wait_for_context_state(PA_CONTEXT_READY, PA_CONTEXT_FAILED)) {
-		ms_error("Connection to the pulseaudio server failed");
+		ms_message("Connection to the pulseaudio server failed");
 		return;
 	}
 
@@ -421,24 +423,25 @@ static bool_t stream_connect(Stream *s) {
 	attr.minreq = -1;
 	attr.prebuf = -1;
 
-	if(s->init_volume >= 0.0) {
-		pa_volume_t value = scale_to_volume(s->init_volume);
-		volume_ptr = pa_cvolume_init(&volume);
-		pa_cvolume_set(&volume, s->sampleSpec.channels, value);
-	}
-
 
 	if (the_pa_context==NULL) {
 		ms_error("No PulseAudio context");
 		return FALSE;
 	}
+	pa_threaded_mainloop_lock(the_pa_loop);
+	if(s->init_volume >= 0.0) {
+		pa_volume_t value = scale_to_volume(s->init_volume);
+		volume_ptr = pa_cvolume_init(&volume);
+		pa_cvolume_set(&volume, s->sampleSpec.channels, value);
+	}
 	s->stream=pa_stream_new(the_pa_context,"phone",&s->sampleSpec,NULL);
 	if (s->stream==NULL){
+		pa_threaded_mainloop_unlock(the_pa_loop);
 		ms_error("fails to create PulseAudio stream");
 		return FALSE;
 	}
 	pa_stream_set_state_callback(s->stream, stream_state_notify_cb, s);
-	pa_threaded_mainloop_lock(the_pa_loop);
+	
 	if(s->type == STREAM_TYPE_PLAYBACK) {
 		pa_stream_set_write_callback(s->stream, stream_write_request_cb, s);
 		pa_stream_set_overflow_callback(s->stream, stream_buffer_overflow_notification, s);
@@ -449,13 +452,15 @@ static bool_t stream_connect(Stream *s) {
 			volume_ptr,
 			NULL);
 	} else {
-		err=pa_stream_connect_record(s->stream,s->dev,&attr, PA_STREAM_ADJUST_LATENCY);
+		err=pa_stream_connect_record(s->stream,s->dev,&attr, PA_STREAM_ADJUST_LATENCY | PA_STREAM_START_CORKED);
 	}
 	pa_threaded_mainloop_unlock(the_pa_loop);
 	if(err < 0 || !stream_wait_for_state(s, PA_STREAM_READY, PA_STREAM_FAILED)) {
 		ms_error("Fails to connect pulseaudio stream. err=%d", err);
+		pa_threaded_mainloop_lock(the_pa_loop);
 		pa_stream_unref(s->stream);
 		s->stream = NULL;
+		pa_threaded_mainloop_unlock(the_pa_loop);
 		return FALSE;
 	}
 	ms_message("pulseaudio %s stream connected (%dHz, %dch)",
@@ -475,8 +480,10 @@ static void stream_disconnect(Stream *s) {
 		if(err!=0 || !stream_wait_for_state(s, PA_STREAM_TERMINATED, PA_STREAM_FAILED)) {
 			ms_error("pa_stream_disconnect() failed. err=%d", err);
 		}
+		pa_threaded_mainloop_lock(the_pa_loop);
 		pa_stream_unref(s->stream);
 		s->stream = NULL;
+		pa_threaded_mainloop_unlock(the_pa_loop);
 		s->state = PA_STREAM_UNCONNECTED;
 		s->init_volume = -1.0;
 	}
@@ -503,7 +510,9 @@ static bool_t stream_set_volume(Stream *s, double volume) {
 			return TRUE;
 		}
 	}
+	pa_threaded_mainloop_lock(the_pa_loop);
 	idx = pa_stream_get_index(s->stream);
+	pa_threaded_mainloop_unlock(the_pa_loop);
 	pa_cvolume_init(&cvolume);
 	pa_cvolume_set(&cvolume, s->sampleSpec.channels, scale_to_volume(volume));
 	pa_threaded_mainloop_lock(the_pa_loop);
@@ -542,9 +551,9 @@ static bool_t stream_get_volume(Stream *s, double *volume) {
 		ms_error("stream_get_volume(): no stream");
 		return FALSE;
 	}
+	pa_threaded_mainloop_lock(the_pa_loop);
 	idx = pa_stream_get_index(s->stream);
 	*volume = -1.0;
-	pa_threaded_mainloop_lock(the_pa_loop);
 	if(s->type == STREAM_TYPE_PLAYBACK) {
 		op = pa_context_get_sink_input_info(the_pa_context, idx, stream_get_sink_volume_cb, volume);
 	} else {
@@ -587,25 +596,35 @@ static void pulse_read_process(MSFilter *f){
 		ms_error("Record stream not connected");
 		return;
 	}
+
 	pa_threaded_mainloop_lock(the_pa_loop);
+
+	/*
+	 * This needs to be done at the first process to get rid of staled data.
+	 * Without this, the estimation of the clock skew could be wrong.
+	 */
+	if (pa_stream_is_corked(s->stream)) pa_stream_cork(s->stream, 0, NULL, NULL);
+
 	while(pa_stream_readable_size(s->stream) > 0) {
-		if(pa_stream_peek(s->stream, &buffer, &nbytes) >= 0) {
-			if(buffer != NULL) {
-				mblk_t *om = allocb(nbytes, 0);
-				memcpy(om->b_wptr, buffer, nbytes);
-				om->b_wptr += nbytes;
-				s->read_samples += ((nbytes / 2) / s->sampleSpec.channels);
-				ms_ticker_synchronizer_update(s->ticker_synchronizer, s->read_samples, (unsigned int)s->sampleSpec.rate);
-				ms_queue_put(f->outputs[0], om);
-			}
-			if(nbytes > 0) {
-				pa_stream_drop(s->stream);
-			}
-		} else {
+		if(pa_stream_peek(s->stream, &buffer, &nbytes) < 0) {
 			ms_error("pa_stream_peek() failed");
 			break;
 		}
+		if(buffer != NULL) {
+			mblk_t *om = allocb(nbytes, 0);
+			memcpy(om->b_wptr, buffer, nbytes);
+			om->b_wptr += nbytes;
+			ms_queue_put(f->outputs[0], om);
+		}
+		if(nbytes > 0) {
+			if (buffer == NULL) ms_message("%s: hole detected while reading samples. %zu bytes of data have been lost", f->desc->name, nbytes);
+			// ticker synchronization must be done here to ensure that lost samples are taken into account while computing total read samples count.
+			s->read_samples += ((nbytes / 2) / s->sampleSpec.channels);
+			ms_ticker_synchronizer_update(s->ticker_synchronizer, s->read_samples, (unsigned int)s->sampleSpec.rate);
+			pa_stream_drop(s->stream);
+		}
 	}
+
 	pa_threaded_mainloop_unlock(the_pa_loop);
 }
 
