@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2014  Belledonne Communications SARL
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "mediastreamer-config.h"
@@ -24,13 +24,16 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #   ifndef MEDIASTREAMER_VERSION
 #   define MEDIASTREAMER_VERSION "unknown"
 #   endif
-#	ifndef MS2_GIT_VERSION
-#	define MS2_GIT_VERSION "unknown"
-#	endif
+#endif
+
+#ifndef MS2_GIT_VERSION
+#define MS2_GIT_VERSION "unknown"
 #endif
 
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/mseventqueue.h"
+#include "mediastreamer2/msvideo.h"
+#include "mediastreamer2/mswebcam.h"
 #include "basedescs.h"
 
 #if !defined(_WIN32_WCE)
@@ -62,20 +65,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #ifdef __APPLE__
    #include "TargetConditionals.h"
+   #include "apple_utils.h"
 #endif
 
 #ifdef __QNX__
 #include <sys/syspage.h>
 #endif
 
-
 MS2_DEPRECATED static MSFactory *fallback_factory=NULL;
 
 static void ms_fmt_descriptor_destroy(MSFmtDescriptor *obj);
 
 #define DEFAULT_MAX_PAYLOAD_SIZE 1440
 
-int ms_factory_get_payload_max_size(MSFactory *factory){
+int ms_factory_get_payload_max_size(const MSFactory *factory) {
 	return factory->max_payload_size;
 }
 
@@ -99,7 +102,7 @@ void ms_factory_set_mtu(MSFactory *obj, int mtu){
 	}
 }
 
-int ms_factory_get_mtu(MSFactory *obj){
+int ms_factory_get_mtu(const MSFactory *obj) {
 	return obj->mtu;
 }
 
@@ -171,7 +174,7 @@ void ms_factory_init(MSFactory *obj){
 	debug_log_enabled=getenv("MEDIASTREAMER_DEBUG");
 #endif
 	if (debug_log_enabled!=NULL && (strcmp("1",debug_log_enabled)==0) ){
-		ortp_set_log_level_mask(ORTP_LOG_DOMAIN, ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+		bctbx_set_log_level(BCTBX_LOG_DOMAIN, BCTBX_LOG_MESSAGE);
 	}
 
 	ms_message("Mediastreamer2 factory " MEDIASTREAMER_VERSION " (git: " MS2_GIT_VERSION ") initialized.");
@@ -214,7 +217,7 @@ void ms_factory_init(MSFactory *obj){
 #ifdef __ANDROID__
 	ms_factory_add_platform_tag(obj, "android");
 #endif
-#ifdef TARGET_OS_IPHONE
+#if TARGET_OS_IPHONE
 	ms_factory_add_platform_tag(obj, "ios");
 #endif
 #if defined(__arm__) || defined(_M_ARM)
@@ -224,13 +227,15 @@ void ms_factory_init(MSFactory *obj){
 #endif
 #if defined(__ANDROID__) || (TARGET_OS_IPHONE == 1) || defined(__arm__) || defined(_M_ARM)
 	ms_factory_add_platform_tag(obj, "embedded");
+	obj->echo_canceller_filtername = ms_strdup("MSWebRTCAECM");
 #else
 	ms_factory_add_platform_tag(obj, "desktop");
+	obj->echo_canceller_filtername = ms_strdup("MSWebRTCAEC");
 #endif
 	tags = ms_factory_get_platform_tags_as_string(obj);
 	ms_message("ms_factory_init() done: platform_tags=%s", tags);
 	ms_free(tags);
-	obj->echo_canceller_filtername = ms_strdup("MSWebRTCAECM");
+
 	obj->image_resources_dir = bctbx_strdup_printf("%s/images", PACKAGE_DATA_DIR);
 }
 
@@ -247,22 +252,50 @@ void ms_factory_register_filter(MSFactory* factory, MSFilterDesc* desc ) {
 	if (desc->id==MS_FILTER_NOT_SET_ID){
 		ms_fatal("MSFilterId for %s not set !",desc->name);
 	}
+
+	if (ms_filter_desc_implements_interface(desc, MSFilterVideoEncoderInterface)) {
+		MSFilterMethod *methods=desc->methods;
+		int i;
+		for(i=0;methods!=NULL && methods[i].method!=NULL; i++) {
+			if (methods[i].id==MS_FILTER_GET_BITRATE || methods[i].id==MS_FILTER_SET_BITRATE
+				|| methods[i].id==MS_FILTER_GET_VIDEO_SIZE || methods[i].id==MS_FILTER_SET_VIDEO_SIZE
+				|| methods[i].id==MS_FILTER_GET_FPS || methods[i].id==MS_FILTER_SET_FPS
+				|| methods[i].id==MS_VIDEO_ENCODER_SET_CONFIGURATION_LIST
+			) {
+				ms_error("MSFilter %s is using a deprecated method (id=%i)",desc->name,methods[i].id);
+				return;
+			}
+		}
+	}
+
 	desc->flags|=MS_FILTER_IS_ENABLED; /*by default a registered filter is enabled*/
 
 	/*lastly registered encoder/decoders may replace older ones*/
 	factory->desc_list=bctbx_list_prepend(factory->desc_list,desc);
 }
 
-bool_t ms_factory_codec_supported(MSFactory* factory, const char *mime){
+bool_t ms_factory_codec_supported(MSFactory* factory, const char *mime) {
+	return ms_factory_has_encoder(factory, mime) &&	ms_factory_has_decoder(factory, mime);
+}
+
+bool_t ms_factory_has_encoder(MSFactory* factory, const char *mime) {
 	MSFilterDesc *enc = ms_factory_get_encoding_capturer(factory, mime);
-	MSFilterDesc *dec = ms_factory_get_decoding_renderer(factory, mime);
 
 	if (enc == NULL) enc = ms_factory_get_encoder(factory, mime);
-	if (dec == NULL) dec = ms_factory_get_decoder(factory, mime);
 
-	if(enc!=NULL && dec!=NULL) return TRUE;
+	if(enc!=NULL) return TRUE;
 
 	if(enc==NULL) ms_message("Could not find encoder for %s", mime);
+	return FALSE;
+}
+
+bool_t ms_factory_has_decoder(MSFactory* factory, const char *mime) {
+	MSFilterDesc *dec = ms_factory_get_decoding_renderer(factory, mime);
+
+	if (dec == NULL) dec = ms_factory_get_decoder(factory, mime);
+
+	if(dec!=NULL) return TRUE;
+
 	if(dec==NULL) ms_message("Could not find decoder for %s", mime);
 	return FALSE;
 }
@@ -440,38 +473,41 @@ void ms_factory_reset_statistics(MSFactory *obj){
 
 	for(elem=obj->stats_list;elem!=NULL;elem=elem->next){
 		MSFilterStats *stats=(MSFilterStats *)elem->data;
-		stats->elapsed=0;
-		stats->count=0;
+		ms_u_box_plot_reset(&stats->bp_elapsed);
 	}
 }
 
 static int usage_compare(const MSFilterStats *s1, const MSFilterStats *s2){
-	if (s1->elapsed==s2->elapsed) return 0;
-	if (s1->elapsed<s2->elapsed) return 1;
+	if (s1->bp_elapsed.mean ==s2->bp_elapsed.mean) return 0;
+	if (s1->bp_elapsed.mean < s2->bp_elapsed.mean) return 1;
 	return -1;
 }
 
 
-void ms_factory_log_statistics(MSFactory *obj){
-	bctbx_list_t *sorted=NULL;
+void ms_factory_log_statistics(MSFactory *obj) {
+	bctbx_list_t *sorted = NULL;
 	bctbx_list_t *elem;
-	uint64_t total=1;
-	for(elem=obj->stats_list;elem!=NULL;elem=elem->next){
-		MSFilterStats *stats=(MSFilterStats *)elem->data;
-		sorted=bctbx_list_insert_sorted(sorted,stats,(bctbx_compare_func)usage_compare);
-		total+=stats->elapsed;
+	double total = 0.0;
+	for (elem = obj->stats_list; elem != NULL; elem = elem->next) {
+		MSFilterStats *stats = (MSFilterStats *)elem->data;
+		sorted = bctbx_list_insert_sorted(sorted, stats, (bctbx_compare_func)usage_compare);
+		total += stats->bp_elapsed.mean;
 	}
-	ms_message("===========================================================");
-	ms_message("                  FILTER USAGE STATISTICS                  ");
-	ms_message("Name                Count     Time/tick (ms)      CPU Usage");
-	ms_message("-----------------------------------------------------------");
-	for(elem=sorted;elem!=NULL;elem=elem->next){
-		MSFilterStats *stats=(MSFilterStats *)elem->data;
-		double percentage=100.0*((double)stats->elapsed)/(double)total;
-		double tpt=((double)stats->elapsed*1e-6)/((double)stats->count+1.0);
-		ms_message("%-19s %-9i %-19g %-10g",stats->name,stats->count,tpt,percentage);
+	ms_message("=================================================================================");
+	ms_message("                         FILTER USAGE STATISTICS                                 ");
+	ms_message("Name                          Count           Time/tick (ms)            CPU Usage");
+	ms_message("                                        min     mean    max     sd               ");
+	ms_message("---------------------------------------------------------------------------------");
+	for (elem = sorted; elem != NULL; elem = elem->next) {
+		MSFilterStats *stats = (MSFilterStats *)elem->data;
+		double min = stats->bp_elapsed.min * 1e-6;
+		double mean = stats->bp_elapsed.mean * 1e-6;
+		double max = stats->bp_elapsed.max * 1e-6;
+		double sd = ms_u_box_plot_get_standard_deviation(&stats->bp_elapsed) * 1e-6;
+		double percentage = total == 0.0 ? 0.0 : 100.0 * stats->bp_elapsed.mean / total;
+		ms_message("%-29s %-9llu %-7.2f %-7.2f %-7.2f %-7.2f %9.1f", stats->name, (long long unsigned)stats->bp_elapsed.count, min, mean, max, sd, percentage);
 	}
-	ms_message("===========================================================");
+	ms_message("=================================================================================");
 	bctbx_list_free(sorted);
 }
 
@@ -480,6 +516,93 @@ void ms_factory_log_statistics(MSFactory *obj){
 #endif
 typedef void (*init_func_t)(MSFactory *);
 
+static bool_t ms_factory_dlopen_plugin(MSFactory *factory, const char *plugin_path, const char *plugin_name) {
+	bool_t plugin_loaded = FALSE;
+#if defined(HAVE_DLOPEN)
+	void *handle = NULL;
+	void *initroutine = NULL;
+	char *initroutine_name = ms_malloc0(strlen(plugin_name) + 10);
+	char *fullpath = NULL;
+
+	if (plugin_path != NULL) {
+		fullpath = ms_strdup_printf("%s/%s", plugin_path, plugin_name);
+	} else {
+		fullpath = ms_strdup(plugin_name);
+	}
+	ms_message("Loading plugin %s...", fullpath);
+
+	if ((handle = dlopen(fullpath, RTLD_NOW)) == NULL) {
+		ms_warning("Fail to load plugin %s : %s", fullpath, dlerror());
+	} else {
+		char *p = NULL;
+		strcpy(initroutine_name, plugin_name);
+		p = strstr(initroutine_name, PLUGINS_EXT);
+		if (p != NULL) {
+			strcpy(p, "_init");
+		}
+		initroutine = dlsym(handle, initroutine_name);
+	}
+
+#ifdef __APPLE__
+	if (initroutine ==  NULL){
+		char *p = NULL;
+		/* on macosx: library name are libxxxx.1.2.3.dylib */
+		/* -> MUST remove the .1.2.3 */
+		p = strstr(initroutine_name, ".");
+		if (p != NULL) {
+			strcpy(p, "_init");
+			initroutine = dlsym(handle, initroutine_name);
+		}
+	}
+#endif
+
+	if (initroutine != NULL) {
+		init_func_t func = (init_func_t)initroutine;
+		func(factory);
+		ms_message("Plugin loaded (%s)", plugin_name);
+		plugin_loaded = TRUE;
+	} else {
+		ms_warning("Could not locate init routine %s of plugin %s", initroutine_name, plugin_name);
+	}
+
+	ms_free(initroutine_name);
+	if (fullpath != NULL) {
+		ms_free(fullpath);
+	}
+#endif
+	return plugin_loaded;
+}
+
+int ms_factory_load_plugins_from_list(MSFactory *factory, const bctbx_list_t *plugins_list, const char *optionnal_plugins_path) {
+	int num = 0;
+	int plugins_list_size = 0;
+	const bctbx_list_t *it = NULL;
+
+	if (plugins_list == NULL || bctbx_list_size(plugins_list) == 0) {
+		ms_error("Couldn't load plugins from empty list");
+		return -1;
+	}
+
+	plugins_list_size = bctbx_list_size(plugins_list);
+
+	for (it = plugins_list; it != NULL; it = bctbx_list_next(it)) {
+		const char *plugin_name = bctbx_list_get_data(it);
+#if defined(HAVE_DLOPEN)
+		if (ms_factory_dlopen_plugin(factory, optionnal_plugins_path, plugin_name)) {
+			num++;
+		}
+#endif
+	}
+
+	if (num != plugins_list_size) {
+		ms_warning("Couldn't load all plugins in list");
+	} else {
+		ms_message("All plugins in list correctly loaded");
+	}
+
+	return num;
+}
+
 int ms_factory_load_plugins(MSFactory *factory, const char *dir){
 	int num=0;
 #if defined(_WIN32) && !defined(_WIN32_WCE)
@@ -601,7 +724,6 @@ int ms_factory_load_plugins(MSFactory *factory, const char *dir){
 	bctbx_list_t *loaded_plugins = NULL;
 	struct dirent *de;
 	char *ext;
-	char *fullpath;
 	ds=opendir(dir);
 	if (ds==NULL){
 		ms_message("Cannot open directory %s: %s",dir,strerror(errno));
@@ -613,50 +735,12 @@ int ms_factory_load_plugins(MSFactory *factory, const char *dir){
 			(de->d_type==DT_REG || de->d_type==DT_UNKNOWN || de->d_type==DT_LNK) &&
 #endif
 			(strstr(de->d_name, "libms") == de->d_name) && ((ext=strstr(de->d_name,PLUGINS_EXT))!=NULL)) {
-			void *handle;
 			snprintf(plugin_name, MIN(sizeof(plugin_name), (size_t)(ext - de->d_name + 1)), "%s", de->d_name);
 			if (bctbx_list_find_custom(loaded_plugins, (bctbx_compare_func)strcmp, plugin_name) != NULL) continue;
 			loaded_plugins = bctbx_list_append(loaded_plugins, ms_strdup(plugin_name));
-			fullpath=ms_strdup_printf("%s/%s",dir,de->d_name);
-			ms_message("Loading plugin %s...",fullpath);
-
-			if ( (handle=dlopen(fullpath,RTLD_NOW))==NULL){
-				ms_warning("Fail to load plugin %s : %s",fullpath,dlerror());
-			}else {
-				char *initroutine_name=ms_malloc0(strlen(de->d_name)+10);
-				char *p;
-				void *initroutine=NULL;
-				strcpy(initroutine_name,de->d_name);
-				p=strstr(initroutine_name,PLUGINS_EXT);
-				if (p!=NULL){
-					strcpy(p,"_init");
-					initroutine=dlsym(handle,initroutine_name);
-				}
-
-#ifdef __APPLE__
-				if (initroutine==NULL){
-					/* on macosx: library name are libxxxx.1.2.3.dylib */
-					/* -> MUST remove the .1.2.3 */
-					p=strstr(initroutine_name,".");
-					if (p!=NULL)
-					{
-						strcpy(p,"_init");
-						initroutine=dlsym(handle,initroutine_name);
-					}
-				}
-#endif
-
-				if (initroutine!=NULL){
-					init_func_t func=(init_func_t)initroutine;
-					func(factory);
-					ms_message("Plugin loaded (%s)", fullpath);
-					num++;
-				}else{
-					ms_warning("Could not locate init routine of plugin %s",de->d_name);
-				}
-				ms_free(initroutine_name);
+			if (ms_factory_dlopen_plugin(factory, dir, de->d_name)) {
+				num++;
 			}
-			ms_free(fullpath);
 		}
 	}
 	bctbx_list_for_each(loaded_plugins, ms_free);
@@ -687,16 +771,42 @@ void ms_factory_uninit_plugins(MSFactory *factory){
 
 void ms_factory_init_plugins(MSFactory *obj) {
 	if (obj->plugins_dir == NULL) {
-#ifdef PACKAGE_PLUGINS_DIR
-		obj->plugins_dir = ms_strdup(PACKAGE_PLUGINS_DIR);
+#ifdef __APPLE__
+	char *dir = getPluginsDir();
+	if (dir != NULL) {
+	    //getPluginsDir returns an allocated str. No need to strdup
+	    obj->plugins_dir = dir;
+	}
 #else
-		obj->plugins_dir = ms_strdup("");
+#ifdef PACKAGE_PLUGINS_DIR
+	obj->plugins_dir = ms_strdup(PACKAGE_PLUGINS_DIR);
+#else
+	obj->plugins_dir = ms_strdup("");
+#endif
 #endif
 	}
-	if (strlen(obj->plugins_dir) > 0) {
-		ms_message("Loading ms plugins from [%s]",obj->plugins_dir);
-		ms_factory_load_plugins(obj,obj->plugins_dir);
+#if defined(__ANDROID__)
+	bctbx_list_t *plugins_list = ms_get_android_plugins_list();
+	if (plugins_list != NULL) {
+		ms_message("Loading ms plugins from list");
+		if (ms_get_android_sdk_version() <= 23) {
+			char *path = ms_get_android_libraries_path();
+			ms_factory_load_plugins_from_list(obj, plugins_list, path);
+			ms_free(path);
+		} else if (ms_get_android_sdk_version() >= 24) {
+			ms_factory_load_plugins_from_list(obj, plugins_list, NULL);
+		}
+		ms_list_free_with_data(plugins_list, ms_free);
+	} else if (strlen(obj->plugins_dir) > 0) {
+		ms_message("Loading ms plugins from directory [%s]", obj->plugins_dir);
+		ms_factory_load_plugins(obj, obj->plugins_dir);
 	}
+#else
+	if (strlen(obj->plugins_dir) > 0) {
+		ms_message("Loading ms plugins from [%s]", obj->plugins_dir);
+		ms_factory_load_plugins(obj, obj->plugins_dir);
+	}
+#endif
 }
 
 void ms_factory_set_plugins_dir(MSFactory *obj, const char *path) {
@@ -716,7 +826,6 @@ struct _MSEventQueue *ms_factory_create_event_queue(MSFactory *obj) {
 }
 
 void ms_factory_destroy_event_queue(MSFactory *obj) {
-	if (obj->image_resources_dir) bctbx_free(obj->image_resources_dir);
 	ms_event_queue_destroy(obj->evq);
 	ms_factory_set_event_queue(obj,NULL);
 }
@@ -885,6 +994,34 @@ int ms_factory_get_expected_bandwidth(MSFactory *f) {
 	return f->expected_video_bandwidth;
 }
 
+const char * ms_factory_get_default_video_renderer(MSFactory *f) {
+#if defined(MS2_WINDOWS_PHONE)
+	return "MSWP8Dis";
+#elif defined(MS2_WINDOWS_DESKTOP)
+	// Check if UWP filter is created
+	if (ms_factory_lookup_filter_by_name(f, "MSWinRTBackgroundDis")) {
+		return "MSWinRTBackgroundDis";
+	} else {
+		return "MSDrawDibDisplay";
+	}
+#elif defined(__ANDROID__)
+	return "MSAndroidTextureDisplay";
+#elif __APPLE__ && !TARGET_OS_IPHONE
+	return "MSOSXGLDisplay";
+#elif defined (HAVE_XV)
+	return "MSX11Video";
+#elif defined(HAVE_GLX)
+	return "MSGLXVideo";
+#elif TARGET_OS_IPHONE
+	return "IOSDisplay";
+#elif defined(__QNX__)
+	return "MSBB10Display";
+#else
+	return "MSVideoOut";
+#endif
+}
+
+
 #ifdef __ANDROID__
 #include "sys/system_properties.h"
 #include <jni.h>
@@ -992,6 +1129,8 @@ void ms_factory_destroy(MSFactory *factory) {
 	factory->platform_tags = bctbx_list_free(factory->platform_tags);
 	if (factory->echo_canceller_filtername) ms_free(factory->echo_canceller_filtername);
 	if (factory->plugins_dir) ms_free(factory->plugins_dir);
+	if (factory->image_resources_dir) ms_free(factory->image_resources_dir);
+	if (factory->wbcmanager) ms_web_cam_manager_destroy(factory->wbcmanager);
 	ms_free(factory);
 	if (factory == fallback_factory) fallback_factory = NULL;
 }
@@ -1012,4 +1151,7 @@ MSFactory *ms_factory_get_fallback(void){
 	return fallback_factory;
 }
 
+const MSList *ms_factory_get_filter_decs(const MSFactory *factory) {
+	return factory->desc_list;
+}
 
