@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "mediastreamer2/msticker.h"
 
@@ -55,6 +55,7 @@ static void ms_ticker_start(MSTicker *s){
 static void ms_ticker_init(MSTicker *ticker, const MSTickerParams *params)
 {
 	ms_mutex_init(&ticker->lock,NULL);
+	ms_mutex_init(&ticker->cur_time_lock, NULL);
 	ticker->execution_list=NULL;
 	ticker->task_list=NULL;
 	ticker->ticks=1;
@@ -97,8 +98,10 @@ static void ms_ticker_stop(MSTicker *s){
 }
 
 void ms_ticker_set_name(MSTicker *s, const char *name){
+	ms_mutex_lock(&s->lock);
 	if (s->name) ms_free(s->name);
 	s->name=ms_strdup(name);
+	ms_mutex_unlock(&s->lock);
 }
 
 void ms_ticker_set_priority(MSTicker *ticker, MSTickerPrio prio){
@@ -110,6 +113,7 @@ static void ms_ticker_uninit(MSTicker *ticker)
 	ms_ticker_stop(ticker);
 	ms_free(ticker->name);
 	ms_mutex_destroy(&ticker->lock);
+	ms_mutex_destroy(&ticker->cur_time_lock);
 }
 
 void ms_ticker_destroy(MSTicker *ticker){
@@ -392,7 +396,10 @@ static int wait_next_tick(void *data, uint64_t virt_ticker_time){
 	int late;
 
 	while(1){
+		ms_mutex_lock(&s->cur_time_lock);
 		realtime=s->get_cur_time_ptr(s->get_cur_time_data)-s->orig;
+		ms_mutex_unlock(&s->cur_time_lock);
+		
 		diff=s->time-realtime;
 		if (diff>0){
 			/* sleep until next tick */
@@ -413,16 +420,18 @@ void * ms_ticker_run(void *arg)
 	int precision=2;
 	int late;
 
+	ms_mutex_lock(&s->lock);
+
 	precision = set_high_prio(s);
 	s->thread_id = ms_thread_self();
 	s->ticks=1;
+	ms_mutex_lock(&s->cur_time_lock);
 	s->orig=s->get_cur_time_ptr(s->get_cur_time_data);
-
-	ms_mutex_lock(&s->lock);
+	ms_mutex_unlock(&s->cur_time_lock);
 
 	while(s->run){
 		uint64_t late_tick_time=0;
-		
+
 		s->ticks++;
 		/*Step 1: run the graphs*/
 		{
@@ -467,12 +476,14 @@ void * ms_ticker_run(void *arg)
 
 void ms_ticker_set_time_func(MSTicker *ticker, MSTickerTimeFunc func, void *user_data){
 	if (func==NULL) func=get_cur_time_ms;
-
+	
+	ms_mutex_lock(&ticker->cur_time_lock);
 	ticker->get_cur_time_ptr=func;
 	ticker->get_cur_time_data=user_data;
 	/*re-set the origin to take in account that previous function ptr and the
 	new one may return different times*/
 	ticker->orig=func(user_data)-ticker->time;
+	ms_mutex_unlock(&ticker->cur_time_lock);
 
 	ms_message("ms_ticker_set_time_func: ticker's time method updated.");
 }
@@ -550,8 +561,13 @@ void ms_ticker_get_last_late_tick(MSTicker *ticker, MSTickerLateEvent *ev){
 	if (need_lock) ms_mutex_unlock(&ticker->lock);
 }
 
+static void ms_ticker_synchronizer_reset(MSTickerSynchronizer* ts){
+	memset(ts, 0, sizeof(*ts));
+}
+
 void ms_ticker_set_synchronizer(MSTicker *ticker, MSTickerSynchronizer *ts) {
 	if (ts) {
+		ms_ticker_synchronizer_reset(ts);
 		ms_ticker_set_time_func(ticker, (MSTickerTimeFunc)ms_ticker_synchronizer_get_corrected_time, ts);
 	} else {
 		ms_ticker_set_time_func(ticker, NULL, NULL);
@@ -577,29 +593,59 @@ MSTickerSynchronizer* ms_ticker_synchronizer_new(void) {
 	return obj;
 }
 
+void ms_ticker_synchronizer_resync(MSTickerSynchronizer* ts){
+	ts->offset = 0;
+	ts->origin_nsamples = 0;
+	ts->current_nsamples = 0;
+}
+
 double ms_ticker_synchronizer_set_external_time(MSTickerSynchronizer* ts, const MSTimeSpec *time) {
 	int64_t sound_time;
 	int64_t diff;
 	uint64_t wc = get_wallclock_ms();
 	uint64_t ms = get_ms(time);
 	if (ts->offset == 0) {
-		ts->offset = wc - ms;
+		/* In case of resync, the av_skew must be preserved */
+		ts->offset = wc - ms - (uint64_t)ts->av_skew;
 	}
 	sound_time = ts->offset + ms;
 	diff = wc - sound_time;
 	ts->av_skew = (ts->av_skew * (1.0 - clock_coef)) + ((double) diff * clock_coef);
-	if ((++ts->external_time_count) % 100 == 0) {
+	
+	if (ts->last_log_time == 0) ts->last_log_time = wc;
+	else if (wc - ts->last_log_time >= 5000 ) {
+		ts->last_log_time = wc;
+#ifndef __ANDROID__
 		ms_message("sound/wall clock skew is average=%f ms", ts->av_skew);
+#endif
 	}
 	return ts->av_skew;
 }
 
 double ms_ticker_synchronizer_update(MSTickerSynchronizer *ts, uint64_t nb_samples, unsigned int sample_rate) {
-	uint64_t ns = ((1000 * nb_samples) / (uint64_t)sample_rate) * 1000000;
-	MSTimeSpec timespec;
-	timespec.tv_nsec = ns % 1000000000;
-	timespec.tv_sec = ns / 1000000000;
-	return ms_ticker_synchronizer_set_external_time(ts, &timespec);
+	/* It is important that the average clock skew is updated ONLY if the user notifies that nb_samples has changed.
+	   Indeed, the fact that the soundcard didn't delivered samples during a period of time doesn't mean that the time has stopped or slowed.
+	   What we need is that each time we get new samples, we correlate them with system time to make a correction on the msticker timer.
+	   Imagine that the soundcard stops for some reason: without this check the msticker would stop as well.
+	*/
+	if (ts->sample_rate != sample_rate){
+		ms_ticker_synchronizer_resync(ts);
+		ts->sample_rate = sample_rate;
+	}
+	if (nb_samples == 0) return 0.0; /* ignore a measurement that measured nothing */
+	if (ts->origin_nsamples == 0){ /* setup origin of sample count*/
+		ts->current_nsamples = ts->origin_nsamples = nb_samples;
+		return 0.0;
+	}
+	if (nb_samples > ts->current_nsamples){
+		MSTimeSpec timespec;
+		uint64_t ms = (1000 * (nb_samples - ts->origin_nsamples)) / (uint64_t)sample_rate;
+		ts->current_nsamples = nb_samples;
+		timespec.tv_nsec = (ms % 1000) * 1000000LL;
+		timespec.tv_sec = ms / 1000LL;
+		return ms_ticker_synchronizer_set_external_time(ts, &timespec);
+	}
+	return ts->av_skew;
 }
 
 uint64_t ms_ticker_round(uint64_t ms) {
@@ -613,5 +659,6 @@ uint64_t ms_ticker_synchronizer_get_corrected_time(MSTickerSynchronizer* ts) {
 }
 
 void ms_ticker_synchronizer_destroy(MSTickerSynchronizer* ts) {
+	ms_message("ms_ticker_synchronizer_destroy(): sound/wall clock skew was in average=%f ms", ts->av_skew);
 	ms_free(ts);
 }
