@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006-2013 Belledonne Communications, Grenoble
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include "mediastreamer2/mediastream.h"
@@ -27,8 +27,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #include "bctoolbox/crypto.h"
 
-
-
 typedef struct _DtlsBcToolBoxContexts {
 	bctbx_x509_certificate_t *crt;
 	bctbx_ssl_config_t *ssl_config;
@@ -62,6 +60,7 @@ struct _MSDtlsSrtpContext{
 	MSMediaStreamSessions *stream_sessions;
 	MSDtlsSrtpRole role; /**< can be unset(at init on caller side), client or server */
 	char peer_fingerprint[256]; /**< used to store peer fingerprint passed through SDP */
+	int mtu;
 
 	RtpTransportModifier *rtp_modifier;
 	RtpTransportModifier *rtcp_modifier;
@@ -255,6 +254,24 @@ static bool_t ms_dtls_srtp_process_dtls_packet(mblk_t *msg, MSDtlsSrtpContext *c
 
 	/* check if it is a DTLS packet (first byte B as 19 < B < 64) rfc5764 section 5.1.2 */
 	if ((*(msg->b_rptr)>19) && (*(msg->b_rptr)<64)) {
+		/* UGLY PATCH CLIENT HELLO PACKET PARSING */
+		const int Content_Type_Index = 0;
+		const int Content_Length_Index = 11;
+		const int Handshake_Type_Index = 13;
+		const int Handshake_Message_Length_Index = 14;
+		const int Handshake_Message_Seq_Index = 17;
+		const int Handshake_Frag_Offset_Index = 19;
+		const int Handshake_Frag_Length_Index = 22;
+		const size_t Handshake_Header_Length = 25;
+		unsigned char *frag = msg->b_rptr;
+		size_t base_index = 0;
+		int message_length = 0;
+		int message_seq = 0;
+		int current_message_seq = -1;
+		int frag_offset = 0;
+		int frag_length = 0;
+		unsigned char* reassembled_packet = NULL;
+		/* end of UGLY PATCH CLIENT HELLO PACKET PARSING */
 
 		DtlsRawPacket *incoming_dtls_packet;
 		incoming_dtls_packet = (DtlsRawPacket *)ms_malloc0(sizeof(DtlsRawPacket));
@@ -268,6 +285,91 @@ static bool_t ms_dtls_srtp_process_dtls_packet(mblk_t *msg, MSDtlsSrtpContext *c
 
 		ms_message("DTLS Receive %s packet len %d sessions: %p rtp session %p", is_rtp==TRUE?"RTP":"RTCP", (int)msgLength, ctx->stream_sessions, ctx->stream_sessions->rtp_session);
 
+		/* UGLY PATCH CLIENT HELLO PACKET PARSING */
+		/* Parse the DTLS packet to check if we have a Client Hello packet fragmented at DTLS level but all set in one datagram
+		 * (some kind of bug in certain versions openssl produce that and mbedtls does not support Client Hello fragmentation )
+		 * This patch is not very resistant to any change and target that very particular situation, a better solution should
+		 * be to implement support of Client Hello Fragmentation in mbedtls */
+		if (msgLength > Handshake_Header_Length && frag[Content_Type_Index] == 0x16 && frag[Handshake_Type_Index] == 0x01) { // If the first fragment(there may be only one) is of a DTLS Handshake Client Hello message
+			while (base_index + Handshake_Header_Length < msgLength) { // loop on the message, parsing all fragments it may contain (loop until we have at least enough unparsed byte to read a handshake header)
+				if (frag[Content_Type_Index] == 0x16) { // Type index 0x16 is DLTS Handshake message
+					if (frag[Handshake_Type_Index] == 0x01) { // Handshake type 0x01 is Client Hello
+						// Get message length
+						message_length = frag[Handshake_Message_Length_Index] <<16 |
+								frag[Handshake_Message_Length_Index+1] <<8 |
+								frag[Handshake_Message_Length_Index+2];
+
+						// message sequence number
+						message_seq = frag[Handshake_Message_Seq_Index]<<8 | frag[Handshake_Message_Seq_Index+1];
+						if (current_message_seq == -1) {
+							current_message_seq = message_seq;
+						}
+
+						// fragment offset
+						frag_offset = frag[Handshake_Frag_Offset_Index] <<16 |
+								frag[Handshake_Frag_Offset_Index+1] <<8 |
+								frag[Handshake_Frag_Offset_Index+2];
+
+						// and fragment length
+						frag_length = frag[Handshake_Frag_Length_Index] <<16 |
+								frag[Handshake_Frag_Length_Index+1] <<8 |
+								frag[Handshake_Frag_Length_Index+2];
+
+						// check the message is not malformed and would lead us to read after the message buffer
+						// or write after our reassembled packet buffer
+						if ( base_index + Handshake_Header_Length + frag_length <= msgLength // we will read frag_length starting at base_index (frag in the code)+ header
+							&& frag_length + frag_offset <= message_length ) { // we will write in the reassembled buffer frag_length byte, starting at frag_offset, the buffer is message_length long
+
+							// If message length and fragment length differs, we have a fragmented Client Hello
+							// Check they are part of the same message (message_seq)
+							// We will just collect all fragments (in our very particuliar case, they are all in the same datagram so we do not need long term storage,
+							// juste parsing this packet)
+							if (message_length != frag_length && message_seq == current_message_seq) {
+								if (reassembled_packet == NULL) { // this is first fragment we get
+									reassembled_packet = malloc(Handshake_Header_Length + message_length);
+									// copy the header
+									memcpy(reassembled_packet, msg->b_rptr, Handshake_Header_Length);
+									// set the message length to be in line with reassembled fragments
+									reassembled_packet[Content_Length_Index] = ((message_length +12)>>8)&0xFF;
+									reassembled_packet[Content_Length_Index+1] = (message_length +12)&0xFF;
+
+									// set the frag length to be the same than message length
+									reassembled_packet[Handshake_Frag_Length_Index] = reassembled_packet[Handshake_Message_Length_Index];
+									reassembled_packet[Handshake_Frag_Length_Index+1] = reassembled_packet[Handshake_Message_Length_Index+1];
+									reassembled_packet[Handshake_Frag_Length_Index+2] = reassembled_packet[Handshake_Message_Length_Index+2];
+								}
+								// copy the received fragment
+								memcpy(reassembled_packet+Handshake_Header_Length+frag_offset, frag+Handshake_Header_Length, frag_length);
+							}
+
+							// read what is next in the datagram
+							base_index += Handshake_Header_Length + frag_length; // bytes parsed so far
+							frag += Handshake_Header_Length + frag_length; // point to the begining of the next fragment
+						} else { // message is malformed in a nasty way
+							ms_warning("DTLS Received %s packet len %d sessions: %p rtp session %p is malformed in an agressive way", is_rtp==TRUE?"RTP":"RTCP", (int)msgLength, ctx->stream_sessions, ctx->stream_sessions->rtp_session);
+							base_index = msgLength; // get out of the while
+							ms_free(reassembled_packet);
+							reassembled_packet = NULL;
+						}
+					} else {
+						base_index = msgLength; // get out of the while
+						ms_free(reassembled_packet);
+						reassembled_packet = NULL;
+					}
+				}
+			}
+		}
+		// if we made a reassembled client hello packet, use this one as incoming dlts packet and discard the original one
+		if (reassembled_packet != NULL) {
+			ms_message("DTLS re-assembled a fragmented Client Hello packet");
+			ms_free(incoming_dtls_packet->data);
+			incoming_dtls_packet->data=(unsigned char *)ms_malloc(Handshake_Header_Length+message_length);
+			incoming_dtls_packet->length=Handshake_Header_Length+message_length;
+			memcpy(incoming_dtls_packet->data, reassembled_packet, Handshake_Header_Length+message_length);
+			ms_free(reassembled_packet);
+		}
+		/* end of UGLY PATCH CLIENT HELLO PACKET PARSING */
+
 		/* store the packet in the incoming buffer */
 		if (is_rtp == TRUE) {
 			if (ctx->rtp_incoming_buffer==NULL) { /* buffer is empty */
@@ -297,6 +399,7 @@ static bool_t ms_dtls_srtp_process_dtls_packet(mblk_t *msg, MSDtlsSrtpContext *c
 			ms_mutex_lock(mutex);
 			/* process the packet and store result */
 			*ret = bctbx_ssl_handshake(ssl);
+			ms_message("DTLS Handshake process %s packet len %d sessions: %p rtp session %p return %s0x%0x", is_rtp==TRUE?"RTP":"RTCP", (int)msgLength, ctx->stream_sessions, ctx->stream_sessions->rtp_session, *ret>0?"+":"-", *ret>0?*ret:-*ret);
 
 			/* if we are client, manage the retransmission timer */
 			if (ctx->role == MSDtlsSrtpRoleIsClient) {
@@ -308,6 +411,7 @@ static bool_t ms_dtls_srtp_process_dtls_packet(mblk_t *msg, MSDtlsSrtpContext *c
 			unsigned char *buf = ms_malloc(msgLength+1);
 			ms_mutex_lock(mutex);
 			*ret = bctbx_ssl_read(ssl, buf, msgLength);
+			ms_message("DTLS Handshake read %s packet len %d sessions: %p rtp session %p return %s0x%0x", is_rtp==TRUE?"RTP":"RTCP", (int)msgLength, ctx->stream_sessions, ctx->stream_sessions->rtp_session, *ret>0?"+":"-", *ret>0?*ret:-*ret);
 			ms_mutex_unlock(mutex);
 		}
 
@@ -485,6 +589,7 @@ static int ms_dtls_srtp_rtp_process_on_receive(struct _RtpTransportModifier *t,
 		
 		if ((ret==0) && (ctx->rtp_channel_status == DTLS_STATUS_HANDSHAKE_ONGOING)) { /* handshake is over, give the keys to srtp : 128 bits client write - 128 bits server write - 112 bits client salt - 112 server salt */
 			MSCryptoSuite agreed_srtp_protection_profile = MS_CRYPTO_SUITE_INVALID;
+			uint8_t old_status = ctx->rtp_channel_status;
 
 			ctx->rtp_channel_status = DTLS_STATUS_HANDSHAKE_OVER;
 
@@ -530,6 +635,12 @@ static int ms_dtls_srtp_rtp_process_on_receive(struct _RtpTransportModifier *t,
 
 					ms_dtls_srtp_check_channels_status(ctx);
 				}
+				/* It is possible peer_fingerprint is not set yet, in such case stay in existing status waiting for it to be set */
+				else if (ctx->peer_fingerprint[0]=='\0') {
+					ms_warning("DTLS-SRTP: empty peer_fingerprint, staying in old state...");
+					ctx->rtp_channel_status=old_status;
+					return (int)msgLength;
+				}
 			}
 
 			if (ctx->role != MSDtlsSrtpRoleIsServer) { /* close the connection only if we are client, if we are server, the client may ask again for last packets */
@@ -787,6 +898,7 @@ MSDtlsSrtpContext* ms_dtls_srtp_context_new(MSMediaStreamSessions *sessions, MSD
 	userData->rtp_dtls_context=rtp_dtls_context;
 	userData->rtcp_dtls_context=rtcp_dtls_context;
 	userData->role = params->role;
+	userData->mtu = params->mtu;
 	userData->rtp_time_reference = 0;
 	userData->rtcp_time_reference = 0;
 
@@ -821,11 +933,12 @@ MSDtlsSrtpContext* ms_dtls_srtp_context_new(MSMediaStreamSessions *sessions, MSD
 }
 
 void ms_dtls_srtp_start(MSDtlsSrtpContext* context) {
+
 	if (context == NULL ) {
 		ms_warning("DTLS start but no context\n");
 		return;
 	}
-	ms_message("DTLS start stream on stream sessions [%p], RCTP mux is %s", context->stream_sessions, rtp_session_rtcp_mux_enabled(context->stream_sessions->rtp_session)?"enabled":"disabled");
+	ms_message("DTLS start stream on stream sessions [%p], RCTP mux is %s, MTU is %d", context->stream_sessions, rtp_session_rtcp_mux_enabled(context->stream_sessions->rtp_session)?"enabled":"disabled", context->mtu);
 
 	/* if we are client, start the handshake(send a clientHello) */
 	if (context->role == MSDtlsSrtpRoleIsClient) {
@@ -833,6 +946,7 @@ void ms_dtls_srtp_start(MSDtlsSrtpContext* context) {
 		bctbx_ssl_config_set_endpoint(context->rtp_dtls_context->ssl_config, BCTBX_SSL_IS_CLIENT);
 		/* complete ssl setup*/
 		bctbx_ssl_context_setup(context->rtp_dtls_context->ssl, context->rtp_dtls_context->ssl_config);
+		bctbx_ssl_set_mtu(context->rtp_dtls_context->ssl, context->mtu);		
 		/* and start the handshake */
 		bctbx_ssl_handshake(context->rtp_dtls_context->ssl);
 		context->rtp_time_reference = get_timeval_in_millis(); /* arm the timer for retransmission */
@@ -844,6 +958,7 @@ void ms_dtls_srtp_start(MSDtlsSrtpContext* context) {
 			bctbx_ssl_config_set_endpoint(context->rtcp_dtls_context->ssl_config, BCTBX_SSL_IS_CLIENT);
 			/* complete ssl setup*/
 			bctbx_ssl_context_setup(context->rtcp_dtls_context->ssl, context->rtcp_dtls_context->ssl_config);
+			bctbx_ssl_set_mtu(context->rtcp_dtls_context->ssl, context->mtu);		
 			/* and start the handshake */
 			bctbx_ssl_handshake(context->rtcp_dtls_context->ssl);
 			context->rtcp_time_reference = get_timeval_in_millis(); /* arm the timer for retransmission */
@@ -859,6 +974,7 @@ void ms_dtls_srtp_start(MSDtlsSrtpContext* context) {
 			bctbx_ssl_config_set_endpoint(context->rtp_dtls_context->ssl_config, BCTBX_SSL_IS_SERVER);
 			/* complete ssl setup*/
 			bctbx_ssl_context_setup(context->rtp_dtls_context->ssl, context->rtp_dtls_context->ssl_config);
+			bctbx_ssl_set_mtu(context->rtp_dtls_context->ssl, context->mtu);		
 			context->rtp_channel_status = DTLS_STATUS_HANDSHAKE_ONGOING;
 			ms_mutex_unlock(&context->rtp_dtls_context->ssl_context_mutex);
 
@@ -868,6 +984,7 @@ void ms_dtls_srtp_start(MSDtlsSrtpContext* context) {
 				bctbx_ssl_config_set_endpoint(context->rtcp_dtls_context->ssl_config, BCTBX_SSL_IS_SERVER);
 				/* complete ssl setup*/
 				bctbx_ssl_context_setup(context->rtcp_dtls_context->ssl, context->rtcp_dtls_context->ssl_config);
+				bctbx_ssl_set_mtu(context->rtcp_dtls_context->ssl, context->mtu);		
 				context->rtcp_channel_status = DTLS_STATUS_HANDSHAKE_ONGOING;
 				ms_mutex_unlock(&context->rtcp_dtls_context->ssl_context_mutex);
 			}
