@@ -1,21 +1,21 @@
 /*
-  mediastreamer2 library - modular sound and video processing and streaming
-  Copyright (C) 2006-2014 Belledonne Communications, Grenoble
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "mediastreamer-config.h"
@@ -34,13 +34,7 @@
 #undef PACKAGE_TARNAME
 #undef PACKAGE_VERSION
 
-#if defined(MS2_WINDOWS_PHONE)
-// Windows phone doesn't use make install
-#include <srtp.h>
-#else
-#include <srtp/srtp.h>
-#endif
-
+#include "srtp_prefix.h"
 
 #include "ortp/b64.h"
 
@@ -51,6 +45,8 @@ typedef struct _MSSrtpStreamContext {
 	bool_t secured;
 	bool_t mandatory_enabled;
 	bool_t is_rtp;
+	/*fixed: for rtcp-mux*/
+	struct _MSSrtpStreamContext *other_rtp;
 } MSSrtpStreamContext;
 
 struct _MSSrtpCtx {
@@ -69,6 +65,13 @@ MSSrtpCtx* ms_srtp_context_new(void) {
 	ctx->recv_rtp_context.is_rtp=TRUE;
 	ms_mutex_init(&ctx->recv_rtp_context.mutex, NULL);
 	ms_mutex_init(&ctx->recv_rtcp_context.mutex, NULL);
+
+	/*fixed: for rtcp-mux*/
+	ctx->send_rtp_context.other_rtp = &ctx->send_rtcp_context;
+	ctx->send_rtcp_context.other_rtp = &ctx->send_rtp_context;
+	ctx->recv_rtp_context.other_rtp = &ctx->recv_rtcp_context;
+	ctx->recv_rtcp_context.other_rtp = &ctx->recv_rtp_context;
+
 	return ctx;
 }
 void ms_srtp_context_delete(MSSrtpCtx* session) {
@@ -105,6 +108,12 @@ static int _process_on_send(RtpSession* session,MSSrtpStreamContext *ctx, mblk_t
 	rtcp_common_header_t *rtcp_header=!is_rtp?(rtcp_common_header_t*)m->b_rptr:NULL;
 	slen=(int)msgdsize(m);
 
+	/* fixed: when rtcp-mux is on and we need to send RTCP packet, we need to use same srtp context as used for RTP stream */
+	if (!is_rtp && session->rtcp_mux) {
+		if (ctx->other_rtp)
+			ctx = ctx->other_rtp;
+	}
+
 	if (rtp_header && (slen>RTP_FIXED_HEADER_SIZE && rtp_header->version==2)) {
 		ms_mutex_lock(&ctx->mutex);
 		if (!ctx->secured) {
@@ -154,6 +163,12 @@ static int _process_on_receive(RtpSession* session,MSSrtpStreamContext *ctx, mbl
 	err_status_t srtp_err;
 	bool_t is_rtp=ctx->is_rtp;
 
+	/* fixed: when rtcp-mux is on and we receive RTCP packet, we need to use same srtp context as used for RTP stream */
+	if (!is_rtp && session->rtcp_mux) {
+		if (ctx->other_rtp)
+			ctx = ctx->other_rtp;
+	}
+
 	/* keep NON-RTP data unencrypted */
 	if (is_rtp){
 		rtp_header_t *rtp=(rtp_header_t*)m->b_rptr;
@@ -276,8 +291,8 @@ static int ms_set_srtp_crypto_policy(MSCryptoSuite suite, crypto_policy_t *polic
 			break;
 		case MS_AES_256_SHA1_80: // For backward compatibility
 		case MS_AES_CM_256_SHA1_80:
-		    crypto_policy_set_aes_cm_256_hmac_sha1_80(policy);
-            break;
+			crypto_policy_set_aes_cm_256_hmac_sha1_80(policy);
+			break;
 		case MS_AES_256_SHA1_32:
 			crypto_policy_set_aes_cm_256_hmac_sha1_32(policy);
 			break;
@@ -288,38 +303,42 @@ static int ms_set_srtp_crypto_policy(MSCryptoSuite suite, crypto_policy_t *polic
 	return 0;
 }
 
-static int ms_add_srtp_stream(srtp_t srtp, MSCryptoSuite suite, uint32_t ssrc, const char* key, size_t key_length, bool_t is_send, bool_t is_rtp)
-{
+static int ms_add_srtp_stream(srtp_t srtp, MSCryptoSuite suite, uint32_t ssrc, const char *key, size_t key_length, bool_t is_send, bool_t is_rtp) {
 	srtp_policy_t policy;
 	err_status_t err;
 	ssrc_t ssrc_conf;
 
 	memset(&policy,0,sizeof(policy));
 
-	if (is_rtp) {
-		if (ms_set_srtp_crypto_policy(suite, &policy.rtp) != 0) {
-			return -1;
-		}
-		/* check if key length match given policy */
-		if ((int)key_length != policy.rtp.cipher_key_len) {
-			ms_error("Key size (%i) doesn't match the selected srtp profile (required %d)", (int)key_length, policy.rtp.cipher_key_len);	
-			return -1;
-		}
-	}else {
-		if (ms_set_srtp_crypto_policy(suite, &policy.rtcp) != 0) {
-			return -1;
-		}
-		if ((int)key_length != policy.rtcp.cipher_key_len) {
-			ms_error("Key size (%i) doesn't match the selected srtp profile (required %d)", (int)key_length, policy.rtcp.cipher_key_len);	
-			return -1;
-		}
+	/* Init both RTP and RTCP policies, even if this srtp_t is used for one of them.
+	 * Indeed the key derivation algorithm that computes the SRTCP auth key depends on parameters 
+	 * of the SRTP stream.*/
+	if (ms_set_srtp_crypto_policy(suite, &policy.rtp) != 0) {
+		return -1;
 	}
+	if ((int)key_length != policy.rtp.cipher_key_len) {
+		ms_error("Key size (%i) doesn't match the selected srtp profile (required %d)", (int)key_length,
+					policy.rtp.cipher_key_len);
+		return -1;
+	}
+	// and RTCP stream
+	if (ms_set_srtp_crypto_policy(suite, &policy.rtcp) != 0) {
+		return -1;
+	}
+	if ((int)key_length != policy.rtcp.cipher_key_len) {
+		ms_error("Key size (%i) doesn't match the selected srtp profile (required %d)", (int)key_length,
+					policy.rtcp.cipher_key_len);
+		return -1;
+	}
+
 
 	if (is_send)
 		policy.allow_repeat_tx=1; /*necessary for telephone-events*/
 
-	ssrc_conf.type=is_send ? ssrc_specific:ssrc_any_inbound;
-	ssrc_conf.value=ssrc;
+	/* When RTP bundle mode is used, the srtp_t is used to encrypt or decrypt several RTP streams (SSRC) at the same time.
+	 * This is why we use the "template" mode of libsrtp, using ssrc_any_inbound and ssrc_any_outbound */
+	ssrc_conf.type=is_send ? ssrc_any_outbound : ssrc_any_inbound;
+	ssrc_conf.value=0;
 
 	policy.ssrc = ssrc_conf;
 	policy.key = (uint8_t *)key;
@@ -352,7 +371,6 @@ int ms_srtp_init(void)
 			srtp_init_done++;
 		}else{
 			ms_fatal("Couldn't initialize SRTP library: %d.", st);
-			err_reporting_init("mediastreamer2");
 		}
 	}else srtp_init_done++;
 	return (int)st;
@@ -380,7 +398,7 @@ static int ms_media_stream_sessions_set_srtp_key_base(MSMediaStreamSessions *ses
 		return error;
 	}
 
-	if ((error = ms_add_srtp_stream(stream_ctx->srtp,suite, ssrc, key, key_length, is_send, is_rtp))) {
+	if ((error = ms_add_srtp_stream(stream_ctx->srtp, suite, ssrc, key, key_length, is_send, is_rtp))) {
 		stream_ctx->secured=FALSE;
 		return error;
 	}
