@@ -1,21 +1,21 @@
 /*
-  mediastreamer2 library - modular sound and video processing and streaming
-  Copyright (C) 2014 Belledonne Communications
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "mediastreamer2/zrtp.h"
 #include "mediastreamer2/mediastream.h"
@@ -39,6 +39,8 @@ struct _MSZrtpContext{
 	/* cache related data */
 	uint32_t limeKeyTimeSpan; /**< amount in seconds of the lime key life span */
 	void *cacheDB; /**< pointer to an already open sqlite db holding the zid cache */
+	bctbx_mutex_t *cacheDBMutex; /**< pointer to a mutex used to lock cache access */
+	bool_t autoStart; /*allow zrtp to start on first hello packet received*/
 };
 
 /***********************************************/
@@ -68,6 +70,8 @@ static const char *bzrtp_keyAgreement_toString(uint8_t keyAgreementAlgo) {
 		case(ZRTP_KEYAGREEMENT_DH3k): return "DHM-3072";
 		case(ZRTP_KEYAGREEMENT_EC38): return "ECDH-384";
 		case(ZRTP_KEYAGREEMENT_EC52): return "ECDH-521";
+		case(ZRTP_KEYAGREEMENT_X255): return "ECDH-255";
+		case(ZRTP_KEYAGREEMENT_X448): return "ECDH-448";
 		case(ZRTP_KEYAGREEMENT_Prsh): return "PreShared";
 		case(ZRTP_KEYAGREEMENT_Mult): return "MultiStream";
 		default: return "Unknown Algo";
@@ -321,7 +325,7 @@ static int ms_zrtp_addExportedKeysInZidCache(void *clientData, int zuid, uint8_t
 	bzrtpContext_t *zrtpContext = userData->zrtpContext;
 	bctoolboxTimeSpec currentTime;
 	/* columns to be written in cache */
-	char *colNames[] = {"sndKey", "rcvKey", "sndSId", "rcvSId", "sndIndex", "rcvIndex", "valid"};
+	const char *colNames[] = {"sndKey", "rcvKey", "sndSId", "rcvSId", "sndIndex", "rcvIndex", "valid"};
 	uint8_t *colValues[7];
 	size_t colLength[] = {32, 32, 32, 32, 4, 4, 8}; /* data length: keys and session ID : 32 bytes, Indexes: 4 bytes(uint32_t), validity : 8 bytes(UTC time as int64_t) */
 	int i,ret;
@@ -361,7 +365,7 @@ static int ms_zrtp_addExportedKeysInZidCache(void *clientData, int zuid, uint8_t
 	colValues[5][0] &= 0x7F;
 
 	/* then insert all in cache */
-	ret = bzrtp_cache_write(userData->cacheDB, zuid, "lime", colNames, colValues, colLength, 7);
+	ret = bzrtp_cache_write_lock(userData->cacheDB, zuid, "lime", colNames, colValues, colLength, 7, userData->cacheDBMutex);
 
 	for (i=0; i<7; i++) {
 		ms_free(colValues[i]);
@@ -420,7 +424,7 @@ static int ms_zrtp_rtp_process_on_receive(RtpTransportModifier *t, mblk_t *msg){
 	ms_message("ZRTP Receive packet type %.8s on rtp session [%p]", rtp+16, t->session);
 
 	// check if the ZRTP channel is started, if not(ZRTP not set on our side but incoming zrtp packets), start it
-	if (bzrtp_getChannelStatus(zrtpContext, userData->self_ssrc) == BZRTP_CHANNEL_INITIALISED) {
+	if (userData->autoStart && bzrtp_getChannelStatus(zrtpContext, userData->self_ssrc) == BZRTP_CHANNEL_INITIALISED) {
 		bzrtp_startChannelEngine(zrtpContext, userData->self_ssrc);
 	}
 
@@ -545,6 +549,8 @@ static void set_key_agreement_suites(bzrtpContext_t *ctx, const MSZrtpKeyAgreeme
 			case MS_ZRTP_KEY_AGREEMENT_EC25:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC25; break;
 			case MS_ZRTP_KEY_AGREEMENT_EC38:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC38; break;
 			case MS_ZRTP_KEY_AGREEMENT_EC52:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_EC52; break;
+			case MS_ZRTP_KEY_AGREEMENT_X255:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_X255; break;
+			case MS_ZRTP_KEY_AGREEMENT_X448:    bzrtpKeyAgreements[bzrtpCount++] = ZRTP_KEYAGREEMENT_X448; break;
 		}
 	}
 
@@ -592,7 +598,7 @@ MSZrtpContext* ms_zrtp_context_new(MSMediaStreamSessions *sessions, MSZrtpParams
 
 	if (params->zidCacheDB != NULL && params->selfUri != NULL && params->peerUri) { /* to enable cache we need a self and peer uri and a pointer to the sqlite cache DB */
 		/*enabling cache*/
-		bzrtp_setZIDCache(context, params->zidCacheDB, params->selfUri, params->peerUri);
+		bzrtp_setZIDCache_lock(context, params->zidCacheDB, params->selfUri, params->peerUri, params->zidCacheDBMutex);
 		cbs.bzrtp_contextReadyForExportedKeys=ms_zrtp_addExportedKeysInZidCache;
 	}
 
@@ -622,8 +628,10 @@ MSZrtpContext* ms_zrtp_context_new(MSMediaStreamSessions *sessions, MSZrtpParams
 	userData->self_ssrc = sessions->rtp_session->snd.ssrc;
 
 	userData->limeKeyTimeSpan = params->limeKeyTimeSpan;
-	userData->cacheDB = params->zidCacheDB; /* add a link to the ZidCache to be able to free it when we will destroy this context */
-
+	userData->cacheDB = params->zidCacheDB; /* add a link to the ZidCache and mutex to be able to access it from callbacks */
+	userData->cacheDBMutex = params->zidCacheDBMutex;
+	userData->autoStart = params->autoStart;
+	
 	bzrtp_setClientData(context, sessions->rtp_session->snd.ssrc, (void *)userData);
 
 	return ms_zrtp_configure_context(userData, sessions->rtp_session);
@@ -678,14 +686,36 @@ void ms_zrtp_sas_verified(MSZrtpContext* ctx){
 	bzrtp_SASVerified(ctx->zrtpContext);
 }
 
+uint8_t ms_zrtp_getAuxiliarySharedSecretMismatch(MSZrtpContext *ctx) {
+	return bzrtp_getAuxiliarySharedSecretMismatch(ctx->zrtpContext);
+}
+
 void ms_zrtp_sas_reset_verified(MSZrtpContext* ctx){
 	bzrtp_resetSASVerified(ctx->zrtpContext);
 }
 
+MSZrtpPeerStatus ms_zrtp_get_peer_status(void *db, const char *peerUri, bctbx_mutex_t *dbMutex){
+	int status = bzrtp_cache_getPeerStatus_lock(db, peerUri, dbMutex);
+	switch (status) {
+		case BZRTP_CACHE_PEER_STATUS_UNKNOWN:
+			return MS_ZRTP_PEER_STATUS_UNKNOWN;
+		case BZRTP_CACHE_PEER_STATUS_INVALID:
+			return MS_ZRTP_PEER_STATUS_INVALID;
+		case BZRTP_CACHE_PEER_STATUS_VALID:
+			return MS_ZRTP_PEER_STATUS_VALID;
+		default:
+			return MS_ZRTP_PEER_STATUS_UNKNOWN;
+	}
+}
+
 int ms_zrtp_getHelloHash(MSZrtpContext* ctx, uint8_t *output, size_t outputLength) {
 	return bzrtp_getSelfHelloHash(ctx->zrtpContext, ctx->self_ssrc, output, outputLength);
 }
 
+int ms_zrtp_setAuxiliarySharedSecret(MSZrtpContext *ctx, const uint8_t *auxSharedSecret, size_t auxSharedSecretLength) {
+	return bzrtp_setAuxiliarySharedSecret(ctx->zrtpContext, auxSharedSecret, auxSharedSecretLength);
+}
+
 int ms_zrtp_setPeerHelloHash(MSZrtpContext *ctx, uint8_t *peerHelloHashHexString, size_t peerHelloHashHexStringLength) {
 	return bzrtp_setPeerHelloHash(ctx->zrtpContext, ctx->self_ssrc, peerHelloHashHexString, peerHelloHashHexStringLength);
 }
@@ -695,11 +725,12 @@ int ms_zrtp_setPeerHelloHash(MSZrtpContext *ctx, uint8_t *peerHelloHashHexString
  * 	Also manage DB schema upgrade
  * @param[in/out]	db	Pointer to the sqlite3 db open connection
  * 				Use a void * to keep this API when building cacheless
+ * @param[in]		dbMutex	a mutex to synchronise zrtp cache database operation. Ignored if NULL
  *
  * @return 0 on succes, MSZRTP_CACHE_SETUP if cache was empty, MSZRTP_CACHE_UPDATE if db structure was updated error code otherwise
  */
-int ms_zrtp_initCache(void *db) {
-	int ret = bzrtp_initCache(db);
+int ms_zrtp_initCache(void *db, bctbx_mutex_t *dbMutex) {
+	int ret = bzrtp_initCache_lock(db, dbMutex);
 	switch (ret) {
 		case BZRTP_CACHE_SETUP:
 			return MSZRTP_CACHE_SETUP;
@@ -738,7 +769,7 @@ int ms_zrtp_cache_migration(void *cacheXmlPtr, void *cacheSqlite, const char *se
 	}
 }
 
-#else
+#else /* HAVE_ZRTP */
 
 MSZrtpContext* ms_zrtp_context_new(MSMediaStreamSessions *sessions, MSZrtpParams *params){
 	ms_message("ZRTP is disabled");
@@ -754,6 +785,7 @@ int ms_zrtp_channel_start(MSZrtpContext *ctx) { return 0;}
 bool_t ms_zrtp_available(){return FALSE;}
 void ms_zrtp_sas_verified(MSZrtpContext* ctx){}
 void ms_zrtp_sas_reset_verified(MSZrtpContext* ctx){}
+MSZrtpPeerStatus ms_zrtp_get_peer_status(void *db, const char *peerUri, bctbx_mutex_t *dbMutex) {return MS_ZRTP_PEER_STATUS_UNKNOWN;}
 void ms_zrtp_context_destroy(MSZrtpContext *ctx){}
 void ms_zrtp_reset_transmition_timer(MSZrtpContext* ctx) {};
 int ms_zrtp_transport_modifier_new(MSZrtpContext* ctx, RtpTransportModifier **rtpt, RtpTransportModifier **rtcpt ) {return 0;}
@@ -761,9 +793,9 @@ void ms_zrtp_transport_modifier_destroy(RtpTransportModifier *tp)  {}
 void ms_zrtp_set_stream_sessions(MSZrtpContext *zrtp_context, MSMediaStreamSessions *stream_sessions) {}
 int ms_zrtp_getHelloHash(MSZrtpContext* ctx, uint8_t *output, size_t outputLength) {return 0;}
 int ms_zrtp_setPeerHelloHash(MSZrtpContext *ctx, uint8_t *peerHelloHashHexString, size_t peerHelloHashHexStringLength) {return 0;}
-int ms_zrtp_initCache(void *db){return 0;}
+int ms_zrtp_initCache(void *db, bctbx_mutex_t *dbMutex){return 0;}
 int ms_zrtp_cache_migration(void *cacheXmlPtr, void *cacheSqlite, const char *selfURI) {return 0;}
-#endif
+#endif /* HAVE_ZRTP */
 
 #define STRING_COMPARE_RETURN(string, value)\
 	if (strcmp(string,#value) == 0) return value
@@ -842,6 +874,8 @@ MSZrtpKeyAgreement ms_zrtp_key_agreement_from_string(const char* str) {
 	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_EC25);
 	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_EC38);
 	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_EC52);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_X255);
+	STRING_COMPARE_RETURN(str, MS_ZRTP_KEY_AGREEMENT_X448);
 	return MS_ZRTP_KEY_AGREEMENT_INVALID;
 }
 
@@ -853,6 +887,8 @@ const char* ms_zrtp_key_agreement_to_string(const MSZrtpKeyAgreement keyAgreemen
 		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_EC25);\
 		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_EC38);\
 		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_EC52);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_X255);\
+		CASE_RETURN_STRING(MS_ZRTP_KEY_AGREEMENT_X448);\
 	);
 }
 
