@@ -1,26 +1,27 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include "mediastreamer2/msfactory.h"
 #include "mediastreamer2/msrtp.h"
 #include "mediastreamer2/msticker.h"
+#include "mediastreamer2/box-plot.h"
 
 #include "ortp/telephonyevents.h"
 #if defined(__cplusplus)
@@ -33,6 +34,7 @@ static const int default_dtmf_duration_ms=100; /*in milliseconds*/
 
 struct SenderData {
 	RtpSession *session;
+	MSBoxPlot processing_delay_stats;
 	uint32_t tsoff;
 	uint32_t last_ts;
 	int64_t last_sent_time;
@@ -378,7 +380,11 @@ static int send_dtmf(MSFilter * f, uint32_t timestamp_start)
 static void check_stun_sending(MSFilter *f) {
 	SenderData *d = (SenderData *) f->data;
 	RtpSession *s = d->session;
-	if ((d->last_rtp_stun_sent_time == -1) || ((f->ticker->time- d->last_sent_time>20000) /*no need to send stun packets if media sent during last 20s*/
+	/* No need to send stun packets if media was sent during last 20s (or the last 2s while we still have not sent any packets) */
+	uint64_t last_sent_timeout = 20000;
+	if (rtp_session_get_stats(s)->packet_sent == 0)
+		last_sent_timeout = 2000;
+	if ((d->last_rtp_stun_sent_time == -1) || (((f->ticker->time - d->last_sent_time) > last_sent_timeout)
 											&& (f->ticker->time - d->last_rtp_stun_sent_time) >= 500)) {
 		d->last_rtp_stun_sent_time = f->ticker->time;
 		send_stun_packet(d,TRUE,FALSE);
@@ -410,6 +416,24 @@ static void process_cn(MSFilter *f, SenderData *d, uint32_t timestamp, mblk_t *i
 	}
 }
 
+static void compute_processing_delay_stats(MSFilter *f, mblk_t *im) {
+	SenderData *d = (SenderData *)f->data;
+	uint64_t source_ts = (mblk_get_timestamp_info(im) * 1000ULL) / (uint64_t)d->rate;
+	int delay = (int)(f->ticker->time - source_ts);
+	ms_box_plot_add_value(&d->processing_delay_stats, delay);
+}
+
+static void print_processing_delay_stats(SenderData *d) {
+	int payloadId = d->session->snd.pt;
+	PayloadType *pt = d->session->snd.profile->payload[payloadId];
+	if (pt && pt->type == PAYLOAD_VIDEO) {
+		char *box_plot_str = ms_box_plot_to_string(&d->processing_delay_stats, "ms");
+		ms_message("video processing delay = %s (RtpSession=%p)", box_plot_str, d->session);
+		ms_box_plot_reset(&d->processing_delay_stats);
+		bctbx_free(box_plot_str);
+	}
+}
+
 static void _sender_process(MSFilter * f)
 {
 	SenderData *d = (SenderData *) f->data;
@@ -446,6 +470,7 @@ static void _sender_process(MSFilter * f)
 			}
 		}
 		if (im){
+			compute_processing_delay_stats(f, im);
 			if (d->skip == FALSE && d->mute==FALSE){
 				header = rtp_session_create_packet(s, 12, NULL, 0);
 				rtp_set_markbit(header, mblk_get_marker_info(im));
@@ -472,6 +497,11 @@ static void _sender_process(MSFilter * f)
 
 	/*every second, compute output bandwidth*/
 	if (f->ticker->time % 1000 == 0) rtp_session_compute_send_bandwidth(d->session);
+
+	if (f->ticker->time % 5000 == 0) {
+		print_processing_delay_stats(d);
+		ms_box_plot_reset(&d->processing_delay_stats);
+	}
 	ms_filter_unlock(f);
 }
 
@@ -587,7 +617,8 @@ static void receiver_init(MSFilter * f)
 }
 
 static void receiver_postprocess(MSFilter * f){
-	/*ReceiverData *d = (ReceiverData *) f->data;*/
+	ReceiverData *d = (ReceiverData *) f->data;
+	rtp_session_reset_recvfrom(d->session);
 }
 
 static void receiver_uninit(MSFilter * f){
