@@ -1,21 +1,21 @@
-/* msdscap - mediastreamer2 plugin for video capture using directshow 
-   This source code requires visual studio to build.
-
-   Copyright (C) 2009 Simon Morlat
-
-   This program is free software: you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>. 
-*/
+/*
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <mediastreamer2/mswebcam.h>
 #include <mediastreamer2/msfilter.h>
@@ -147,8 +147,6 @@ public:
 		_vsize.width=MS_VIDEO_SIZE_CIF_W;
 		_vsize.height=MS_VIDEO_SIZE_CIF_H;
 		_fps=15;
-		_start_time=0;
-		_frame_count=0;
 		_pixfmt=MS_YUV420P;
 		_ready=false;
 		m_refCount=1;
@@ -172,7 +170,7 @@ public:
 		ms_mutex_unlock(&_mutex);
 		return ret;
 	}
-	bool isTimeToSend(uint64_t ticker_time);
+	bool_t isTimeToSend(uint64_t ticker_time);
 	MSVideoSize getVSize(){
 		if (!_ready) createDshowGraph(); /* so that _vsize is updated according to hardware capabilities*/
 		return _vsize;
@@ -194,6 +192,7 @@ public:
 		_devid=index;
 	}
 	MSAverageFPS avgfps;
+	MSFrameRateController framerate_controller;
 protected:
   	long m_refCount;
 private:
@@ -205,8 +204,6 @@ private:
 	queue_t _rq;
 	ms_mutex_t _mutex;
 	float _fps;
-	float _start_time;
-	int _frame_count;
 	MSPixFmt _pixfmt;
 	SharedComPtr< IGraphBuilder > _graphBuilder;
 	SharedComPtr< IBaseFilter > _source;
@@ -371,7 +368,11 @@ int DSCapture::selectBestFormat(SharedComPtr<IAMStreamConfig> streamConfig, int
 int DSCapture::createDshowGraph(){
 	SharedComPtr< ICreateDevEnum > createDevEnum;
 	
+#ifdef ENABLE_MICROSOFT_STORE_APP
+	CoInitializeEx(NULL, COINIT_MULTITHREADED);
+#else
 	CoInitialize(NULL);
+#endif
 	if ((createDevEnum=makeShared<ICreateDevEnum>( CLSID_SystemDeviceEnum,
                                     IID_ICreateDevEnum ))==NULL){
 		ms_error("Could not create device enumerator");
@@ -534,17 +535,8 @@ void DSCapture::stopAndClean(){
 	_ready=false;
 }
 
-bool DSCapture::isTimeToSend(uint64_t ticker_time){
-	if (_frame_count==-1){
-		_start_time=(float)ticker_time;
-		_frame_count=0;
-	}
-	int cur_frame=(int)(((float)ticker_time-_start_time)*_fps/1000.0);
-	if (cur_frame>_frame_count){
-		_frame_count++;
-		return true;
-	}
-	return false;
+bool_t DSCapture::isTimeToSend(uint64_t ticker_time){
+	return ms_video_capture_new_frame(&framerate_controller, ticker_time);
 }
 
 
@@ -605,6 +597,8 @@ static void dscap_process(MSFilter * obj){
 static int dscap_set_fps(MSFilter *f, void *arg){
 	DSCapture *s=(DSCapture*)f->data;
 	s->setFps(*(float*)arg);
+	ms_video_init_framerate_controller(&s->framerate_controller, *(float*)arg);
+	ms_average_fps_init(&s->avgfps,"msdscap: fps=%f");
 	return 0;
 }
 
@@ -688,14 +682,28 @@ MSWebCamDesc ms_dshow_cam_desc={
 static void ms_dshow_detect(MSWebCamManager *obj){
 	SharedComPtr<IPropertyBag> pBag;
 	
+#ifdef ENABLE_MICROSOFT_STORE_APP
+	CoInitializeEx(NULL, COINIT_MULTITHREADED);
+#else
 	CoInitialize(NULL);
- 
+#endif
+
+#ifdef 	ENABLE_MICROSOFT_STORE_APP
+	ICreateDevEnum *createDevEnum;
+	if (!SUCCEEDED(CoCreateInstance(CLSID_SystemDeviceEnum, NULL,
+					CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&createDevEnum))))
+	{
+		ms_error( "Could not create device enumerator" );
+		return ;
+	}
+#else
 	SharedComPtr< ICreateDevEnum > createDevEnum;
 	if ((createDevEnum=makeShared<ICreateDevEnum>( CLSID_SystemDeviceEnum,
                                     IID_ICreateDevEnum))==NULL){
 		ms_error( "Could not create device enumerator" );
 		return ;
 	}
+#endif
  	SharedComPtr< IEnumMoniker > enumMoniker;
 	if (createDevEnum->CreateClassEnumerator( CLSID_VideoInputDeviceCategory, &enumMoniker, 0 )!=S_OK){
 		ms_error("Fail to create class enumerator.");
