@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "mediastreamer-config.h"
@@ -53,18 +53,20 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #endif
 
-
+/*
 static void inc_ref(mblk_t*m){
 	dblk_ref(m->b_datap);
 	if (m->b_cont)
 		inc_ref(m->b_cont);
 }
 
+
 static void dec_ref(mblk_t *m){
 	if (m->b_cont)
 		dec_ref(m->b_cont);
 	dblk_unref(m->b_datap);
 }
+*/
 
 typedef struct V4l2State{
 	int fd;
@@ -75,7 +77,7 @@ typedef struct V4l2State{
 	char *dev;
 	char *mmapdbuf;
 	int msize;/*mmapped size*/
-	MSVideoSize vsize;
+	MSVideoSize requested_vsize, used_vsize;
 	MSVideoSize got_vsize;
 	int pix_fmt;
 	int picture_size;
@@ -83,11 +85,13 @@ typedef struct V4l2State{
 	int frame_ind;
 	int frame_max;
 	float fps;
-	unsigned int start_time;
 	MSAverageFPS avgfps;
-	int th_frame_count;
 	int queued;
 	bool_t configured;
+	MSFrameRateController framerate_controller;
+	MSYuvBufAllocator* buf_allocator;
+	int rotation;
+	bool_t use_rotation;
 }V4l2State;
 
 static int msv4l2_open(V4l2State *s){
@@ -201,7 +205,7 @@ static MSPixFmt v4l2_format_to_ms(int v4l2format) {
 	}
 }
 
-static const V4L2FormatDescription* query_format_description_for_size(int fd, MSVideoSize vsize) {
+static const V4L2FormatDescription* query_format_description_for_size(int fd, MSVideoSize vsize, bool_t use_rotation) {
 	/* hardcode supported format in preferred order*/
 	static V4L2FormatDescription formats[POSSIBLE_FORMATS_COUNT];
 	int i=0;
@@ -211,27 +215,31 @@ static const V4L2FormatDescription* query_format_description_for_size(int fd, MS
 	formats[i].pixel_format = V4L2_PIX_FMT_YUV420;
 	formats[i].max_fps = -1;
 	i++;
-	/* we must avoid YUYV (and actually any YUV format different than YUV420P) because the pixel converter/scaler implementation
-	 * of ffmpeg is not optimized for arm. So we need to prefer YUV420P if directly available or MJPEG*/
+	
+	/* We force the use of YUV420P when rotation is enabled */
+	if (!use_rotation) {
+		/* we must avoid YUYV (and actually any YUV format different than YUV420P) because the pixel converter/scaler implementation
+		* of ffmpeg is not optimized for arm. So we need to prefer YUV420P if directly available or MJPEG*/
 #ifndef __arm__
-	formats[i].pixel_format = V4L2_PIX_FMT_YUYV;
-	formats[i].max_fps = -1;
-	i++;
+		formats[i].pixel_format = V4L2_PIX_FMT_YUYV;
+		formats[i].max_fps = -1;
+		i++;
 #endif
 
-	formats[i].pixel_format = V4L2_PIX_FMT_MJPEG;
-	formats[i].max_fps = -1;
-	i++;
+		formats[i].pixel_format = V4L2_PIX_FMT_MJPEG;
+		formats[i].max_fps = -1;
+		i++;
 
 #ifdef __arm__
-	formats[i].pixel_format = V4L2_PIX_FMT_YUYV;
-	formats[i].max_fps = -1;
-	i++;
+		formats[i].pixel_format = V4L2_PIX_FMT_YUYV;
+		formats[i].max_fps = -1;
+		i++;
 #endif
 
-	formats[i].pixel_format = V4L2_PIX_FMT_RGB24;
-	formats[i].max_fps = -1;
-	i++;
+		formats[i].pixel_format = V4L2_PIX_FMT_RGB24;
+		formats[i].max_fps = -1;
+		i++;
+	}
 
 	{
 		struct v4l2_fmtdesc fmt;
@@ -404,45 +412,49 @@ static int msv4l2_configure(V4l2State *s){
 	if (v4l2_ioctl (s->fd, VIDIOC_G_FMT, &fmt)<0){
 		ms_error("VIDIOC_G_FMT failed: %s",strerror(errno));
 	}
-	vsize=s->vsize;
+	vsize=s->requested_vsize;
 
 	do{
-		const V4L2FormatDescription* formats_desc = query_format_description_for_size(s->fd, s->vsize);
-		s->pix_fmt = pick_best_format(s->fd, formats_desc, s->vsize, s->fps);
+		const V4L2FormatDescription* formats_desc = query_format_description_for_size(s->fd, s->requested_vsize, s->use_rotation);
+		s->pix_fmt = pick_best_format(s->fd, formats_desc, s->requested_vsize, s->fps);
 
 		if (s->pix_fmt == MS_PIX_FMT_UNKNOWN)
-			s->vsize=ms_video_size_get_just_lower_than(s->vsize);
-	} while(s->vsize.width!=0 && (s->pix_fmt == MS_PIX_FMT_UNKNOWN));
+			s->requested_vsize=ms_video_size_get_just_lower_than(s->requested_vsize);
+	} while(s->requested_vsize.width!=0 && (s->pix_fmt == MS_PIX_FMT_UNKNOWN));
 
-	if (s->vsize.width==0){
+	if (s->requested_vsize.width==0){
 		ms_message("Could not find any combination of resolution/pixel-format that works !");
-		s->vsize=vsize;
+		s->requested_vsize=vsize;
 		ms_message("Fallback. Trying to force YUV420 format");
 		memset(&fmt, 0, sizeof(fmt));
 		fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 		fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUV420;
-		fmt.fmt.pix.width = s->vsize.width;
-		fmt.fmt.pix.height = s->vsize.height;
+		fmt.fmt.pix.width = s->requested_vsize.width;
+		fmt.fmt.pix.height = s->requested_vsize.height;
 		fmt.fmt.pix.field = V4L2_FIELD_ANY;
 		if(v4l2_ioctl(s->fd, VIDIOC_S_FMT, &fmt) != 0) {
-			ms_error("VIDIOC_S_FMT failed: %s", strerror(errno));
-			return -1;
+			ms_error("VIDIOC_S_FMT failed: %s. Read-only driver maybe ?", strerror(errno));
 		}
-		s->pix_fmt = v4l2_format_to_ms(fmt.fmt.pix.pixelformat);
+		
 	}
 
 	memset(&fmt,0,sizeof(fmt));
 	fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
 	if (v4l2_ioctl (s->fd, VIDIOC_G_FMT, &fmt)<0){
-		ms_error("VIDIOC_G_FMT failed: %s",strerror(errno));
+		ms_error("VIDIOC_G_FMT failed: %s. Irrecoverable error, video capture will not work.",strerror(errno));
+		return -1;
 	}else{
-		ms_message("Size of webcam delivered pictures is %ix%i. Format:0x%08x",fmt.fmt.pix.width,fmt.fmt.pix.height, s->pix_fmt);
-		s->vsize.width=fmt.fmt.pix.width;
-		s->vsize.height=fmt.fmt.pix.height;
+		s->requested_vsize.width=fmt.fmt.pix.width;
+		s->requested_vsize.height=fmt.fmt.pix.height;
+		s->pix_fmt = v4l2_format_to_ms(fmt.fmt.pix.pixelformat);
+		ms_message("Size of webcam delivered pictures is %ix%i. Format:0x%08x (%s)",fmt.fmt.pix.width,fmt.fmt.pix.height, s->pix_fmt, ms_pix_fmt_to_string(s->pix_fmt));
 	}
-	s->picture_size=get_picture_buffer_size(s->pix_fmt,s->vsize.width,s->vsize.height);
-
+	s->picture_size=get_picture_buffer_size(s->pix_fmt,s->requested_vsize.width,s->requested_vsize.height);
+	if (s->used_vsize.width < s->used_vsize.height){
+		s->used_vsize.width = s->requested_vsize.height;
+		s->used_vsize.height = s->requested_vsize.width;
+	}else s->used_vsize = s->requested_vsize;
 	focus=getenv("MS2_CAM_FOCUS");
 	if (focus){
 		if (strcasecmp(focus,"auto")==0){
@@ -508,7 +520,7 @@ static int msv4l2_do_mmap(V4l2State *s){
 		}
 		msg=esballoc(start,buf.length,0,NULL);
 		msg->b_wptr+=buf.length;
-		s->frames[i]=ms_yuv_buf_alloc_from_buffer(s->vsize.width, s->vsize.height, msg);
+		s->frames[i]=ms_yuv_buf_alloc_from_buffer(s->requested_vsize.width, s->requested_vsize.height, msg);
 	}
 	s->frame_max=req.count;
 	for (i = 0; i < s->frame_max; ++i) {
@@ -521,8 +533,9 @@ static int msv4l2_do_mmap(V4l2State *s){
 		if (-1==v4l2_ioctl (s->fd, VIDIOC_QBUF, &buf)){
 			ms_error("VIDIOC_QBUF failed: %s",strerror(errno));
 		}else {
-			inc_ref(s->frames[i]);
 			s->queued++;
+			/* We internaly use here the marker bit to mark buffers that are queued into the V4L2 driver. */
+			mblk_set_marker_info(s->frames[i], TRUE);
 		}
 	}
 	/*start capture immediately*/
@@ -564,7 +577,7 @@ static mblk_t *v4l2_dequeue_ready_buffer(V4l2State *s, int poll_timeout_ms){
 			ms_debug("v4l2: de-queue buf %i",buf.index);
 			/*decrement ref count of dequeued buffer */
 			ret=s->frames[buf.index];
-			dec_ref(ret);
+			
 			if ((int)buf.index >= s->frame_max){
 				ms_error("buf.index>=s->max_frames !");
 				return NULL;
@@ -573,6 +586,9 @@ static mblk_t *v4l2_dequeue_ready_buffer(V4l2State *s, int poll_timeout_ms){
 				ms_warning("Ignoring empty buffer...");
 				return NULL;
 			}
+			/* Remove marker bit to indicate that the buffer is no longer queued to the V4L2 driver. */
+			mblk_set_marker_info(ret, FALSE);
+			
 			/*normally buf.bytesused should contain the right buffer size; however we have found a buggy
 			driver that puts a random value inside */
 			if (s->picture_size!=0)
@@ -594,19 +610,22 @@ static mblk_t * v4lv2_grab_image(V4l2State *s, int poll_timeout_ms){
 	buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 	buf.memory = V4L2_MEMORY_MMAP;
 
-	/*queue buffers whose ref count is 1, because they are not
-	 used anywhere in the filter chain */
+	/* Queue buffers whose ref count is 1 and marker bit is FALSE, which means respectively:
+	 * - they are no longer used anywhere and in the chain
+	 * - they are not queued already in the V4L2 driver.
+	 */
 	for(k=0;k<s->frame_max;++k){
-		if (dblk_ref_value(s->frames[k]->b_datap)==1){
+		if (dblk_ref_value(s->frames[k]->b_datap)==1 && mblk_get_marker_info(s->frames[k]) == FALSE ){
 			no_slot_available = FALSE;
 			buf.index=k;
 			if (-1==v4l2_ioctl (s->fd, VIDIOC_QBUF, &buf))
 				ms_warning("VIDIOC_QBUF %i failed: %s",k,  strerror(errno));
 			else {
-				/*increment ref count of queued buffer*/
-				inc_ref(s->frames[k]);
 				s->queued++;
+				/* We internaly use here the marker bit to mark buffers that are queued into the V4L2 driver. */
+				mblk_set_marker_info(s->frames[k], TRUE);
 			}
+			
 		}
 	}
 
@@ -642,13 +661,24 @@ static void msv4l2_do_munmap(V4l2State *s){
 
 static void msv4l2_init(MSFilter *f){
 	V4l2State *s=ms_new0(V4l2State,1);
+	char* tmp=NULL;
+	
 	s->dev=ms_strdup("/dev/video0");
 	s->fd=-1;
-	s->vsize=MS_VIDEO_SIZE_CIF;
+	s->requested_vsize=MS_VIDEO_SIZE_CIF;
 	s->fps=15;
 	s->configured=FALSE;
 	f->data=s;
 	qinit(&s->rq);
+	
+	tmp=getenv("MS2_V4L2_USE_ROTATION");
+	if (tmp != NULL && (strcmp("1", tmp) == 0)) {
+		s->use_rotation=TRUE;
+		s->rotation=0;
+		s->buf_allocator=ms_yuv_buf_allocator_new();
+	} else {
+		s->use_rotation=FALSE;
+	}
 }
 
 static void msv4l2_uninit(MSFilter *f){
@@ -656,6 +686,7 @@ static void msv4l2_uninit(MSFilter *f){
 	ms_free(s->dev);
 	flushq(&s->rq,0);
 	ms_mutex_destroy(&s->mutex);
+	if (s->use_rotation) ms_yuv_buf_allocator_free(s->buf_allocator);
 	ms_free(s);
 }
 
@@ -689,10 +720,9 @@ static void *msv4l2_thread(void *ptr){
 			mblk_t *m;
 			m=v4lv2_grab_image(s,50);
 			if (m){
-				mblk_t *om=dupmsg(m);
-				mblk_set_marker_info(om,(s->pix_fmt==MS_MJPEG));
+				mblk_t *om = dupmsg(m);
 				ms_mutex_lock(&s->mutex);
-				putq(&s->rq,om);
+				putq(&s->rq, om);
 				ms_mutex_unlock(&s->mutex);
 			}
 		}
@@ -714,29 +744,42 @@ close:
 	return NULL;
 }
 
+static mblk_t *msv4l2_rotate_image(V4l2State *s, mblk_t *frame) {
+	mblk_t *rotated_frame;
+	YuvBuf buf;
+	
+	ms_yuv_buf_init_from_mblk(&buf, frame);
+		
+	rotated_frame = copy_yuv_with_rotation(s->buf_allocator
+										   , buf.planes[0]
+										   , buf.planes[1]
+										   , buf.planes[2]
+										   , s->rotation
+										   , s->used_vsize.width
+										   , s->used_vsize.height
+										   , buf.strides[0]
+										   , buf.strides[1]
+										   , buf.strides[2]);
+	
+	if (rotated_frame) {
+		freemsg(frame);
+		return rotated_frame;
+	}
+	return frame;
+}
+
 static void msv4l2_preprocess(MSFilter *f){
 	V4l2State *s=(V4l2State*)f->data;
 	s->thread_run=TRUE;
 	ms_thread_create(&s->thread,NULL,msv4l2_thread,s);
-	s->th_frame_count=-1;
 	ms_average_fps_init(&s->avgfps,"V4L2 capture: fps=%f");
 }
 
 static void msv4l2_process(MSFilter *f){
 	V4l2State *s=(V4l2State*)f->data;
 	uint32_t timestamp;
-	int cur_frame;
-	uint32_t curtime=f->ticker->time;
-	float elapsed;
 
-	if (s->th_frame_count==-1){
-		s->start_time=curtime;
-		s->th_frame_count=0;
-	}
-	elapsed=((float)(curtime-s->start_time))/1000.0;
-	cur_frame=elapsed*s->fps;
-
-	if (cur_frame>=s->th_frame_count){
+	if (ms_video_capture_new_frame(&s->framerate_controller, f->ticker->time)){
 		mblk_t *om=NULL;
 		ms_mutex_lock(&s->mutex);
 		/*keep the most recent frame if several frames have been captured */
@@ -749,13 +792,18 @@ static void msv4l2_process(MSFilter *f){
 		}
 		ms_mutex_unlock(&s->mutex);
 		if (om!=NULL){
+			if (s->use_rotation && s->rotation > 0) {
+				om = msv4l2_rotate_image(s, om);
+			}
 			timestamp=f->ticker->time*90;/* rtp uses a 90000 Hz clockrate for video*/
 			mblk_set_timestamp_info(om,timestamp);
-			mblk_set_marker_info(om,TRUE);
+			/* When pushing the buffer outside, the marker bit must have its common meaning, ie for encoded formats 
+			 * (such as MJPEG) signal the buffer that has the end of a frame.
+			 */
+			mblk_set_marker_info(om, (s->pix_fmt==MS_MJPEG));
 			ms_queue_put(f->outputs[0],om);
 			ms_average_fps_update(&s->avgfps,f->ticker->time);
 		}
-		s->th_frame_count++;
 	}
 }
 
@@ -777,13 +825,23 @@ static void msv4l2_postprocess(MSFilter *f){
 static int msv4l2_set_fps(MSFilter *f, void *arg){
 	V4l2State *s=(V4l2State*)f->data;
 	s->fps=*(float*)arg;
+	ms_video_init_framerate_controller(&s->framerate_controller, s->fps);
+	ms_average_fps_init(&s->avgfps,"V4L2 capture: fps=%f");
 	return 0;
 }
 
 static int msv4l2_set_vsize(MSFilter *f, void *arg){
 	V4l2State *s=(V4l2State*)f->data;
-	s->vsize=*(MSVideoSize*)arg;
+	s->requested_vsize=*(MSVideoSize*)arg;
 	s->configured=FALSE;
+	
+	if (s->use_rotation && (s->rotation == 90 || s->rotation == 270)) {
+		s->used_vsize.width = s->requested_vsize.height;
+		s->used_vsize.height = s->requested_vsize.width;
+	} else {
+		s->used_vsize = s->requested_vsize;
+	}
+		
 	return 0;
 }
 
@@ -801,7 +859,7 @@ static int msv4l2_check_configured(V4l2State *s){
 static int msv4l2_get_vsize(MSFilter *f, void *arg){
 	V4l2State *s=(V4l2State*)f->data;
 	msv4l2_check_configured(s);
-	*(MSVideoSize*)arg=s->vsize;
+	*(MSVideoSize*)arg=s->used_vsize;
 	return 0;
 }
 
@@ -827,12 +885,25 @@ static int msv4l2_get_fps(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int ms4vl2_set_device_orientation(MSFilter *f, void *arg) {
+	V4l2State *s=(V4l2State*)f->data;
+	
+	if (s->use_rotation) {
+		s->rotation = *((int*)arg);
+	} else {
+		ms_warning("msv4l2: set_device_orientation was called while env MS2_USE_ROTATION is not set.");
+	}
+		
+	return 0;
+}
+
 static MSFilterMethod msv4l2_methods[]={
 	{	MS_FILTER_SET_FPS	,	msv4l2_set_fps	},
 	{	MS_FILTER_SET_VIDEO_SIZE,	msv4l2_set_vsize	},
 	{	MS_FILTER_GET_VIDEO_SIZE,	msv4l2_get_vsize	},
 	{	MS_FILTER_GET_PIX_FMT	,	msv4l2_get_pixfmt	},
 	{	MS_FILTER_GET_FPS	,	msv4l2_get_fps	},
+	{	MS_VIDEO_CAPTURE_SET_DEVICE_ORIENTATION	,	ms4vl2_set_device_orientation},
 	{	0			,	NULL		}
 };
 
@@ -895,18 +966,12 @@ static void msv4l2_detect(MSWebCamManager *obj){
 #else
 				camera_caps = V4L2_CAP_VIDEO_CAPTURE;
 #endif
-				if (((camera_caps & V4L2_CAP_VIDEO_CAPTURE)
-#ifdef V4L2_CAP_VIDEO_CAPTURE_MPLANE
-					|| (camera_caps & V4L2_CAP_VIDEO_CAPTURE_MPLANE)
-#endif
-					) && !((camera_caps & V4L2_CAP_VIDEO_OUTPUT)
-#ifdef V4L2_CAP_VIDEO_OUTPUT_MPLANE
-					|| (camera_caps & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
-#endif
-					)) {
+				if (camera_caps & V4L2_CAP_VIDEO_CAPTURE) {
 					MSWebCam *cam=ms_web_cam_new(&v4l2_card_desc);
 					cam->name=ms_strdup(devname);
 					ms_web_cam_manager_add_cam(obj,cam);
+				}else{
+					ms_message("Ignored %s, not a capture device.", devname);
 				}
 			}
 			close(fd);
