@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "mediastreamer-config.h"
@@ -661,6 +661,7 @@ static mblk_t *get_as_yuvmsg(MSFilter *f, DecState *s, AVFrame *orig){
 #endif
 		ms_error("%s: error in ms_sws_scale().",f->desc->name);
 	}
+	mblk_set_timestamp_info(yuv_msg, (uint32_t)orig->pkt_pts);
 	return yuv_msg;
 }
 /* Bitmasks to select bits of a byte from low side */
@@ -705,13 +706,14 @@ static void dec_process_frame(MSFilter *f, mblk_t *inm){
 			msgpullup(s->input,msgdsize(s->input)+8);
 			frame=s->input;
 			s->input=NULL;
+			uint32_t frame_ts = mblk_get_timestamp_info(frame);
 			while ( (remain=frame->b_wptr-frame->b_rptr)> 0) {
 				AVPacket pkt;
 				
 				av_init_packet(&pkt);
 				pkt.data = frame->b_rptr;
 				pkt.size = remain;
-
+				pkt.pts = frame_ts;
 				len=avcodec_decode_video2(&s->av_context, s->orig, &got_picture,&pkt);
 
 				if (len<=0) {
