@@ -1,28 +1,28 @@
- /*
- * vp8.c -VP8 encoder/decoder wrapper
+/*
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
  *
+ * This file is part of mediastreamer2.
  *
- * Copyright (C) 2011  Belledonne Communications, Grenoble, France
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msticker.h"
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/mscodecutils.h"
+#include "mediastreamer2/msasync.h"
+#include "mediastreamer2/msqueue.h"
 #include "vp8rtpfmt.h"
 
 #define PICTURE_ID_ON_16_BITS
@@ -32,6 +32,9 @@
 #include <vpx/vpx_encoder.h>
 #include <vpx/vp8cx.h>
 
+#ifdef __APPLE__
+   #include "TargetConditionals.h"
+#endif
 
 #define PICID_NEWER_THAN(s1,s2)	( (uint16_t)((uint16_t)s1-(uint16_t)s2) < 1<<15)
 
@@ -40,31 +43,32 @@
 
 static const MSVideoConfiguration vp8_conf_list[] = {
 #if defined(__ANDROID__) || (TARGET_OS_IPHONE == 1) || defined(__arm__) || defined(_M_ARM)
-	MS_VP8_CONF(2048000, 2560000,       UXGA, 12, 2),
-	MS_VP8_CONF(1024000, 1536000, SXGA_MINUS, 12, 2),
-	MS_VP8_CONF( 750000, 1024000,        XGA, 12, 2),
-	MS_VP8_CONF( 500000,  750000,       SVGA, 12, 2),
+	MS_VP8_CONF(1536000, 3000000,       720P, 25, 8),
+	MS_VP8_CONF(1024000, 1536000,        XGA, 25, 8),
+	MS_VP8_CONF( 850000, 1024000,       SVGA, 25, 8),
+	MS_VP8_CONF( 700000, 3000000,        VGA, 30, 8),
 	MS_VP8_CONF( 600000, 3000000,        VGA, 25, 4),
-	MS_VP8_CONF( 300000,  500000,        VGA, 12, 2),
+	MS_VP8_CONF( 400000,  800000,        VGA, 12, 2),
 	MS_VP8_CONF( 100000,  300000,       QVGA, 18, 2),
 	MS_VP8_CONF(  64000,  100000,       QCIF, 12, 2),
-	MS_VP8_CONF(300000, 600000,          VGA, 12, 1),
-	MS_VP8_CONF(100000, 300000,         QVGA, 10, 1),
-	MS_VP8_CONF( 64000, 100000,         QCIF, 10, 1),
-	MS_VP8_CONF(      0,   64000,       QCIF,  5, 1)
+	MS_VP8_CONF( 400000,  800000,        VGA, 12, 1),
+	MS_VP8_CONF( 100000,  300000,       QVGA, 10, 1),
+	MS_VP8_CONF(  64000,  100000,       QCIF, 10, 1),
+	MS_VP8_CONF(      0,   64000,       QCIF,  5, 1),
 #else
-	MS_VP8_CONF(1536000,  2560000, SXGA_MINUS, 25, 4),
-	MS_VP8_CONF(800000,  2000000,       720P, 25, 4),
-	MS_VP8_CONF(800000,  1536000,        XGA, 25, 4),
-	MS_VP8_CONF( 600000,  1024000,       SVGA, 25, 2),
+//	MS_VP8_CONF(2048000,  5000000,       UXGA, 25, 4),
+//	MS_VP8_CONF(1536000,  2560000, SXGA_MINUS, 25, 4),
+	MS_VP8_CONF(1536000,  3000000,       720P, 25, 8),
+	MS_VP8_CONF( 1024000,  3000000,        XGA, 25, 8),
+	MS_VP8_CONF( 850000,  3000000,       SVGA, 25, 4),
 	MS_VP8_CONF( 600000,  3000000,        VGA, 30, 2),
-	MS_VP8_CONF( 350000,   600000,        VGA, 25, 2),
+	MS_VP8_CONF( 450000,   600000,        VGA, 25, 2),
 	MS_VP8_CONF( 350000,   600000,        VGA, 15, 1),
 	MS_VP8_CONF( 200000,   350000,        CIF, 18, 1),
 	MS_VP8_CONF( 150000,   200000,       QVGA, 15, 1),
 	MS_VP8_CONF( 100000,   150000,       QVGA, 10, 1),
 	MS_VP8_CONF(  64000,   100000,       QCIF, 12, 1),
-	MS_VP8_CONF(      0,    64000,       QCIF,  5 ,1)
+	MS_VP8_CONF(      0,    64000,       QCIF,  5 ,1),
 #endif
 };
 
@@ -101,6 +105,9 @@ typedef struct EncState {
 	bool_t invalid_frame_reported;
 	bool_t avpf_enabled;
 	bool_t ready;
+	MSWorkerThread *process_thread;
+	queue_t entry_q;
+	MSQueue *exit_q;
 } EncState;
 
 #define MIN_KEY_FRAME_DIST 4 /*since one i-frame is allowed to be 4 times bigger of the target bitrate*/
@@ -175,6 +182,9 @@ static void enc_preprocess(MSFilter *f) {
 	}
 #if TARGET_IPHONE_SIMULATOR
 	s->cfg.g_threads = 1; /*workaround to remove crash on ipad simulator*/
+#elif TARGET_OS_OSX
+	//On macosx 10.14 realtime processing is not ensured on dual core machine with ms_factory_get_cpu_count() <= 4. Value belows is a tradeoff between scalability and realtime
+	s->cfg.g_threads= MAX(ms_factory_get_cpu_count(f->factory)-2,1);
 #else
 	s->cfg.g_threads = ms_factory_get_cpu_count(f->factory);
 #endif
@@ -187,7 +197,13 @@ static void enc_preprocess(MSFilter *f) {
 #if defined(__ANDROID__) || (TARGET_OS_IPHONE == 1) || defined(__arm__) || defined(_M_ARM)
 	cpuused = 10 - s->cfg.g_threads; /*cpu/quality tradeoff: positive values decrease CPU usage at the expense of quality*/
 	if (cpuused < 7) cpuused = 7; /*values beneath 7 consume too much CPU*/
-	if( s->cfg.g_threads == 1 ){
+	if (ms_video_size_area_greater_than(MS_VIDEO_SIZE_VGA, s->vconf.vsize)){
+		if (s->cfg.g_threads <= 2) cpuused = 8;
+		else if (s->cfg.g_threads <= 4) cpuused = 5;
+		else cpuused = 1;
+
+	}
+	if (s->cfg.g_threads == 1){
 		/* on mono-core iOS devices, we reduce the quality a bit more due to VP8 being slower with new Clang compilers */
 		cpuused = 16;
 	}
@@ -219,6 +235,11 @@ static void enc_preprocess(MSFilter *f) {
 	} else if (s->frame_count == 0) {
 		ms_video_starter_init(&s->starter);
 	}
+
+	s->process_thread = ms_worker_thread_new();
+	qinit(&s->entry_q);
+	s->exit_q = ms_queue_new(0, 0, 0, 0);
+
 	s->ready = TRUE;
 }
 
@@ -426,15 +447,171 @@ static bool_t is_frame_independent(unsigned int flags){
 	return FALSE;
 }
 
-static void enc_process(MSFilter *f) {
-	mblk_t *im;
-	uint64_t timems = f->ticker->time;
-	uint32_t timestamp = (uint32_t)(timems*90);
+static void enc_process_frame_task(void *obj) {
+	mblk_t *im, *prev_im = NULL;
+	MSFilter *f = (MSFilter*)obj;
 	EncState *s = (EncState *)f->data;
 	unsigned int flags = 0;
 	vpx_codec_err_t err;
 	MSPicture yuv;
 	bool_t is_ref_frame=FALSE;
+	vpx_image_t img;
+	int skipped_count = 0;
+
+	ms_filter_lock(f);
+	while ((im = getq(&s->entry_q)) != NULL) {
+		if (prev_im) {
+			freemsg(prev_im);
+			skipped_count++;
+		}
+		prev_im = im;
+	}
+	if (!im) {
+		im = prev_im;
+	}
+	
+	ms_filter_unlock(f);
+
+	if (skipped_count > 0){
+		/*don't let too much buffers to be queued here, it makes no sense for a real time processing and would consume too much memory*/
+		ms_warning("VP8 async encoding process: %i frames skipped", skipped_count);
+	}
+	if (!im){
+		ms_message("VP8 async encoding process: no frame to encode, probably skipped by previous task");
+		return;
+	}
+
+	flags = 0;
+	ms_yuv_buf_init_from_mblk(&yuv, im);
+	vpx_img_wrap(&img, VPX_IMG_FMT_I420, s->vconf.vsize.width, s->vconf.vsize.height, 1, yuv.planes[0]);
+
+	if ((s->avpf_enabled != TRUE) && ms_video_starter_need_i_frame(&s->starter, f->ticker->time)) {
+		s->force_keyframe = TRUE;
+	}
+	if (s->force_keyframe == TRUE) {
+		ms_message("Forcing vp8 key frame for filter [%p]", f);
+		flags = VPX_EFLAG_FORCE_KF;
+	} else if (s->avpf_enabled == TRUE) {
+		if (s->frame_count == 0) s->force_keyframe = TRUE;
+		enc_fill_encoder_flags(s, &flags);
+	}
+
+#ifdef AVPF_DEBUG
+	ms_message("VP8 encoder frames state:");
+	ms_message("\tgolden: count=%" PRIi64 ", picture_id=0x%04x, ack=%s",
+		s->frames_state.golden.count, s->frames_state.golden.picture_id, (s->frames_state.golden.acknowledged == TRUE) ? "Y" : "N");
+	ms_message("\taltref: count=%" PRIi64 ", picture_id=0x%04x, ack=%s",
+		s->frames_state.altref.count, s->frames_state.altref.picture_id, (s->frames_state.altref.acknowledged == TRUE) ? "Y" : "N");
+#endif
+	err = vpx_codec_encode(&s->codec, &img, s->frame_count, 1, flags, (unsigned long)((double)1000000/(2.0*(double)s->vconf.fps))); /*encoder has half a framerate interval to encode*/
+	if (err) {
+		ms_error("vpx_codec_encode failed : %d %s (%s)\n", err, vpx_codec_err_to_string(err), vpx_codec_error_detail(&s->codec));
+	} else {
+		vpx_codec_iter_t iter = NULL;
+		const vpx_codec_cx_pkt_t *pkt;
+		bctbx_list_t *list = NULL;
+		int current_partition_id = -1;
+
+		/* Update the frames state. */
+		is_ref_frame=FALSE;
+		if (flags & VPX_EFLAG_FORCE_KF) {
+			enc_mark_reference_frame_as_sent(s, VP8_GOLD_FRAME);
+			enc_mark_reference_frame_as_sent(s, VP8_ALTR_FRAME);
+			s->frames_state.golden.is_independant=TRUE;
+			s->frames_state.altref.is_independant=TRUE;
+			s->frames_state.last_independent_frame=s->frame_count;
+			s->force_keyframe = FALSE;
+			is_ref_frame=TRUE;
+		}else if (flags & VP8_EFLAG_FORCE_GF) {
+			enc_mark_reference_frame_as_sent(s, VP8_GOLD_FRAME);
+			is_ref_frame=TRUE;
+		}else if (flags & VP8_EFLAG_FORCE_ARF) {
+			enc_mark_reference_frame_as_sent(s, VP8_ALTR_FRAME);
+			is_ref_frame=TRUE;
+		}else if (flags & VP8_EFLAG_NO_REF_LAST) {
+			enc_mark_reference_frame_as_sent(s, VP8_LAST_FRAME);
+			is_ref_frame=is_reconstruction_frame_sane(s,flags);
+		}
+		if (is_frame_independent(flags)){
+			s->frames_state.last_independent_frame=s->frame_count;
+		}
+
+		/* Pack the encoded frame. */
+		while ((pkt = vpx_codec_get_cx_data(&s->codec, &iter))) {
+			if ((pkt->kind == VPX_CODEC_CX_FRAME_PKT) && (pkt->data.frame.sz > 0)) {
+				Vp8RtpFmtPacket *packet = ms_new0(Vp8RtpFmtPacket, 1);
+
+				packet->m = allocb(pkt->data.frame.sz, 0);
+				memcpy(packet->m->b_wptr, pkt->data.frame.buf, pkt->data.frame.sz);
+				packet->m->b_wptr += pkt->data.frame.sz;
+				mblk_set_timestamp_info(packet->m, mblk_get_timestamp_info(im));
+				packet->pd = ms_new0(Vp8RtpFmtPayloadDescriptor, 1);
+				packet->pd->non_reference_frame = s->avpf_enabled && !is_ref_frame;
+				if (s->avpf_enabled == TRUE) {
+					packet->pd->extended_control_bits_present = TRUE;
+					packet->pd->pictureid_present = TRUE;
+					packet->pd->pictureid = s->picture_id;
+				} else {
+					packet->pd->extended_control_bits_present = FALSE;
+					packet->pd->pictureid_present = FALSE;
+				}
+				if (s->flags & VPX_CODEC_USE_OUTPUT_PARTITION) {
+					if (pkt->data.frame.partition_id != current_partition_id) {
+						current_partition_id = pkt->data.frame.partition_id;
+						packet->pd->start_of_partition = TRUE;
+					}
+					packet->pd->pid = (uint8_t)pkt->data.frame.partition_id;
+					if (!(pkt->data.frame.flags & VPX_FRAME_IS_FRAGMENT)) {
+						mblk_set_marker_info(packet->m, TRUE);
+					}
+				} else {
+					packet->pd->pid = 0;
+					if (current_partition_id != 0) {
+						packet->pd->start_of_partition = TRUE;
+						current_partition_id = 0;
+					}
+					mblk_set_marker_info(packet->m, TRUE);
+				}
+				list = bctbx_list_append(list, packet);
+			}
+		}
+
+#ifdef AVPF_DEBUG
+		ms_message("VP8 encoder picture_id=%i ***| %s | %s | %s | %s", (int)s->picture_id,
+			(flags & VPX_EFLAG_FORCE_KF) ? "KF " : (flags & VP8_EFLAG_FORCE_GF) ? "GF " :  (flags & VP8_EFLAG_FORCE_ARF) ? "ARF" : "   ",
+			(flags & VP8_EFLAG_NO_REF_GF) ? "NOREFGF" : "       ",
+			(flags & VP8_EFLAG_NO_REF_ARF) ? "NOREFARF" : "        ",
+			(flags & VP8_EFLAG_NO_REF_LAST) ? "NOREFLAST" : "         ");
+#endif
+
+		ms_filter_lock(f);
+		vp8rtpfmt_packer_process(&s->packer, list, s->exit_q, f->factory);
+		ms_filter_unlock(f);
+
+		/* Handle video starter if AVPF is not enabled. */
+		s->frame_count++;
+		if ((s->avpf_enabled != TRUE) && (s->frame_count == 1)) {
+			ms_video_starter_first_frame(&s->starter, f->ticker->time);
+		}
+
+		/* Increment the pictureID. */
+		s->picture_id++;
+#ifdef PICTURE_ID_ON_16_BITS
+		if (s->picture_id == 0)
+			s->picture_id = 0x8000;
+#else
+		if (s->picture_id == 0x0080)
+			s->picture_id = 0;
+#endif
+	}
+
+	freemsg(im);
+}
+
+static void enc_process(MSFilter *f) {
+	EncState *s = (EncState *)f->data;
+	mblk_t *entry_f;
+	mblk_t *exit_f;
 
 	ms_filter_lock(f);
 
@@ -448,144 +625,52 @@ static void enc_process(MSFilter *f) {
 		return;
 	}
 
-	if ((im = ms_queue_peek_last(f->inputs[0])) != NULL) {
-		vpx_image_t img;
-
-		flags = 0;
-		ms_yuv_buf_init_from_mblk(&yuv, im);
-		vpx_img_wrap(&img, VPX_IMG_FMT_I420, s->vconf.vsize.width, s->vconf.vsize.height, 1, yuv.planes[0]);
-
-		if ((s->avpf_enabled != TRUE) && ms_video_starter_need_i_frame(&s->starter, f->ticker->time)) {
-			s->force_keyframe = TRUE;
-		}
-		if (s->force_keyframe == TRUE) {
-			ms_message("Forcing vp8 key frame for filter [%p]", f);
-			flags = VPX_EFLAG_FORCE_KF;
-		} else if (s->avpf_enabled == TRUE) {
-			if (s->frame_count == 0) s->force_keyframe = TRUE;
-			enc_fill_encoder_flags(s, &flags);
-		}
-
-#ifdef AVPF_DEBUG
-		ms_message("VP8 encoder frames state:");
-		ms_message("\tgolden: count=%" PRIi64 ", picture_id=0x%04x, ack=%s",
-			s->frames_state.golden.count, s->frames_state.golden.picture_id, (s->frames_state.golden.acknowledged == TRUE) ? "Y" : "N");
-		ms_message("\taltref: count=%" PRIi64 ", picture_id=0x%04x, ack=%s",
-			s->frames_state.altref.count, s->frames_state.altref.picture_id, (s->frames_state.altref.acknowledged == TRUE) ? "Y" : "N");
-#endif
-		err = vpx_codec_encode(&s->codec, &img, s->frame_count, 1, flags, 1000000LL/(2*(int)s->vconf.fps)); /*encoder has half a framerate interval to encode*/
-		if (err) {
-			ms_error("vpx_codec_encode failed : %d %s (%s)\n", err, vpx_codec_err_to_string(err), vpx_codec_error_detail(&s->codec));
-		} else {
-			vpx_codec_iter_t iter = NULL;
-			const vpx_codec_cx_pkt_t *pkt;
-			bctbx_list_t *list = NULL;
-
-			/* Update the frames state. */
-			is_ref_frame=FALSE;
-			if (flags & VPX_EFLAG_FORCE_KF) {
-				enc_mark_reference_frame_as_sent(s, VP8_GOLD_FRAME);
-				enc_mark_reference_frame_as_sent(s, VP8_ALTR_FRAME);
-				s->frames_state.golden.is_independant=TRUE;
-				s->frames_state.altref.is_independant=TRUE;
-				s->frames_state.last_independent_frame=s->frame_count;
-				s->force_keyframe = FALSE;
-				is_ref_frame=TRUE;
-			}else if (flags & VP8_EFLAG_FORCE_GF) {
-				enc_mark_reference_frame_as_sent(s, VP8_GOLD_FRAME);
-				is_ref_frame=TRUE;
-			}else if (flags & VP8_EFLAG_FORCE_ARF) {
-				enc_mark_reference_frame_as_sent(s, VP8_ALTR_FRAME);
-				is_ref_frame=TRUE;
-			}else if (flags & VP8_EFLAG_NO_REF_LAST) {
-				enc_mark_reference_frame_as_sent(s, VP8_LAST_FRAME);
-				is_ref_frame=is_reconstruction_frame_sane(s,flags);
-			}
-			if (is_frame_independent(flags)){
-				s->frames_state.last_independent_frame=s->frame_count;
-			}
-
-			/* Pack the encoded frame. */
-			while( (pkt = vpx_codec_get_cx_data(&s->codec, &iter)) ) {
-				if ((pkt->kind == VPX_CODEC_CX_FRAME_PKT) && (pkt->data.frame.sz > 0)) {
-					Vp8RtpFmtPacket *packet = ms_new0(Vp8RtpFmtPacket, 1);
-
-					packet->m = allocb(pkt->data.frame.sz, 0);
-					memcpy(packet->m->b_wptr, pkt->data.frame.buf, pkt->data.frame.sz);
-					packet->m->b_wptr += pkt->data.frame.sz;
-					mblk_set_timestamp_info(packet->m, timestamp);
-					packet->pd = ms_new0(Vp8RtpFmtPayloadDescriptor, 1);
-					packet->pd->start_of_partition = TRUE;
-					packet->pd->non_reference_frame = s->avpf_enabled && !is_ref_frame;
-					if (s->avpf_enabled == TRUE) {
-						packet->pd->extended_control_bits_present = TRUE;
-						packet->pd->pictureid_present = TRUE;
-						packet->pd->pictureid = s->picture_id;
-					} else {
-						packet->pd->extended_control_bits_present = FALSE;
-						packet->pd->pictureid_present = FALSE;
-					}
-					if (s->flags & VPX_CODEC_USE_OUTPUT_PARTITION) {
-						packet->pd->pid = (uint8_t)pkt->data.frame.partition_id;
-						if (!(pkt->data.frame.flags & VPX_FRAME_IS_FRAGMENT)) {
-							mblk_set_marker_info(packet->m, TRUE);
-						}
-					} else {
-						packet->pd->pid = 0;
-						mblk_set_marker_info(packet->m, TRUE);
-					}
-					list = bctbx_list_append(list, packet);
-				}
-			}
-
-#ifdef AVPF_DEBUG
-			ms_message("VP8 encoder picture_id=%i ***| %s | %s | %s | %s", (int)s->picture_id,
-				(flags & VPX_EFLAG_FORCE_KF) ? "KF " : (flags & VP8_EFLAG_FORCE_GF) ? "GF " :  (flags & VP8_EFLAG_FORCE_ARF) ? "ARF" : "   ",
-				(flags & VP8_EFLAG_NO_REF_GF) ? "NOREFGF" : "       ",
-				(flags & VP8_EFLAG_NO_REF_ARF) ? "NOREFARF" : "        ",
-				(flags & VP8_EFLAG_NO_REF_LAST) ? "NOREFLAST" : "         ");
-#endif
-
-			vp8rtpfmt_packer_process(&s->packer, list, f->outputs[0], f->factory);
-
-			/* Handle video starter if AVPF is not enabled. */
-			s->frame_count++;
-			if ((s->avpf_enabled != TRUE) && (s->frame_count == 1)) {
-				ms_video_starter_first_frame(&s->starter, f->ticker->time);
-			}
-
-			/* Increment the pictureID. */
-			s->picture_id++;
-#ifdef PICTURE_ID_ON_16_BITS
-			if (s->picture_id == 0)
-				s->picture_id = 0x8000;
-#else
-			if (s->picture_id == 0x0080)
-				s->picture_id = 0;
-#endif
-		}
+	/* If we have a frame then we put it in the entry queue and we call the task enc_process_frame_task in async */
+	if ((entry_f = ms_queue_peek_last(f->inputs[0])) != NULL) {
+		ms_queue_remove(f->inputs[0], entry_f);
+		putq(&s->entry_q, entry_f);
+		ms_worker_thread_add_task(s->process_thread, enc_process_frame_task, (void*)f);
 	}
+
+	/* Put each frame we have in exit_q in f->output[0] */
+	while ((exit_f = ms_queue_get(s->exit_q)) != NULL) {
+		ms_queue_put(f->outputs[0], exit_f);
+	}
+
 	ms_filter_unlock(f);
 	ms_queue_flush(f->inputs[0]);
 }
 
 static void enc_postprocess(MSFilter *f) {
 	EncState *s = (EncState *)f->data;
+	ms_worker_thread_destroy(s->process_thread, FALSE);
 	if (s->ready) vpx_codec_destroy(&s->codec);
 	vp8rtpfmt_packer_uninit(&s->packer);
+	flushq(&s->entry_q,0);
+	ms_queue_destroy(s->exit_q);
+	s->exit_q = NULL;
 	s->ready = FALSE;
 }
 
 static int enc_set_configuration(MSFilter *f, void *data) {
 	EncState *s = (EncState *)f->data;
 	const MSVideoConfiguration *vconf = (const MSVideoConfiguration *)data;
+	MSVideoSize vsize = s->vconf.vsize;
+
 	if (vconf != &s->vconf) memcpy(&s->vconf, vconf, sizeof(MSVideoConfiguration));
 
 	s->cfg.rc_target_bitrate = (unsigned int)(((float)s->vconf.required_bitrate) * 0.92f / 1024.0f); //0.92=take into account IP/UDP/RTP overhead, in average.
+	s->cfg.g_timebase.num = 1;
+	s->cfg.g_timebase.den = (int)s->vconf.fps;
 	if (s->ready) {
+		/* Do not change video size if encoder is running */
+		if (!ms_video_size_equal(s->vconf.vsize, vsize)) {
+			ms_warning("Video configuration: cannot change video size when encoder is running, actual=%dx%d, wanted=%dx%d", vsize.width, vsize.height, s->vconf.vsize.width, s->vconf.vsize.height);
+			s->vconf.vsize = vsize;
+		}
+
 		ms_filter_lock(f);
-		enc_postprocess(f);
-		enc_preprocess(f);
+		vpx_codec_enc_config_set(&s->codec, &s->cfg);
 		ms_filter_unlock(f);
 	}
 
@@ -594,58 +679,10 @@ static int enc_set_configuration(MSFilter *f, void *data) {
 	return 0;
 }
 
-static int enc_set_vsize(MSFilter *f, void *data) {
-	MSVideoConfiguration best_vconf;
-	MSVideoSize *vs = (MSVideoSize *)data;
+static int enc_get_configuration(MSFilter *f, void *data) {
 	EncState *s = (EncState *)f->data;
-	best_vconf = ms_video_find_best_configuration_for_size_and_bitrate(s->vconf_list, *vs, ms_factory_get_cpu_count(f->factory), s->vconf.required_bitrate);
-	s->vconf.vsize = *vs;
-	s->vconf.fps = best_vconf.fps;
-	s->vconf.bitrate_limit = best_vconf.bitrate_limit;
-	s->vconf.required_bitrate = best_vconf.bitrate_limit;
-	enc_set_configuration(f, &s->vconf);
-	return 0;
-}
-
-static int enc_get_vsize(MSFilter *f, void *data) {
-	EncState *s = (EncState *)f->data;
-	MSVideoSize *vs = (MSVideoSize *)data;
-	*vs = s->vconf.vsize;
-	return 0;
-}
-
-static int enc_set_fps(MSFilter *f, void *data) {
-	EncState *s = (EncState *)f->data;
-	float *fps = (float *)data;
-	s->vconf.fps = *fps;
-	enc_set_configuration(f, &s->vconf);
-	return 0;
-}
-
-static int enc_get_fps(MSFilter *f, void *data) {
-	EncState *s = (EncState *)f->data;
-	float *fps = (float *)data;
-	*fps = s->vconf.fps;
-	return 0;
-}
-
-static int enc_get_br(MSFilter *f, void *data) {
-	EncState *s = (EncState *)f->data;
-	*(int *)data = s->vconf.required_bitrate;
-	return 0;
-}
-
-static int enc_set_br(MSFilter *f, void *data) {
-	EncState *s = (EncState *)f->data;
-	int br = *(int *)data;
-	if (s->ready) {
-		/* Encoding is already ongoing, do not change video size, only bitrate. */
-		s->vconf.required_bitrate = br;
-		enc_set_configuration(f, &s->vconf);
-	} else {
-		MSVideoConfiguration best_vconf = ms_video_find_best_configuration_for_size_and_bitrate(s->vconf_list, s->vconf.vsize, ms_factory_get_cpu_count(f->factory), br);
-		enc_set_configuration(f, &best_vconf);
-	}
+	MSVideoConfiguration *vconf = (MSVideoConfiguration *)data;
+	memcpy(vconf, &s->vconf, sizeof(MSVideoConfiguration));
 	return 0;
 }
 
@@ -683,9 +720,11 @@ static int enc_notify_pli(MSFilter *f, void *data) {
 static int enc_notify_fir(MSFilter *f, void *data) {
 	EncState *s = (EncState *)f->data;
 	uint8_t seq_nr = *((uint8_t *)data);
-	if (seq_nr != s->last_fir_seq_nr) {
-		s->force_keyframe = TRUE;
-		s->last_fir_seq_nr = seq_nr;
+	if (should_generate_key_frame(s,MIN_KEY_FRAME_DIST)){
+		if (seq_nr != s->last_fir_seq_nr) {
+			s->force_keyframe = TRUE;
+			s->last_fir_seq_nr = seq_nr;
+		}
 	}
 	return 0;
 }
@@ -706,6 +745,7 @@ static int enc_notify_sli(MSFilter *f, void *data) {
 
 	diff=(64 + (int)(most_recent & 0x3F) - (int)(sli->picture_id & 0x3F)) % 64;
 	s->last_sli_id=most_recent-diff;
+
 	fs=enc_get_most_recent_reference_frame(s,FALSE);
 	ms_message("VP8: receiving SLI with pic id [%i], last-ref=[%i], most recent pic id=[%i]",s->last_sli_id, fs ? (int)fs->picture_id : 0, most_recent);
 	if (s->frames_state.golden.picture_id == s->last_sli_id) {
@@ -720,7 +760,9 @@ static int enc_notify_sli(MSFilter *f, void *data) {
 		/* Last key frame has been lost. */
 		s->force_keyframe = TRUE;
 	} else {
-		if (!fs || PICID_NEWER_THAN(s->last_sli_id,fs->picture_id)) {
+		/* If no reference frame known, OR sli is for a frame that comes after the last reference frame transmitted
+		 * OR last reference frame is older than last 16 bit picture id half cycle. */
+		if (!fs || PICID_NEWER_THAN(s->last_sli_id,fs->picture_id) || (s->frame_count - fs->count >= 1<<15)) {
 			s->invalid_frame_reported = TRUE;
 		} else {
 #ifdef AVPF_DEBUG
@@ -758,17 +800,6 @@ static int enc_get_configuration_list(MSFilter *f, void *data) {
 	return 0;
 }
 
-static int enc_set_configuration_list(MSFilter *f, void *data) {
-	EncState *s = (EncState *)f->data;
-	const MSVideoConfiguration **vconf_list = (const MSVideoConfiguration **)data;
-	if (*vconf_list == NULL) {
-		s->vconf_list = &vp8_conf_list[0];
-	} else {
-		s->vconf_list = *vconf_list;
-	}
-	return 0;
-}
-
 static int enc_enable_avpf(MSFilter *f, void *data) {
 	EncState *s = (EncState *)f->data;
 	s->avpf_enabled = *((bool_t *)data) ? TRUE : FALSE;
@@ -776,12 +807,6 @@ static int enc_enable_avpf(MSFilter *f, void *data) {
 }
 
 static MSFilterMethod enc_methods[] = {
-	{ MS_FILTER_SET_VIDEO_SIZE,                enc_set_vsize              },
-	{ MS_FILTER_SET_FPS,                       enc_set_fps                },
-	{ MS_FILTER_GET_VIDEO_SIZE,                enc_get_vsize              },
-	{ MS_FILTER_GET_FPS,                       enc_get_fps                },
-	{ MS_FILTER_SET_BITRATE,                   enc_set_br                 },
-	{ MS_FILTER_GET_BITRATE,                   enc_get_br                 },
 	{ MS_FILTER_REQ_VFU,                       enc_req_vfu                },
 	{ MS_VIDEO_ENCODER_REQ_VFU,                enc_req_vfu                },
 	{ MS_VIDEO_ENCODER_NOTIFY_PLI,             enc_notify_pli             },
@@ -789,7 +814,7 @@ static MSFilterMethod enc_methods[] = {
 	{ MS_VIDEO_ENCODER_NOTIFY_SLI,             enc_notify_sli             },
 	{ MS_VIDEO_ENCODER_NOTIFY_RPSI,            enc_notify_rpsi            },
 	{ MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, enc_get_configuration_list },
-	{ MS_VIDEO_ENCODER_SET_CONFIGURATION_LIST, enc_set_configuration_list },
+	{ MS_VIDEO_ENCODER_GET_CONFIGURATION,      enc_get_configuration      },
 	{ MS_VIDEO_ENCODER_SET_CONFIGURATION,      enc_set_configuration      },
 	{ MS_VIDEO_ENCODER_ENABLE_AVPF,            enc_enable_avpf            },
 	{ 0,                                       NULL                       }
@@ -801,7 +826,7 @@ static MSFilterMethod enc_methods[] = {
 #define MS_VP8_ENC_ENC_FMT     "VP8"
 #define MS_VP8_ENC_NINPUTS     1 /*MS_YUV420P is assumed on this input */
 #define MS_VP8_ENC_NOUTPUTS    1
-#define MS_VP8_ENC_FLAGS       0
+#define MS_VP8_ENC_FLAGS       MS_FILTER_IS_PUMP
 
 #ifdef _MSC_VER
 
@@ -858,7 +883,7 @@ typedef struct DecState {
 	long last_cseq; /*last receive sequence number, used to locate missing partition fragment*/
 	int current_partition_id; /*current partition id*/
 	uint64_t last_error_reported_time;
-	mblk_t *yuv_msg;
+	MSYuvBufAllocator *allocator;
 	MSPicture outbuf;
 	unsigned int yuv_width, yuv_height;
 	MSQueue q;
@@ -867,8 +892,15 @@ typedef struct DecState {
 	bool_t avpf_enabled;
 	bool_t freeze_on_error;
 	bool_t ready;
+	ms_thread_t thread;
+	ms_cond_t thread_cond;
+	MSQueue entry_q;
+	MSQueue exit_q;
+	bool_t thread_running;
+	bool_t waiting; // TRUE when the processing thread is in ms_cond_wait()
 } DecState;
 
+static void * dec_processing_thread(void *obj);
 
 static void dec_init(MSFilter *f) {
 	DecState *s = (DecState *)ms_new0(DecState, 1);
@@ -879,11 +911,13 @@ static void dec_init(MSFilter *f) {
 	s->last_error_reported_time = 0;
 	s->yuv_width = 0;
 	s->yuv_height = 0;
-	s->yuv_msg = 0;
+	s->allocator = ms_yuv_buf_allocator_new();
+	ms_yuv_buf_allocator_set_max_frames(s->allocator, 3);
 	ms_queue_init(&s->q);
 	s->first_image_decoded = FALSE;
 	s->avpf_enabled = FALSE;
 	s->freeze_on_error = TRUE;
+	ms_cond_init(&s->thread_cond, NULL);
 	f->data = s;
 	ms_average_fps_init(&s->fps, "VP8 decoder: FPS: %f");
 }
@@ -893,7 +927,12 @@ static int dec_initialize_impl(MSFilter *f){
 	vpx_codec_dec_cfg_t cfg;
 
 	memset(&cfg, 0, sizeof(cfg));
+#if TARGET_OS_OSX
+	//On macosx 10.14 realtime processing is not ensured on dual core machine with ms_factory_get_cpu_count() <= 4. Value belows is a tradeoff between scalability and realtime
+	cfg.threads = MAX(ms_factory_get_cpu_count(f->factory)-2,1);
+#else
 	cfg.threads = ms_factory_get_cpu_count(f->factory);
+#endif
 	if (vpx_codec_dec_init(&s->codec, s->iface, &cfg, s->flags)){
 		ms_error("Failed to initialize VP8 decoder");
 		return -1;
@@ -926,104 +965,160 @@ static void dec_preprocess(MSFilter* f) {
 		s->first_image_decoded = FALSE;
 		s->ready=TRUE;
 	}
-
+	ms_queue_init(&s->entry_q);
+	ms_queue_init(&s->exit_q);
+	s->thread_running = TRUE;
+	s->waiting = FALSE;
+	ms_thread_create(&s->thread, NULL, dec_processing_thread, f);
 }
 
 static void dec_uninit(MSFilter *f) {
 	DecState *s = (DecState *)f->data;
 	vp8rtpfmt_unpacker_uninit(&s->unpacker);
 	vpx_codec_destroy(&s->codec);
-	if (s->yuv_msg) freemsg(s->yuv_msg);
+	ms_yuv_buf_allocator_free(s->allocator);
 	ms_queue_flush(&s->q);
+	ms_cond_destroy(&s->thread_cond);
 	ms_free(s);
 }
 
+static void *dec_processing_thread(void *obj) {
+	MSFilter *f = (MSFilter*)obj;
+	DecState *s = (DecState *)f->data;
+	mblk_t *im;
+	MSQueue frame;
+	Vp8RtpFmtFrameInfo frame_info;
+
+	ms_queue_init(&frame);
+
+	ms_filter_lock(f);
+	while(s->thread_running){
+		if (ms_queue_empty(&s->entry_q)){
+			s->waiting = TRUE;
+			ms_cond_wait(&s->thread_cond, &f->lock);
+			s->waiting = FALSE;
+			continue;
+		}
+		
+		/* Unpack RTP payload format for VP8. */
+		vp8rtpfmt_unpacker_feed(&s->unpacker, &s->entry_q);
+		ms_filter_unlock(f);
+	
+		/* Decode unpacked VP8 frames. */
+		while (vp8rtpfmt_unpacker_get_frame(&s->unpacker, &frame, &frame_info) == 0) {
+			vpx_codec_err_t err;
+			vpx_image_t *img;
+			vpx_codec_iter_t iter = NULL;
+			while ((im = ms_queue_get(&frame)) != NULL) {
+				err = vpx_codec_decode(&s->codec, im->b_rptr, (unsigned int)(im->b_wptr - im->b_rptr), NULL, 0);
+				if ((s->flags & VPX_CODEC_USE_INPUT_FRAGMENTS) && mblk_get_marker_info(im)) {
+					err = vpx_codec_decode(&s->codec, NULL, 0, NULL, 0);
+				}
+				if (err) {
+					ms_warning("vp8 decode failed : %d %s (%s)\n", err, vpx_codec_err_to_string(err), vpx_codec_error_detail(&s->codec)?vpx_codec_error_detail(&s->codec):"no details");
+				}
+				freemsg(im);
+			}
+	
+			/* Get decoded frame */
+			while ((img = vpx_codec_get_frame(&s->codec, &iter))) {
+				int i, j;
+				int reference_updates = 0;
+				mblk_t *yuv_msg;
+	
+				if (vpx_codec_control(&s->codec, VP8D_GET_LAST_REF_UPDATES, &reference_updates) == 0) {
+					if (frame_info.pictureid_present && ((reference_updates & VP8_GOLD_FRAME) || (reference_updates & VP8_ALTR_FRAME))) {
+						vp8rtpfmt_send_rpsi(&s->unpacker, frame_info.pictureid);
+					}
+				}
+	
+				if (s->yuv_width != img->d_w || s->yuv_height != img->d_h) {
+					ms_message("MSVp8Dec: video is %ix%i", img->d_w, img->d_h);
+					s->yuv_width = img->d_w;
+					s->yuv_height = img->d_h;
+					ms_filter_notify_no_arg(f, MS_FILTER_OUTPUT_FMT_CHANGED);
+				}
+	
+				yuv_msg = ms_yuv_buf_allocator_get(s->allocator, &s->outbuf, img->d_w, img->d_h);
+				
+				ms_average_fps_update(&s->fps, (uint32_t)f->ticker->time);
+				if (!s->first_image_decoded) {
+					s->first_image_decoded = TRUE;
+					ms_filter_notify_no_arg(f, MS_VIDEO_DECODER_FIRST_IMAGE_DECODED);
+				}
+				
+				if (yuv_msg == NULL) {
+					ms_warning("MSVp8Dec: no more output buffers, frame is discarded.");
+				}else{
+					/* scale/copy frame to destination mblk_t */
+					for (i = 0; i < 3; i++) {
+						uint8_t *dest = s->outbuf.planes[i];
+						uint8_t *src = img->planes[i];
+						int h = img->d_h >> ((i > 0) ? 1 : 0);
+		
+						for (j = 0; j < h; j++) {
+							memcpy(dest, src, s->outbuf.strides[i]);
+							dest += s->outbuf.strides[i];
+							src += img->stride[i];
+						}
+					}
+					
+					ms_filter_lock(f);
+					ms_queue_put(&s->exit_q, yuv_msg);
+					ms_filter_unlock(f);
+				}
+			}
+		}
+		ms_filter_lock(f);
+	}
+	ms_filter_unlock(f);
+	return NULL;
+}
+
 static void dec_process(MSFilter *f) {
 	DecState *s = (DecState *)f->data;
-	mblk_t *im;
-	vpx_codec_err_t err;
-	vpx_image_t *img;
-	vpx_codec_iter_t iter = NULL;
-	MSQueue frame;
-	MSQueue mtofree_queue;
-	Vp8RtpFmtFrameInfo frame_info;
-
-	if (!s->ready){
-		ms_queue_flush(f->inputs[0]);
-		return;
-	}
+	mblk_t *entry_f;
+	mblk_t *exit_f;
+	bool_t queued_something = FALSE;
 
 	ms_filter_lock(f);
 
-	ms_queue_init(&frame);
-	ms_queue_init(&mtofree_queue);
+#ifdef AVPF_DEBUG
+	ms_message("VP8 dec_process:");
+#endif
 
-	/* Unpack RTP payload format for VP8. */
-	vp8rtpfmt_unpacker_feed(&s->unpacker, f->inputs[0]);
-
-	/* Decode unpacked VP8 frames. */
-	while (vp8rtpfmt_unpacker_get_frame(&s->unpacker, &frame, &frame_info) == 0) {
-		while ((im = ms_queue_get(&frame)) != NULL) {
-			err = vpx_codec_decode(&s->codec, im->b_rptr, (unsigned int)(im->b_wptr - im->b_rptr), NULL, 0);
-			if ((s->flags & VPX_CODEC_USE_INPUT_FRAGMENTS) && mblk_get_marker_info(im)) {
-				err = vpx_codec_decode(&s->codec, NULL, 0, NULL, 0);
-			}
-			if (err) {
-				ms_warning("vp8 decode failed : %d %s (%s)\n", err, vpx_codec_err_to_string(err), vpx_codec_error_detail(&s->codec)?vpx_codec_error_detail(&s->codec):"no details");
-			}
-			ms_queue_put(&mtofree_queue, im);
-		}
-
-		/* Get decoded frame */
-		if ((img = vpx_codec_get_frame(&s->codec, &iter))) {
-			int i, j;
-			int reference_updates = 0;
-
-			if (vpx_codec_control(&s->codec, VP8D_GET_LAST_REF_UPDATES, &reference_updates) == 0) {
-				if (frame_info.pictureid_present && ((reference_updates & VP8_GOLD_FRAME) || (reference_updates & VP8_ALTR_FRAME))) {
-					vp8rtpfmt_send_rpsi(&s->unpacker, frame_info.pictureid);
-				}
-			}
-
-			if (s->yuv_width != img->d_w || s->yuv_height != img->d_h) {
-				if (s->yuv_msg) freemsg(s->yuv_msg);
-				s->yuv_msg = ms_yuv_buf_alloc(&s->outbuf, img->d_w, img->d_h);
-				ms_message("MSVp8Dec: video is %ix%i", img->d_w, img->d_h);
-				s->yuv_width = img->d_w;
-				s->yuv_height = img->d_h;
-				ms_filter_notify_no_arg(f, MS_FILTER_OUTPUT_FMT_CHANGED);
-			}
-
-			/* scale/copy frame to destination mblk_t */
-			for (i = 0; i < 3; i++) {
-				uint8_t *dest = s->outbuf.planes[i];
-				uint8_t *src = img->planes[i];
-				int h = img->d_h >> ((i > 0) ? 1 : 0);
-
-				for (j = 0; j < h; j++) {
-					memcpy(dest, src, s->outbuf.strides[i]);
-					dest += s->outbuf.strides[i];
-					src += img->stride[i];
-				}
-			}
-			ms_queue_put(f->outputs[0], dupmsg(s->yuv_msg));
-
-			ms_average_fps_update(&s->fps, (uint32_t)f->ticker->time);
-			if (!s->first_image_decoded) {
-				s->first_image_decoded = TRUE;
-				ms_filter_notify_no_arg(f, MS_VIDEO_DECODER_FIRST_IMAGE_DECODED);
-			}
-		}
-
-		while ((im = ms_queue_get(&mtofree_queue)) != NULL) {
-			freemsg(im);
-		}
+	if (!s->ready) {
+		ms_queue_flush(f->inputs[0]);
+		ms_filter_unlock(f);
+		return;
+	}
+	
+	if (f->ticker->time % 1000 == 0){
+		ms_message("VP8 dec: entry queue size: %i packets.", (int)s->entry_q.q.q_mcount);
+	}
+	while ((entry_f = ms_queue_get(f->inputs[0])) != NULL) {
+		ms_queue_put(&s->entry_q, entry_f);
+		queued_something = TRUE;
 	}
 
+	/* Put each frame we have in exit_q in f->output[0] */
+	while ((exit_f = ms_queue_get(&s->exit_q)) != NULL) {
+		ms_queue_put(f->outputs[0], exit_f);
+	}
+	if (queued_something && s->waiting){
+		/* Wake up the processing thread if it is waiting some job. */
+		ms_cond_signal(&s->thread_cond);
+	}
 	ms_filter_unlock(f);
 }
 
 static void dec_postprocess(MSFilter *f) {
+	DecState *s = (DecState *)f->data;
+	ms_filter_lock(f);
+	s->thread_running = FALSE;
+	if (s->waiting) ms_cond_signal(&s->thread_cond);
+	ms_filter_unlock(f);
+	ms_thread_join(s->thread, NULL);
 }
 
 static int dec_reset_first_image(MSFilter* f, void *data) {
@@ -1100,7 +1195,7 @@ static MSFilterMethod dec_methods[] = {
 #define MS_VP8_DEC_ENC_FMT     "VP8"
 #define MS_VP8_DEC_NINPUTS     1
 #define MS_VP8_DEC_NOUTPUTS    1
-#define MS_VP8_DEC_FLAGS       0
+#define MS_VP8_DEC_FLAGS       MS_FILTER_IS_PUMP
 
 #ifdef _MSC_VER
 
