@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -201,8 +201,10 @@ static void audio_stream_configure_resampler(AudioStream *st, MSFilter *resample
 	ms_filter_call_method(resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&to_rate);
 	ms_filter_call_method(resampler, MS_FILTER_SET_NCHANNELS, &from_channels);
 	ms_filter_call_method(resampler, MS_FILTER_SET_OUTPUT_NCHANNELS, &to_channels);
-	ms_message("configuring %s:%p-->%s:%p from rate [%i] to rate [%i] and from channel [%i] to channel [%i]",
-			   from->desc->name, from, to->desc->name, to, from_rate, to_rate, from_channels, to_channels);
+	ms_message(
+		"configuring %s:%p-->%s:%p from rate [%i] to rate [%i] and from channel [%i] to channel [%i]",
+		from->desc->name, from, to->desc->name, to, from_rate, to_rate, from_channels, to_channels
+	);
 }
 
 static void audio_stream_process_rtcp(MediaStream *media_stream, mblk_t *m){
@@ -241,6 +243,36 @@ bool_t audio_stream_started(AudioStream *stream){
 	return media_stream_started(&stream->ms);
 }
 
+static void write_callback(void *ud, MSFilter *f, unsigned int id, void *arg){
+	AudioStream *stream=(AudioStream *)ud;
+	switch(id){
+		case MS_FILTER_OUTPUT_FMT_CHANGED:
+			if (f==stream->soundwrite && stream->write_resampler){
+				MSFilter *to = stream->soundwrite;
+				if (stream->write_encoder) to = stream->write_encoder;
+				audio_stream_configure_resampler(stream, stream->write_resampler, stream->ms.decoder, to);
+			}
+		break;
+		default:
+		break;
+	}
+}
+
+static void read_callback(void *ud, MSFilter *f, unsigned int id, void *arg){
+	AudioStream *stream=(AudioStream *)ud;
+	switch(id){
+		case MS_FILTER_OUTPUT_FMT_CHANGED:
+			if (f==stream->soundread && stream->read_resampler){
+				MSFilter *from = stream->soundread;
+				if (stream->read_decoder) from = stream->read_decoder;
+				audio_stream_configure_resampler(stream, stream->read_resampler, from, stream->ms.encoder);
+			}
+		break;
+		default:
+		break;
+	}
+}
+
 /* This function is used either on IOS to workaround the long time to initialize the Audio Unit or for ICE candidates gathering. */
 void audio_stream_prepare_sound(AudioStream *stream, MSSndCard *playcard, MSSndCard *captcard){
 	audio_stream_unprepare_sound(stream);
@@ -263,7 +295,7 @@ void audio_stream_prepare_sound(AudioStream *stream, MSSndCard *playcard, MSSndC
 	} else {
 		stream->ms.voidsink=ms_factory_create_filter(stream->ms.factory,  MS_VOID_SINK_ID);
 		ms_filter_link(stream->dummy,0,stream->ms.voidsink,0);
-		
+
 	}
 	if (stream->ms.sessions.ticker == NULL) media_stream_start_ticker(&stream->ms);
 	ms_ticker_attach(stream->ms.sessions.ticker,stream->dummy);
@@ -330,6 +362,8 @@ static void setup_local_player(AudioStream *stream, int samplerate, int channels
 	ms_filter_call_method(stream->local_mixer,MS_FILTER_SET_NCHANNELS,&channels);
 	ms_filter_call_method(stream->local_mixer,MS_AUDIO_MIXER_SET_MASTER_CHANNEL,&master);
 	ms_filter_add_notify_callback(stream->local_player,player_callback,stream,TRUE);
+	ms_filter_add_notify_callback(stream->soundwrite, write_callback, stream, TRUE);
+	ms_filter_add_notify_callback(stream->soundread, read_callback, stream, TRUE);
 }
 
 static OrtpRtcpXrPlcStatus audio_stream_get_rtcp_xr_plc_status(void *userdata) {
@@ -787,9 +821,7 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 	MSConnectionHelper h;
 	int sample_rate;
 	int nchannels;
-	int err1,err2;
 	bool_t has_builtin_ec=FALSE;
-	bool_t resampler_missing = FALSE;
 	bool_t skip_encoder_and_decoder = FALSE;
 	bool_t do_ts_adjustments = TRUE;
 
@@ -813,9 +845,20 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 		stream->dtmfgen=ms_factory_create_filter(stream->ms.factory, MS_DTMF_GEN_ID);
 	else
 		stream->dtmfgen=NULL;
+
+	/* FIXME: Temporary workaround for -Wcast-function-type. */
+	#if __GNUC__ >= 8
+		_Pragma("GCC diagnostic push")
+		_Pragma("GCC diagnostic ignored \"-Wcast-function-type\"")
+	#endif // if __GNUC__ >= 8
+
 	rtp_session_signal_connect(rtps,"telephone-event",(RtpCallback)on_dtmf_received,stream);
 	rtp_session_signal_connect(rtps,"payload_type_changed",(RtpCallback)audio_stream_payload_type_changed,stream);
 
+	#if __GNUC__ >= 8
+		_Pragma("GCC diagnostic pop")
+	#endif // if __GNUC__ >= 8
+
 	if (stream->ms.state==MSStreamPreparing){
 		/*we were using the dummy preload graph, destroy it but keep sound filters unless no soundcard is given*/
 		_audio_stream_unprepare_sound(stream, io->input.type == MSResourceSoundcard);
@@ -835,8 +878,6 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 		stream->read_decoder = ms_factory_create_decoder(stream->ms.factory, pt->mime_type);
 	} else {
 		stream->soundread=ms_factory_create_filter(stream->ms.factory, MS_FILE_PLAYER_ID);
-		stream->read_resampler=ms_factory_create_filter(stream->ms.factory, MS_RESAMPLE_ID);
-		resampler_missing = stream->read_resampler == NULL;
 	}
 	if (io->output.type == MSResourceSoundcard) {
 		if (stream->soundwrite==NULL)
@@ -864,8 +905,8 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 
 	if ((stream->features & AUDIO_STREAM_FEATURE_DTMF) != 0 && (tev_pt == -1)
 		&& ( strcasecmp(pt->mime_type,"pcmu")==0 || strcasecmp(pt->mime_type,"pcma")==0)){
-		/*if no telephone-event payload is usable and pcma or pcmu is used, we will generate
-		  inband dtmf*/
+		// If no telephone-event payload is usable and pcma or pcmu is used, we will generate
+		// inband dtmf.
 		stream->dtmfgen_rtp=ms_factory_create_filter (stream->ms.factory, MS_DTMF_GEN_ID);
 
 	} else {
@@ -878,7 +919,7 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 		ms_error("Sample rate is unknown for RTP side !");
 		return -1;
 	}
-	
+
 	if (stream->features == 0) {
 		MSPinFormat sndread_format = {0};
 		MSPinFormat rtpsend_format = {0};
@@ -894,10 +935,13 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 	}
 	do_ts_adjustments = !skip_encoder_and_decoder;
 	ms_filter_call_method(stream->ms.rtpsend, MS_RTP_SEND_ENABLE_TS_ADJUSTMENT, &do_ts_adjustments);
-	
+
 	if (!skip_encoder_and_decoder) {
+		ms_message("audio_stream_start_from_io: create encoder, decoder and resamplers.");
 		stream->ms.encoder=ms_factory_create_encoder(stream->ms.factory, pt->mime_type);
 		stream->ms.decoder=ms_factory_create_decoder(stream->ms.factory, pt->mime_type);
+		stream->read_resampler = ms_factory_create_filter(stream->ms.factory, MS_RESAMPLE_ID);
+		stream->write_resampler = ms_factory_create_filter(stream->ms.factory, MS_RESAMPLE_ID);
 	}
 
 	/* sample rate is already set for rtpsend and rtprcv, check if we have to adjust it to */
@@ -986,28 +1030,11 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 		ms_filter_call_method(stream->dtmfgen_rtp,MS_FILTER_SET_NCHANNELS,&nchannels);
 	}
 
-	/*don't put these two statements in a single if, because the second one will not be executed if the first one evaluates as true*/
-	err1 = ms_filter_call_method(stream->soundread, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
-	err2 = ms_filter_call_method(stream->soundread, MS_FILTER_SET_NCHANNELS, &nchannels);
-	/* give the sound filters some properties */
-	if (err1 != 0 || err2 != 0){
-		/* need to add resampler*/
-		if (stream->read_resampler == NULL) stream->read_resampler = ms_factory_create_filter(stream->ms.factory, MS_RESAMPLE_ID);
-		resampler_missing = stream->read_resampler == NULL;
-	}
+	ms_filter_call_method(stream->soundread, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
+	ms_filter_call_method(stream->soundread, MS_FILTER_SET_NCHANNELS, &nchannels);
 
-	err1 = ms_filter_call_method(stream->soundwrite, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
-	err2 = ms_filter_call_method(stream->soundwrite, MS_FILTER_SET_NCHANNELS, &nchannels);
-	if (err1 !=0 || err2 != 0){
-		/* need to add resampler*/
-		if (stream->write_resampler == NULL) stream->write_resampler = ms_factory_create_filter(stream->ms.factory, MS_RESAMPLE_ID);
-		resampler_missing = stream->write_resampler == NULL;
-	}
-	
-	if (resampler_missing){
-		ms_fatal("AudioStream: no resampler implementation found, but resampler is required to perform the AudioStream. "
-			"Does mediastreamer2 was compiled with libspeex dependency ?");
-	}
+	ms_filter_call_method(stream->soundwrite, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
+	ms_filter_call_method(stream->soundwrite, MS_FILTER_SET_NCHANNELS, &nchannels);
 
 	if (stream->ec){
 		if (!stream->is_ec_delay_set) {
@@ -1078,7 +1105,7 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 		/*configure equalizer if needed*/
 		MSDevicesInfo *devices = ms_factory_get_devices_info(stream->ms.factory);
 		SoundDeviceDescription *device = ms_devices_info_get_sound_device_description(devices);
-		
+
 		audio_stream_set_mic_gain_db(stream, 0);
 		audio_stream_set_spk_gain_db(stream, 0);
 		if (device && device->hacks) {
@@ -1165,7 +1192,7 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 			if (ms_filter_call_method(stream->ms.decoder, MS_DECODER_ENABLE_PLC, &decoder_enable_plc) != 0) {
 				ms_warning(" MS_DECODER_ENABLE_PLC on stream %p function error ", stream);
 			}
-			
+
 		}
 		stream->plc = NULL;
 	}
@@ -1270,7 +1297,7 @@ int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const c
 	stream->ms.state=MSStreamStarted;
 
 	if (stream->soundwrite) {
-		if (ms_filter_implements_interface(stream->soundwrite, MSFilterAudioPlaybackInterface)) {
+		if (ms_filter_has_method(stream->soundwrite, MS_AUDIO_PLAYBACK_SET_ROUTE)) {
 			ms_filter_call_method(stream->soundwrite, MS_AUDIO_PLAYBACK_SET_ROUTE, &stream->audio_route);
 		}
 	}
@@ -1427,6 +1454,7 @@ int audio_stream_mixed_record_start(AudioStream *st){
 		mctl.pin=pin;
 		mctl.param.enabled=TRUE;
 		ms_filter_call_method(st->outbound_mixer,MS_AUDIO_MIXER_ENABLE_OUTPUT,&mctl);
+		if (st->videostream) video_stream_enable_recording(st->videostream, TRUE);
 		return 0;
 	}
 	return -1;
@@ -1440,11 +1468,13 @@ int audio_stream_mixed_record_stop(AudioStream *st){
 
 		if (recorder==NULL) return -1;
 		ms_filter_call_method(st->recv_tee,MS_TEE_MUTE,&pin);
+		if (st->videostream) video_stream_enable_recording(st->videostream, FALSE);
 		mctl.pin=pin;
 		mctl.param.enabled=FALSE;
 		ms_filter_call_method(st->outbound_mixer,MS_AUDIO_MIXER_ENABLE_OUTPUT,&mctl);
 		ms_filter_call_method_noarg(recorder,MS_RECORDER_PAUSE);
 		ms_filter_call_method_noarg(recorder,MS_RECORDER_CLOSE);
+
 	}
 	return 0;
 }
@@ -1475,8 +1505,9 @@ AudioStream *audio_stream_new_with_sessions(MSFactory *factory, const MSMediaStr
 	}
 
 	stream->ms.type = MSAudio;
+	media_stream_set_direction(&stream->ms, MediaStreamSendRecv);
 	media_stream_init(&stream->ms,factory, sessions);
-	
+
 	ms_factory_enable_statistics(factory, TRUE);
 	ms_factory_reset_statistics(factory);
 
@@ -1780,10 +1811,23 @@ void audio_stream_stop(AudioStream * stream){
 		}
 	}
 	rtp_session_set_rtcp_xr_media_callbacks(stream->ms.sessions.rtp_session, NULL);
+
+	/* FIXME: Temporary workaround for -Wcast-function-type. */
+	#if __GNUC__ >= 8
+		_Pragma("GCC diagnostic push")
+		_Pragma("GCC diagnostic ignored \"-Wcast-function-type\"")
+	#endif // if __GNUC__ >= 8
+
 	rtp_session_signal_disconnect_by_callback(stream->ms.sessions.rtp_session,"telephone-event",(RtpCallback)on_dtmf_received);
 	rtp_session_signal_disconnect_by_callback(stream->ms.sessions.rtp_session,"payload_type_changed",(RtpCallback)audio_stream_payload_type_changed);
-	/*before destroying the filters, pump the event queue so that pending events have a chance to reach their listeners.
-	 * When the filter are destroyed, all their pending events in the event queue will be cancelled*/
+
+	#if __GNUC__ >= 8
+		_Pragma("GCC diagnostic pop")
+	#endif // if __GNUC__ >= 8
+
+	// Before destroying the filters, pump the event queue so that pending events have a chance
+	// to reach their listeners. When the filter are destroyed, all their pending events in the
+	// event queue will be cancelled.
 	evq = ms_factory_get_event_queue(stream->ms.factory);
 	if (evq) ms_event_queue_pump(evq);
 	ms_factory_log_statistics(stream->ms.factory);
@@ -1839,10 +1883,23 @@ void audio_stream_set_spk_gain_db(AudioStream *stream, float gain_db) {
 	}
 #endif
 
-	if (stream->volrecv){
+	if (stream->volrecv)
 		ms_filter_call_method(stream->volrecv, MS_VOLUME_SET_DB_GAIN, &gain);
-	} else ms_warning("Could not apply gain on received RTP packet: gain control wasn't activated. "
-			"Use audio_stream_enable_gain_control() before starting the stream.");
+	else
+		ms_warning(
+			"Could not apply gain on received RTP packet: gain control wasn't activated. "
+			"Use audio_stream_enable_gain_control() before starting the stream."
+		);
+}
+
+void audio_stream_set_spk_gain(AudioStream *stream, float gain) {
+	if (stream->volrecv)
+		ms_filter_call_method(stream->volrecv, MS_VOLUME_SET_GAIN, &gain);
+	else
+		ms_warning(
+			"Could not apply gain on received RTP packet: gain control wasn't activated. "
+			"Use audio_stream_enable_gain_control() before starting the stream."
+		);
 }
 
 float audio_stream_get_quality_rating(AudioStream *stream){
@@ -1896,6 +1953,7 @@ static void configure_av_recorder(AudioStream *stream){
 
 void audio_stream_link_video(AudioStream *stream, VideoStream *video){
 	stream->videostream=video;
+	video->audiostream=stream;
 	if (stream->av_recorder.video_input && video->recorder_output){
 		ms_message("audio_stream_link_video() connecting itc filters");
 		ms_filter_call_method(video->recorder_output,MS_ITC_SINK_CONNECT,stream->av_recorder.video_input);
@@ -1905,8 +1963,10 @@ void audio_stream_link_video(AudioStream *stream, VideoStream *video){
 
 void audio_stream_unlink_video(AudioStream *stream, VideoStream *video){
 	stream->videostream=NULL;
+	video->audiostream=NULL;
 	if (stream->av_recorder.video_input && video->recorder_output){
 		ms_filter_call_method(video->recorder_output,MS_ITC_SINK_CONNECT,NULL);
+		video_stream_enable_recording(video, FALSE);
 	}
 }
 
@@ -1918,4 +1978,3 @@ void audio_stream_set_audio_route(AudioStream *stream, MSAudioRoute route) {
 		}
 	}
 }
-
