@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006-2013 Belledonne Communications, Grenoble
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -107,12 +107,12 @@ MSTickerPrio __ms_get_default_prio(bool_t is_video) {
 
 void media_stream_init(MediaStream *stream, MSFactory *factory, const MSMediaStreamSessions *sessions) {
 	stream->sessions = *sessions;
-	
+
 	stream->evd = ortp_ev_dispatcher_new(stream->sessions.rtp_session);
 	stream->evq = ortp_ev_queue_new();
 	stream->factory = factory; /*the factory is used later to instanciate everything in mediastreamer2.*/
 	rtp_session_register_event_queue(stream->sessions.rtp_session, stream->evq);
-	
+
 	/*we give to the zrtp and dtls sessions a backpointer to all the stream sessions*/
 	if (sessions->zrtp_context != NULL) {
 		ms_zrtp_set_stream_sessions(sessions->zrtp_context, &stream->sessions);
@@ -129,22 +129,41 @@ void media_stream_init(MediaStream *stream, MSFactory *factory, const MSMediaStr
 
 RtpSession * ms_create_duplex_rtp_session(const char* local_ip, int loc_rtp_port, int loc_rtcp_port, int mtu) {
 	RtpSession *rtpr;
-
+	const int socket_buf_size=2000000;
+	
 	rtpr = rtp_session_new(RTP_SESSION_SENDRECV);
 	rtp_session_set_recv_buf_size(rtpr, MAX(mtu , MS_MINIMAL_MTU));
 	rtp_session_set_scheduling_mode(rtpr, 0);
 	rtp_session_set_blocking_mode(rtpr, 0);
 	rtp_session_enable_adaptive_jitter_compensation(rtpr, TRUE);
 	rtp_session_set_symmetric_rtp(rtpr, TRUE);
-	rtp_session_set_local_addr(rtpr, local_ip, loc_rtp_port, loc_rtcp_port);
+	if (local_ip) {
+		rtp_session_set_local_addr(rtpr, local_ip, loc_rtp_port, loc_rtcp_port);
+	} else {
+		local_ip = "::0";
+		if (rtp_session_set_local_addr(rtpr, local_ip, loc_rtp_port, loc_rtcp_port) < 0) {
+			local_ip = "0.0.0.0";
+			rtp_session_set_local_addr(rtpr, local_ip, loc_rtp_port, loc_rtcp_port);
+		}
+	}
+
 	rtp_session_signal_connect(rtpr, "timestamp_jump", (RtpCallback)rtp_session_resync, NULL);
 	rtp_session_signal_connect(rtpr, "ssrc_changed", (RtpCallback)rtp_session_resync, NULL);
+
 	rtp_session_set_ssrc_changed_threshold(rtpr, 0);
 	rtp_session_set_rtcp_report_interval(rtpr, 2500);	/* At the beginning of the session send more reports. */
-	rtp_session_set_multicast_loopback(rtpr,TRUE); /*very useful, specially for testing purposes*/
+	rtp_session_set_multicast_loopback(rtpr, TRUE); /*very useful, specially for testing purposes*/
 	rtp_session_set_send_ts_offset(rtpr, (uint32_t)bctbx_random());
 	rtp_session_enable_avpf_feature(rtpr, ORTP_AVPF_FEATURE_TMMBR, TRUE);
 	disable_checksums(rtp_session_get_rtp_socket(rtpr));
+	
+	/* Enlarge kernel socket buffers, which is necessary for video streams because large amounts of data can arrive between
+	 * two ticks of mediastreamer2 processing.
+	 * Previously, this was done only for VideoStreams, but since the sockets in the audio RtpSession may be used for video too
+	 * (in RTP bundle mode), then it has to be done also for audio sockets.
+	 */
+	rtp_session_set_rtp_socket_recv_buffer_size(rtpr, socket_buf_size);
+	rtp_session_set_rtp_socket_send_buffer_size(rtpr, socket_buf_size);
 	return rtpr;
 }
 
@@ -199,7 +218,7 @@ void media_stream_free(MediaStream *stream) {
 	if (stream->sessions.dtls_context != NULL) {
 		ms_dtls_srtp_set_stream_sessions(stream->sessions.dtls_context, NULL);
 	}
-	
+
 	if (stream->sessions.rtp_session != NULL) rtp_session_unregister_event_queue(stream->sessions.rtp_session, stream->evq);
 	if (stream->evq != NULL) ortp_ev_queue_destroy(stream->evq);
 	if (stream->evd != NULL) ortp_ev_dispatcher_destroy(stream->evd);
@@ -211,6 +230,13 @@ void media_stream_free(MediaStream *stream) {
 	if (stream->decoder != NULL) ms_filter_destroy(stream->decoder);
 	if (stream->voidsink != NULL) ms_filter_destroy(stream->voidsink);
 	if (stream->qi) ms_quality_indicator_destroy(stream->qi);
+#ifdef VIDEO_ENABLED
+	if (stream->video_quality_controller) ms_video_quality_controller_destroy(stream->video_quality_controller);
+#endif
+}
+
+MSFactory* media_stream_get_factory(MediaStream *stream) {
+	return (stream) ? stream->factory : NULL;
 }
 
 bool_t media_stream_started(MediaStream *stream) {
@@ -247,10 +273,12 @@ void media_stream_enable_adaptive_jittcomp(MediaStream *stream, bool_t enabled)
 	rtp_session_enable_adaptive_jitter_compensation(stream->sessions.rtp_session, enabled);
 }
 
-void media_stream_enable_dtls(MediaStream *stream, MSDtlsSrtpParams *params){
-	if (stream->sessions.dtls_context==NULL) {
+void media_stream_enable_dtls(MediaStream *stream, const MSDtlsSrtpParams *params){
+	if (stream->sessions.dtls_context == NULL) {
+		MSDtlsSrtpParams params_copy = *params;
 		ms_message("Start DTLS media stream context in stream session [%p]", &(stream->sessions));
-		stream->sessions.dtls_context=ms_dtls_srtp_context_new(&(stream->sessions), params);
+		if (params_copy.mtu == 0) params_copy.mtu = ms_factory_get_mtu(stream->factory);
+		stream->sessions.dtls_context=ms_dtls_srtp_context_new(&(stream->sessions), &params_copy);
 	}
 }
 
@@ -261,6 +289,10 @@ void media_stream_set_ice_check_list(MediaStream *stream, IceCheckList *cl) {
 		ice_check_list_set_rtp_session(stream->ice_check_list, stream->sessions.rtp_session);
 		stun_enabled = FALSE;
 	}
+	if (stream->sessions.rtp_session->bundle && !stream->sessions.rtp_session->is_primary){
+		ms_message("Disabling stun sending for rtp session %p", stream->sessions.rtp_session);
+		stun_enabled = FALSE;
+	}
 	if (stream->rtpsend != NULL) {
 		ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_ENABLE_STUN, &stun_enabled);
 	}
@@ -332,6 +364,14 @@ static void media_stream_process_rtcp(MediaStream *stream, mblk_t *m, time_t cur
 	}while(rtcp_next_packet(m));
 }
 
+void media_stream_set_direction(MediaStream *stream, MediaStreamDir dir) {
+	stream->direction = dir;
+}
+
+MediaStreamDir media_stream_get_direction(const MediaStream *stream) {
+	return stream->direction;
+}
+
 void media_stream_iterate(MediaStream *stream){
 	time_t curtime=ms_time(NULL);
 
@@ -342,7 +382,11 @@ void media_stream_iterate(MediaStream *stream){
 			rtp_session_set_rtcp_report_interval(stream->sessions.rtp_session,5000);
 			stream->is_beginning=FALSE;
 		}
-		if (stream->qi && curtime>stream->last_iterate_time) ms_quality_indicator_update_local(stream->qi);
+		if (stream->qi && curtime>stream->last_iterate_time) {
+			if (stream->direction != MediaStreamSendOnly) { // Local quality indicator in send only will be wrong
+				ms_quality_indicator_update_local(stream->qi);
+			}
+		} 
 	}
 	stream->last_iterate_time=curtime;
 
@@ -427,6 +471,11 @@ int media_stream_set_target_network_bitrate(MediaStream *stream,int target_bitra
 	return 0;
 }
 
+int media_stream_set_max_network_bitrate(MediaStream *stream,int max_bitrate){
+	stream->max_target_bitrate = max_bitrate;
+	return 0;
+}
+
 int media_stream_get_target_network_bitrate(const MediaStream *stream) {
 	return stream->target_bitrate;
 }
@@ -596,7 +645,7 @@ bool_t ms_media_resource_is_consistent(const MSMediaResource *r){
 			ms_error("Invalid resource type specified");
 			return FALSE;
 	}
-	ms_error("Unsupported media resource type [%i]", (int)r->type);  
+	ms_error("Unsupported media resource type [%i]", (int)r->type);
 	return FALSE;
 }
 
@@ -612,9 +661,7 @@ void video_stream_open_player(VideoStream *stream, MSFilter *sink){
 void video_stream_close_player(VideoStream *stream){
 }
 
-const char *video_stream_get_default_video_renderer(void){
-	return NULL;
-}
+void video_stream_enable_recording(VideoStream *stream, bool_t enabled) {}
 
 MSWebCamDesc *ms_mire_webcam_desc_get(void){
 	return NULL;
@@ -623,47 +670,32 @@ MSWebCamDesc *ms_mire_webcam_desc_get(void){
 #endif
 
 
-static void apply_bitrate_limit(MediaStream *obj, int br_limit){
+static int update_bitrate_limit_from_tmmbr(MediaStream *obj, int br_limit){
 	int previous_br_limit = rtp_session_get_target_upload_bandwidth(obj->sessions.rtp_session);
 	if (!obj->encoder){
-		ms_warning("TMMNR not applicable because no encoder for this stream.");
-		return;
-	}
-	if (previous_br_limit == br_limit) {
-		ms_message("Previous bitrate limit was already %i, skipping...", br_limit);
-		return;
+		ms_warning("TMMBR not applicable because no encoder for this stream.");
+		return -1;
 	}
 
-	if (ms_filter_call_method(obj->encoder,MS_FILTER_SET_BITRATE, &br_limit) != 0){
-		ms_warning("Failed to apply bitrate constraint to %s", obj->encoder->desc->name);
+	if (obj->max_target_bitrate > 0 && br_limit > obj->max_target_bitrate){
+		ms_message("TMMBR is greater than maximum target bitrate set (%i > %i), capping to %i bits/s", br_limit, obj->max_target_bitrate, obj->max_target_bitrate);
+		br_limit = obj->max_target_bitrate;
+	}
+
+	if (previous_br_limit == br_limit) {
+		ms_message("Previous bitrate limit was already %i, skipping...", br_limit);
+		return -1;
+	}
+
+	if (obj->type != MSVideo) {
+		if (ms_filter_call_method(obj->encoder,MS_FILTER_SET_BITRATE, &br_limit) != 0){
+			ms_warning("Failed to apply bitrate constraint to %s", obj->encoder->desc->name);
+		}
 	}
 
 	media_stream_set_target_network_bitrate(obj, br_limit);
 	rtp_session_set_target_upload_bandwidth(obj->sessions.rtp_session, br_limit);
-	
-#ifdef VIDEO_ENABLED
-	if (obj->type == MSVideo) {
-		MSVideoConfiguration *vconf_list = NULL;
-		MSVideoSize vsize;
-		MSVideoConfiguration vconf1, vconf2;
-		
-		ms_filter_call_method(obj->encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, &vconf_list);
-		ms_filter_call_method(obj->encoder, MS_FILTER_GET_VIDEO_SIZE, &vsize);
-
-		vconf1 = ms_video_find_best_configuration_for_size_and_bitrate(vconf_list, vsize, ms_factory_get_cpu_count(obj->factory), previous_br_limit);
-		vconf2 = ms_video_find_best_configuration_for_size_and_bitrate(vconf_list, vsize, ms_factory_get_cpu_count(obj->factory), br_limit);
-		if (vconf1.required_bitrate != vconf2.required_bitrate || vconf1.bitrate_limit != vconf2.bitrate_limit) {
-			ms_message("VideoStream[%p]: bitrate update will change video configuration, recreate graph", obj);
-			video_stream_recreate_graph((VideoStream *)obj);
-		} else {
-			int new_bitrate_limit = br_limit < vconf1.bitrate_limit ? br_limit : vconf1.bitrate_limit;
-			ms_message("VideoStream[%p]: bitrate update won't change video configuration, update limit to %i", obj, new_bitrate_limit);
-			if (ms_filter_call_method(obj->encoder,MS_FILTER_SET_BITRATE, &new_bitrate_limit) != 0){
-				ms_warning("Failed to apply bitrate constraint to %s", obj->encoder->desc->name);
-			}
-		}
-	}
-#endif
+	return br_limit;
 }
 
 static void tmmbr_received(const OrtpEventData *evd, void *user_pointer) {
@@ -671,16 +703,51 @@ static void tmmbr_received(const OrtpEventData *evd, void *user_pointer) {
 	switch (rtcp_RTPFB_get_type(evd->packet)) {
 		case RTCP_RTPFB_TMMBR: {
 			int tmmbr_mxtbr = (int)rtcp_RTPFB_tmmbr_get_max_bitrate(evd->packet);
-			
+
 			ms_message("MediaStream[%p]: received a TMMBR for bitrate %i kbits/s"
 						, ms, (int)(tmmbr_mxtbr/1000));
-			apply_bitrate_limit(ms, tmmbr_mxtbr);
+			tmmbr_mxtbr = update_bitrate_limit_from_tmmbr(ms, tmmbr_mxtbr);
+			if (tmmbr_mxtbr < 0) return;
+
+
+#ifdef VIDEO_ENABLED
+			if (ms->type == MSVideo) {
+				const char* preset = video_stream_get_video_preset((VideoStream*) ms);
+
+				if (preset && strcmp(preset, "custom") == 0) {
+					MSVideoConfiguration *vconf_list = NULL;
+					MSVideoConfiguration current_vconf, vconf;
+					int new_bitrate_limit;
+
+					ms_filter_call_method(ms->encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, &vconf_list);
+
+					if (vconf_list != NULL) {
+						ms_filter_call_method(ms->encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION, &current_vconf);
+
+						vconf = ms_video_find_best_configuration_for_size_and_bitrate(vconf_list, current_vconf.vsize, ms_factory_get_cpu_count(ms->factory), tmmbr_mxtbr);
+
+						new_bitrate_limit = tmmbr_mxtbr < vconf.bitrate_limit ? tmmbr_mxtbr : vconf.bitrate_limit;
+						ms_message("Changing video encoder's output bitrate to %i", new_bitrate_limit);
+						current_vconf.required_bitrate = new_bitrate_limit;
+
+						if (ms_filter_call_method(ms->encoder,MS_VIDEO_ENCODER_SET_CONFIGURATION, &current_vconf) != 0) {
+							ms_warning("Failed to apply fps and bitrate constraint to %s", ms->encoder->desc->name);
+						}
+					}
+
+					break;
+				}
+
+				if (!ms->video_quality_controller) {
+					ms->video_quality_controller = ms_video_quality_controller_new((VideoStream*) ms);
+				}
+
+				ms_video_quality_controller_update_from_tmmbr(ms->video_quality_controller, tmmbr_mxtbr);
+			}
+#endif
 			break;
 		}
 		default:
 			break;
 	}
 }
-
-
-
