@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006-2010  Belledonne Communications SARL (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include "mediastreamer2/msvideo.h"
@@ -35,6 +35,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #define INT32_MAX 017777777777
 #endif
 
+bool_t ms_rect_equal(const MSRect *r1, const MSRect *r2) {
+	return r1->x == r2->x && r1->y == r2->y && r1->w == r2->w && r1->h == r2->h;
+}
+
 const char *ms_pix_fmt_to_string(MSPixFmt fmt){
 	switch(fmt){
 		case MS_YUV420P: return "MS_YUV420P";
@@ -178,7 +182,7 @@ static void plane_copy(const uint8_t *src_plane, size_t src_row_stride, size_t s
 	const uint8_t *r_ptr = src_plane + (src_roi->y * src_row_stride + src_roi->x * src_pix_stride);
 	uint8_t *w_ptr = dst_plane + (dst_roi->y * dst_row_stride + dst_roi->x * dst_pix_stride);
 
-	if (src_row_stride == dst_row_stride && src_pix_stride == 1 && dst_pix_stride == 1 && src_roi == dst_roi) {
+	if (src_row_stride == dst_row_stride && src_pix_stride == 1 && dst_pix_stride == 1 && ms_rect_equal(src_roi, dst_roi)) {
 		memcpy(w_ptr, r_ptr, dst_row_stride * dst_roi->h);
 	} else {
 		int i;
@@ -224,12 +228,18 @@ MSYuvBufAllocator *ms_yuv_buf_allocator_new(void) {
 	return allocator;
 }
 
+void ms_yuv_buf_allocator_set_max_frames(MSYuvBufAllocator *obj, int max_frames){
+	msgb_allocator_set_max_blocks(obj, max_frames);
+}
+
 mblk_t *ms_yuv_buf_allocator_get(MSYuvBufAllocator *obj, MSPicture *buf, int w, int h) {
 	int size=(w * (h & 0x1 ? h+1 : h) *3)/2; /*swscale doesn't like odd numbers of line*/
 	const int header_size = sizeof(mblk_video_header);
 	const int padding=16;
+	mblk_video_header* hdr;
 	mblk_t *msg = msgb_allocator_alloc(obj, header_size + size+padding);
-	mblk_video_header* hdr = (mblk_video_header*)msg->b_wptr;
+	if (msg == NULL) return NULL;
+	hdr = (mblk_video_header*)msg->b_wptr;
 	hdr->w = w;
 	hdr->h = h;
 	msg->b_rptr += header_size;
@@ -643,18 +653,9 @@ static MSScalerDesc *scaler_impl=NULL;
 
 MSScalerContext *ms_scaler_create_context(int src_w, int src_h, MSPixFmt src_fmt,
                                           int dst_w, int dst_h, MSPixFmt dst_fmt, int flags){
-	if (!scaler_impl){
-#if defined(__ANDROID__) && defined(MS_HAS_ARM) && !defined(__aarch64__)
-		if (android_getCpuFamily() == ANDROID_CPU_FAMILY_ARM && (android_getCpuFeatures() & ANDROID_CPU_ARM_FEATURE_NEON) != 0){
-			scaler_impl = &ms_android_scaler;
-		}
-#elif !defined(NO_FFMPEG)
-		scaler_impl=&ffmpeg_scaler;
-#endif
-	}
-	
-	if (scaler_impl)
-		return scaler_impl->create_context(src_w,src_h,src_fmt,dst_w,dst_h,dst_fmt, flags);
+	MSScalerDesc *impl = ms_video_get_scaler_impl();
+	if (impl)
+		return impl->create_context(src_w,src_h,src_fmt,dst_w,dst_h,dst_fmt, flags);
 	ms_fatal("No scaler implementation built-in, please supply one with ms_video_set_scaler_impl ()");
 	return NULL;
 }
@@ -671,6 +672,18 @@ void ms_video_set_scaler_impl(MSScalerDesc *desc){
 	scaler_impl=desc;
 }
 
+MSScalerDesc * ms_video_get_scaler_impl(void){
+	if (scaler_impl) return scaler_impl;
+#if defined(__ANDROID__) && defined(MS_HAS_ARM) && !defined(__aarch64__)
+	if (android_getCpuFamily() == ANDROID_CPU_FAMILY_ARM && (android_getCpuFeatures() & ANDROID_CPU_ARM_FEATURE_NEON) != 0){
+		scaler_impl = &ms_android_scaler;
+	}
+#elif !defined(NO_FFMPEG)
+	scaler_impl=&ffmpeg_scaler;
+#endif
+	return scaler_impl;
+}
+
 /* Can rotate Y, U or V plane; use step=2 for interleaved UV planes otherwise step=1*/
 static void rotate_plane_down_scale_by_2(int wDest, int hDest, int full_width, const uint8_t* src, uint8_t* dst, int step, bool_t clockWise,bool_t downscale) {
 	int factor = downscale?2:1;
@@ -840,6 +853,75 @@ mblk_t *copy_ycbcrbiplanar_to_true_yuv_with_rotation(MSYuvBufAllocator *allocato
 	return copy_ycbcrbiplanar_to_true_yuv_with_rotation_and_down_scale_by_2(allocator, y, cbcr, rotation, w, h, y_byte_per_row, cbcr_byte_per_row, uFirstvSecond, FALSE);
 }
 
+mblk_t *copy_yuv_with_rotation(MSYuvBufAllocator *allocator, const uint8_t* y, const uint8_t* u, const uint8_t* v, int rotation, int w, int h, int y_byte_per_row, int u_byte_per_row, int v_byte_per_row) {
+	MSPicture pict;
+	int uv_w=w/2;
+	int uv_h=h/2;
+	mblk_t * yuv_block;
+	
+	yuv_block = ms_yuv_buf_allocator_get(allocator, &pict, w, h);
+	
+	if (rotation % 180 == 0) {
+		int i,j;
+
+		if (rotation == 0) {
+			// plain y copy
+			for(i=0; i<h; i++) {
+				memcpy(&pict.planes[0][i*w], &y[i*y_byte_per_row], w);
+			}
+			// plain u/v copy
+			for (i=0; i<uv_h; i++) {
+				memcpy(&pict.planes[1][i*uv_w], &u[i*u_byte_per_row], uv_w);
+				memcpy(&pict.planes[2][i*uv_w], &v[i*v_byte_per_row], uv_w);
+			}
+		} else {
+			// 180° y rotation
+			for(i=0; i<h; i++) {
+				for(j=0 ; j<w;j++) {
+					pict.planes[0][i*w+j] = y[(h-1-i)*y_byte_per_row+(w-1-j)];
+				}
+			}
+			// 180° u/v rotation
+			for (i=0; i<uv_h; i++) {
+				for(j=0; j<uv_w; j++) {
+					pict.planes[1][i*uv_w+j] = u[(uv_h-1-i)*u_byte_per_row+(uv_w-1-j)];
+					pict.planes[2][i*uv_w+j] = v[(uv_h-1-i)*v_byte_per_row+(uv_w-1-j)];
+				}
+			}
+		}
+	} else {
+		bool_t clockwise = rotation == 90 ? TRUE : FALSE;
+		// Rotate Y/U/V
+#if defined(__arm__)
+		if (hasNeon) {
+			if (clockwise) {
+				rotate_down_scale_plane_neon_clockwise(w,h,y_byte_per_row,(uint8_t*)y,pict.planes[0],FALSE);
+				rotate_down_scale_plane_neon_clockwise(uv_w,uv_h,u_byte_per_row,(uint8_t*)u,pict.planes[1],FALSE);
+				rotate_down_scale_plane_neon_clockwise(uv_w,uv_h,v_byte_per_row,(uint8_t*)v,pict.planes[2],FALSE);
+			} else {
+				rotate_down_scale_plane_neon_anticlockwise(w,h,y_byte_per_row,(uint8_t*)y,pict.planes[0],FALSE);
+				rotate_down_scale_plane_neon_anticlockwise(uv_w,uv_h,u_byte_per_row,(uint8_t*)u,pict.planes[1],FALSE);
+				rotate_down_scale_plane_neon_anticlockwise(uv_w,uv_h,v_byte_per_row,(uint8_t*)v,pict.planes[2],FALSE);
+			}
+		} else
+#endif
+		{
+			uint8_t* dsty = pict.planes[0];
+			uint8_t* srcy = (uint8_t*) y;
+			uint8_t* dstu = pict.planes[1];
+			uint8_t* srcu = (uint8_t*) u;
+			uint8_t* dstv = pict.planes[2];
+			uint8_t* srcv = (uint8_t*) v;
+			
+			rotate_plane_down_scale_by_2(w,h,y_byte_per_row,srcy,dsty,1,clockwise,FALSE);
+			rotate_plane_down_scale_by_2(uv_w,uv_h,u_byte_per_row,srcu,dstu,1,clockwise,FALSE);
+			rotate_plane_down_scale_by_2(uv_w,uv_h,v_byte_per_row,srcv,dstv,1,clockwise,FALSE);
+		}
+	}
+
+	return yuv_block;
+}
+
 void ms_video_init_framerate_controller(MSFrameRateController* ctrl, float fps) {
 	ctrl->start_time = 0;
 	ctrl->th_frame_count = -1;
@@ -970,15 +1052,13 @@ MSVideoConfiguration ms_video_find_best_configuration_for_size(const MSVideoConf
 	return best_vconf;
 }
 
-MSVideoConfiguration ms_video_find_best_configuration_for_size_and_bitrate(const MSVideoConfiguration *vconf_list, MSVideoSize vsize, int cpu_count, int bitrate) {
+MSVideoConfiguration ms_video_find_worst_configuration_for_size(const MSVideoConfiguration *vconf_list, MSVideoSize vsize, int cpu_count) {
 	const MSVideoConfiguration *vconf_it = vconf_list;
 	MSVideoConfiguration best_vconf={0};
 	int min_score=INT32_MAX;
 	int ref_pixels=vsize.height*vsize.width;
 
-	if (bitrate == 0) return ms_video_find_best_configuration_for_size(vconf_list, vsize, cpu_count);
-
-	/* search for configuration that is first nearest to target video size, then target bitrate and finally has the greater fps,
+	/* search for configuration that is first nearest to target video size, then second has the lowest bitrate but the greater fps,
 	 * but any case making sure the the cpu count is sufficient*/
 	while(TRUE) {
 		int pixels=vconf_it->vsize.width*vconf_it->vsize.height;
@@ -987,8 +1067,12 @@ MSVideoConfiguration ms_video_find_best_configuration_for_size_and_bitrate(const
 			if (score<min_score){
 				best_vconf=*vconf_it;
 				min_score=score;
-			}else if (score==min_score) {
-				if (bitrate <= vconf_it->bitrate_limit && bitrate >= vconf_it->required_bitrate) {
+			}else if (score==min_score){
+				if (best_vconf.required_bitrate == vconf_it->required_bitrate && best_vconf.bitrate_limit == vconf_it->bitrate_limit){
+					if (best_vconf.fps<vconf_it->fps) {
+						best_vconf=*vconf_it;
+					}
+				} else {
 					best_vconf=*vconf_it;
 				}
 			}
@@ -999,6 +1083,60 @@ MSVideoConfiguration ms_video_find_best_configuration_for_size_and_bitrate(const
 		vconf_it++;
 
 	}
-	best_vconf.vsize = vsize;
+	best_vconf.vsize=vsize;
 	return best_vconf;
 }
+
+MSVideoConfiguration ms_video_find_best_configuration_for_size_and_bitrate(const MSVideoConfiguration *vconf_list, MSVideoSize vsize, int cpu_count, int bitrate) {
+	const MSVideoConfiguration *vconf_it = vconf_list;
+	MSVideoConfiguration best_vconf={0};
+	MSVideoConfiguration *last_good_vconf = NULL;
+	int min_score=INT32_MAX;
+	int ref_pixels=vsize.height*vsize.width;
+
+	if (bitrate == 0) return ms_video_find_best_configuration_for_size(vconf_list, vsize, cpu_count);
+
+	/* Search for configuration that is first nearest to target video size, then target bitrate and finally has the greater fps,
+	 * but any case making sure the cpu count is sufficient. 
+	 * We suppose that the configuration list is ordered. */
+	while(TRUE) {
+		int pixels=vconf_it->vsize.width*vconf_it->vsize.height;
+		int score=abs(pixels-ref_pixels);
+		if (cpu_count>=vconf_it->mincpu){
+			if (score<min_score) {
+				best_vconf=*vconf_it;
+				min_score=score;
+				
+				if (bitrate <= vconf_it->bitrate_limit && bitrate >= vconf_it->required_bitrate)
+					last_good_vconf=&best_vconf;
+				else
+					last_good_vconf = NULL;
+			} else if (score==min_score) {
+				if (bitrate <= vconf_it->bitrate_limit && bitrate >= vconf_it->required_bitrate) {
+					if (last_good_vconf == NULL || last_good_vconf->fps < vconf_it->fps) {
+						best_vconf=*vconf_it;
+						last_good_vconf=&best_vconf;
+					}
+				} else if (bitrate < vconf_it->required_bitrate) {
+					best_vconf=*vconf_it;
+					last_good_vconf=NULL;
+				}
+			}
+		}
+		if (vconf_it->required_bitrate==0) {
+			break;
+		}
+		vconf_it++;
+	}
+	last_good_vconf = NULL;
+	best_vconf.vsize = vsize;
+
+	ms_message("Best video configuration for %dbits/s: rb=%d, bl=%d, fps=%f, vsize=%dx%d, mincpu=%d", bitrate, best_vconf.required_bitrate, best_vconf.bitrate_limit, best_vconf.fps, best_vconf.vsize.width, best_vconf.vsize.height, best_vconf.mincpu);
+
+	return best_vconf;
+}
+
+bool_t ms_video_configuratons_equal(const MSVideoConfiguration *vconf1, const MSVideoConfiguration *vconf2) {
+	if (vconf1 == NULL || vconf2 == NULL) return 0;
+	return (vconf1->required_bitrate == vconf2->required_bitrate && vconf1->bitrate_limit == vconf2->bitrate_limit && vconf1->fps == vconf2->fps && vconf1->mincpu == vconf2->mincpu);
+}
