@@ -1,24 +1,25 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2012-2016  Belledonne Communications
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include <mediastreamer2/stun.h>
+#include <bctoolbox/compiler.h>
 #include <bctoolbox/crypto.h>
 #include <bctoolbox/port.h>
 
@@ -198,7 +199,7 @@ static void encode_string(StunMessageEncoder *encoder, uint16_t type, const char
 
 static void encode_error_code(StunMessageEncoder *encoder, uint16_t number, const char *reason) {
 	size_t reason_len = 0;
-	size_t padding = 4 - (reason_len % 4);
+	size_t padding;
 	if (reason != NULL) reason_len = strlen(reason);
 	encode16(encoder, MS_STUN_ATTR_ERROR_CODE);
 	encode16(encoder, 4 + (uint16_t)reason_len);
@@ -206,6 +207,7 @@ static void encode_error_code(StunMessageEncoder *encoder, uint16_t number, cons
 	encode8(encoder, number / 100);
 	encode8(encoder, number - ((number / 100) * 100));
 	if (reason != NULL) encode(encoder, reason, reason_len);
+	padding = 4 - (reason_len % 4);
 	if (padding < 4) {
 		size_t i;
 		for (i = 0; i < padding; i++) encode8(encoder, 0);
@@ -358,7 +360,7 @@ static uint32_t decode32(StunMessageDecoder *decoder) {
 	return value;
 }
 
-static uint64_t decode64(StunMessageDecoder *decoder) {
+BCTBX_DISABLE_UBSAN static uint64_t decode64(StunMessageDecoder *decoder) {
 	uint64_t value = ntohq(*((uint64_t *)decoder->ptr));
 	decoder->ptr += sizeof(uint64_t);
 	decoder->remaining -= sizeof(uint64_t);
@@ -1335,24 +1337,67 @@ void ms_turn_context_destroy(MSTurnContext *context) {
 	if (context->endpoint != NULL) context->endpoint->data = NULL;
 	bctbx_list_for_each(context->allowed_peer_addresses, (MSIterateFunc)ms_free);
 	bctbx_list_free(context->allowed_peer_addresses);
+	if (context->turn_tcp_client) ms_turn_tcp_client_destroy(context->turn_tcp_client);
+	if (context->cn) ms_free(context->cn);
+	if (context->root_certificate) ms_free(context->root_certificate);
 	ms_free(context);
 }
 
 void ms_turn_context_set_server_addr(MSTurnContext *context, struct sockaddr *addr, socklen_t addrlen) {
-	context->turn_server_addr = addr;
-	context->turn_server_addrlen = addrlen;
+	// The media sockets are bound in IPv6 so convert the TURN server destination address to IPv6
+	// so that we do not get errors on Mac OS X
+	if ((addr->sa_family == AF_INET) && (context->rtp_session->rtp.gs.sockfamily == AF_INET6)) {
+		bctbx_sockaddr_ipv4_to_ipv6(addr, (struct sockaddr *)&context->turn_server_addr, &context->turn_server_addrlen);
+	} else {
+		memcpy(&context->turn_server_addr, addr, addrlen);
+		context->turn_server_addrlen = addrlen;
+	}
 }
 
 MSTurnContextState ms_turn_context_get_state(const MSTurnContext *context) {
 	return context->state;
 }
 
+static const char *ms_turn_context_state_to_string(MSTurnContextState state){
+	switch(state){
+		case MS_TURN_CONTEXT_STATE_IDLE: return "IDLE";
+		case MS_TURN_CONTEXT_STATE_CREATING_ALLOCATION: return "CREATING_ALLOCATION";
+		case MS_TURN_CONTEXT_STATE_ALLOCATION_CREATED: return "ALLOCATIION_CREATED";
+		case MS_TURN_CONTEXT_STATE_CREATING_PERMISSIONS: return "CREATING_PERMISSIONS";
+		case MS_TURN_CONTEXT_STATE_PERMISSIONS_CREATED: return "PERMISSIONS_CREATED";
+		case MS_TURN_CONTEXT_STATE_BINDING_CHANNEL: return "BINDING_CHANNEL";
+		case MS_TURN_CONTEXT_STATE_CHANNEL_BOUND: return "CHANNEL_BOUND";
+	}
+	return "BAD_STATE";
+}
+
 void ms_turn_context_set_state(MSTurnContext *context, MSTurnContextState state) {
+	ms_message("ms_turn_context_set_state(): context=%p, type=%s, state=%s", context, context->type == MS_TURN_CONTEXT_TYPE_RTP ? "RTP" : "RTCP", ms_turn_context_state_to_string(state));
 	context->state = state;
 	if (state == MS_TURN_CONTEXT_STATE_ALLOCATION_CREATED) context->stats.nb_successful_allocate++;
 	else if (state == MS_TURN_CONTEXT_STATE_CHANNEL_BOUND) context->stats.nb_successful_channel_bind++;
 }
 
+MSTurnContextTransport ms_turn_get_transport_from_string(const char *transport) {
+	if (transport == NULL) return MS_TURN_CONTEXT_TRANSPORT_UDP;
+
+	if (strcmp(transport, "tcp") == 0) {
+		return MS_TURN_CONTEXT_TRANSPORT_TCP;
+	} else if (strcmp(transport, "tls") == 0) {
+		return MS_TURN_CONTEXT_TRANSPORT_TLS;
+	} else {
+		return MS_TURN_CONTEXT_TRANSPORT_UDP;
+	}
+}
+
+MSTurnContextTransport ms_turn_context_get_transport(const MSTurnContext *context) {
+	return context->transport;
+}
+
+void ms_turn_context_set_transport(MSTurnContext *context, MSTurnContextTransport transport) {
+	context->transport = transport;
+}
+
 const char * ms_turn_context_get_realm(const MSTurnContext *context) {
 	return context->realm;
 }
@@ -1417,6 +1462,14 @@ void ms_turn_context_set_force_rtp_sending_via_relay(MSTurnContext *context, boo
 	context->force_rtp_sending_via_relay = force;
 }
 
+void ms_turn_context_set_root_certificate(MSTurnContext *context, const char *root_certificate) {
+	STUN_STR_SETTER(context->root_certificate, root_certificate);
+}
+
+void ms_turn_context_set_cn(MSTurnContext *context, const char *cn) {
+	STUN_STR_SETTER(context->cn, cn);
+}
+
 bool_t ms_turn_context_peer_address_allowed(const MSTurnContext *context, const MSStunAddress *peer_address) {
 	bctbx_list_t *elem = context->allowed_peer_addresses;
 	while (elem != NULL) {
@@ -1441,12 +1494,22 @@ static int ms_turn_rtp_endpoint_recvfrom(RtpTransport *rtptp, mblk_t *msg, int f
 	int msgsize = 0;
 
 	if ((context != NULL) && (context->rtp_session != NULL)) {
-		msgsize = rtp_session_recvfrom(context->rtp_session, context->type == MS_TURN_CONTEXT_TYPE_RTP, msg, flags, from, fromlen);
+		// Check first if we received a message from turn tcp
+		if (context->transport != MS_TURN_CONTEXT_TRANSPORT_UDP && context->turn_tcp_client) {
+			msgsize = ms_turn_tcp_client_recvfrom(context->turn_tcp_client, msg, flags, from, fromlen);
+		}
+
+		// If not use the common way
+		if (msgsize == 0) {
+			msgsize = rtp_session_recvfrom(context->rtp_session, context->type == MS_TURN_CONTEXT_TYPE_RTP, msg, flags, from, fromlen);
+		}
+
 		if ((msgsize >= RTP_FIXED_HEADER_SIZE) && (rtp_get_version(msg) != 2)) {
 			/* This is not a RTP packet, try to see if it is a TURN ChannelData message */
 			if ((ms_turn_context_get_state(context) >= MS_TURN_CONTEXT_STATE_BINDING_CHANNEL) && (*msg->b_rptr & 0x40)) {
 				uint16_t channel = ntohs(*((uint16_t *)msg->b_rptr));
 				uint16_t datasize = ntohs(*(((uint16_t *)msg->b_rptr) + 1));
+
 				if ((channel == ms_turn_context_get_channel_number(context)) && (msgsize >= (datasize + 4))) {
 					msg->b_rptr += 4; /* Unpack the TURN ChannelData message */
 					context->stats.nb_received_channel_msg++;
@@ -1501,7 +1564,7 @@ static int ms_turn_rtp_endpoint_recvfrom(RtpTransport *rtptp, mblk_t *msg, int f
 	return msgsize;
 }
 
-static bool_t ms_turn_rtp_endpoint_send_via_turn_server(MSTurnContext *context, const struct sockaddr *from, socklen_t fromlen) {
+static bool_t ms_turn_rtp_endpoint_send_via_turn_relay(MSTurnContext *context, const struct sockaddr *from, socklen_t fromlen) {
 	struct sockaddr_storage relay_ss;
 	struct sockaddr *relay_sa = (struct sockaddr *)&relay_ss;
 	socklen_t relay_sa_len = sizeof(relay_ss);
@@ -1516,7 +1579,22 @@ static bool_t ms_turn_rtp_endpoint_send_via_turn_server(MSTurnContext *context,
 	} else if (relay_sa->sa_family == AF_INET6) {
 		struct sockaddr_in6 *relay_sa_in6 = (struct sockaddr_in6 *)relay_sa;
 		struct sockaddr_in6 *from_in6 = (struct sockaddr_in6 *)from;
-		return (relay_sa_in6->sin6_port == from_in6->sin6_port) && (memcmp(&relay_sa_in6->sin6_addr, &from_in6->sin6_addr, sizeof(struct in6_addr)) == 0);
+		return (relay_sa_in6->sin6_port == from_in6->sin6_port) && (memcmp(&relay_sa_in6->sin6_addr, &from_in6->sin6_addr, sizeof(from_in6->sin6_addr)) == 0);
+	} else return FALSE;
+}
+
+static bool_t ms_turn_rtp_endpoint_send_via_turn_server(MSTurnContext *context, const struct sockaddr *from, socklen_t fromlen) {
+	struct sockaddr *turn_server_sa = (struct sockaddr *)&context->turn_server_addr;
+
+	if (turn_server_sa->sa_family != from->sa_family) return FALSE;
+	if (turn_server_sa->sa_family == AF_INET) {
+		struct sockaddr_in *turn_server_sa_in = (struct sockaddr_in *)turn_server_sa;
+		struct sockaddr_in *from_in = (struct sockaddr_in *)from;
+		return (turn_server_sa_in->sin_port == from_in->sin_port) && (turn_server_sa_in->sin_addr.s_addr == from_in->sin_addr.s_addr);
+	} else if (turn_server_sa->sa_family == AF_INET6) {
+		struct sockaddr_in6 *turn_server_sa_in6 = (struct sockaddr_in6 *)turn_server_sa;
+		struct sockaddr_in6 *from_in6 = (struct sockaddr_in6 *)from;
+		return (turn_server_sa_in6->sin6_port == from_in6->sin6_port) && (memcmp(&turn_server_sa_in6->sin6_addr, &from_in6->sin6_addr, sizeof(from_in6->sin6_addr)) == 0);
 	} else return FALSE;
 }
 
@@ -1524,12 +1602,16 @@ static int ms_turn_rtp_endpoint_sendto(RtpTransport *rtptp, mblk_t *msg, int fla
 	MSTurnContext *context = (MSTurnContext *)rtptp->data;
 	MSStunMessage *stun_msg = NULL;
 	bool_t rtp_packet = FALSE;
+	bool_t send_via_turn_tcp = FALSE;
 	int ret = 0;
 	mblk_t *new_msg = NULL;
+	struct sockaddr_storage sourceAddr;
+	socklen_t sourceAddrLen;
 
 	if ((context != NULL) && (context->rtp_session != NULL)) {
 		if ((msgdsize(msg) >= RTP_FIXED_HEADER_SIZE) && (rtp_get_version(msg) == 2)) rtp_packet = TRUE;
-		if ((rtp_packet && context->force_rtp_sending_via_relay) || ms_turn_rtp_endpoint_send_via_turn_server(context, (struct sockaddr *)&msg->net_addr, msg->net_addrlen)) {
+		ortp_recvaddr_to_sockaddr(&msg->recv_addr, (struct sockaddr*) &sourceAddr, &sourceAddrLen);
+		if ((rtp_packet && context->force_rtp_sending_via_relay) || ms_turn_rtp_endpoint_send_via_turn_relay(context, (struct sockaddr*) &sourceAddr, sourceAddrLen)) {
 			if (ms_turn_context_get_state(context) >= MS_TURN_CONTEXT_STATE_CHANNEL_BOUND) {
 				/* Use a TURN ChannelData message */
 				new_msg = allocb(4, 0);
@@ -1564,10 +1646,26 @@ static int ms_turn_rtp_endpoint_sendto(RtpTransport *rtptp, mblk_t *msg, int fla
 				ms_free(buf);
 				context->stats.nb_send_indication++;
 			}
-			to = (const struct sockaddr *)context->turn_server_addr;
+			to = (const struct sockaddr *)&context->turn_server_addr;
 			tolen = context->turn_server_addrlen;
+
+			if (context->transport != MS_TURN_CONTEXT_TRANSPORT_UDP) {
+				send_via_turn_tcp = TRUE;
+			}
+		} else if (ms_turn_rtp_endpoint_send_via_turn_server(context, to, tolen)) {
+			// This is not a RTP packet and it is in destination of the turn server
+			// then it's a STUN packet and it still should go through the turn TCP client
+			// if the transport is not set to UDP
+			if (context->transport != MS_TURN_CONTEXT_TRANSPORT_UDP) {
+				send_via_turn_tcp = TRUE;
+			}
+		}
+
+		if (send_via_turn_tcp && context->turn_tcp_client) {
+			ret = ms_turn_tcp_client_sendto(context->turn_tcp_client, msg, flags, to, tolen);
+		} else {
+			ret = rtp_session_sendto(context->rtp_session, context->type == MS_TURN_CONTEXT_TYPE_RTP, msg, flags, to, tolen);
 		}
-		ret = rtp_session_sendto(context->rtp_session, context->type == MS_TURN_CONTEXT_TYPE_RTP, msg, flags, to, tolen);
 	}
 	if (stun_msg != NULL) ms_stun_message_destroy(stun_msg);
 	if (new_msg != NULL) {
