@@ -1,29 +1,29 @@
 /*
-vp8rtpfmt.c
-
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2011-2012 Belledonne Communications, Grenoble, France
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include "vp8rtpfmt.h"
 
 #include <limits.h>
 
+static const int max_non_reference_frame_after_sli_or_pli = 60;
+
 /*#define VP8RTPFMT_DEBUG*/
 
 #if (defined(__GNUC__) && __GNUC__) || defined(__SUNPRO_C)
@@ -404,6 +404,7 @@ static MSVideoSize get_size_from_key_frame(Vp8RtpFmtFrame *frame) {
 
 static void send_pli(Vp8RtpFmtUnpackerCtx *ctx) {
 	if (ctx->avpf_enabled == TRUE) {
+		ctx->sli_pending = FALSE;
 		if(ctx->filter) ms_filter_notify_no_arg(ctx->filter, MS_VIDEO_DECODER_SEND_PLI);
 	} else {
 		if(ctx->filter) ms_filter_notify_no_arg(ctx->filter, MS_VIDEO_DECODER_DECODING_ERRORS);
@@ -411,14 +412,24 @@ static void send_pli(Vp8RtpFmtUnpackerCtx *ctx) {
 	}
 }
 
+static void send_fir(Vp8RtpFmtUnpackerCtx *ctx) {
+	if (ctx->avpf_enabled == TRUE) {
+		ctx->sli_pending = FALSE;
+		if(ctx->filter) ms_filter_notify_no_arg(ctx->filter, MS_VIDEO_DECODER_SEND_FIR);
+	} else {
+		if(ctx->filter) ms_filter_notify_no_arg(ctx->filter, MS_VIDEO_DECODER_DECODING_ERRORS);
+		ctx->error_notified = TRUE;
+	}
+}
+
 static void send_sli(Vp8RtpFmtUnpackerCtx *ctx, Vp8RtpFmtFrame *frame) {
 	if (ctx->avpf_enabled == TRUE) {
 		if (frame->pictureid_present == TRUE) {
-			MSVideoCodecSLI sli;
-			sli.first = 0;
-			sli.number = (ctx->video_size.width * ctx->video_size.height) / (16 * 16);
-			sli.picture_id = frame->pictureid & 0x3F;
-			if(ctx->filter) ms_filter_notify(ctx->filter, MS_VIDEO_DECODER_SEND_SLI, &sli);
+			ctx->current_sli.first = 0;
+			ctx->current_sli.number = (ctx->video_size.width * ctx->video_size.height) / (16 * 16);
+			ctx->current_sli.picture_id = frame->pictureid & 0x3F;
+			ctx->sli_pending = TRUE;
+			if(ctx->filter) ms_filter_notify(ctx->filter, MS_VIDEO_DECODER_SEND_SLI, &ctx->current_sli);
 		} else {
 			send_pli(ctx);
 		}
@@ -618,14 +629,14 @@ static void notify_frame_error_if_any(Vp8RtpFmtUnpackerCtx *ctx, Vp8RtpFmtFrame
 static void add_frame(Vp8RtpFmtUnpackerCtx *ctx, bctbx_list_t **packets_list) {
 	Vp8RtpFmtFrame *frame;
 
-	if (bctbx_list_size(*packets_list) > 0) {
+	if (*packets_list != NULL) {
 		frame = ms_new0(Vp8RtpFmtFrame, 1);
 		generate_frame_partitions_list(frame, *packets_list);
 		check_frame_partitions_list(ctx, frame);
 		notify_frame_error_if_any(ctx, frame);
 		ctx->frames_list = bctbx_list_append(ctx->frames_list, (void *)frame);
+		bctbx_list_free(*packets_list);
 	}
-	bctbx_list_free(*packets_list);
 	*packets_list = NULL;
 }
 
@@ -636,11 +647,9 @@ static void generate_frames_list(Vp8RtpFmtUnpackerCtx *ctx, bctbx_list_t *packet
 	uint32_t ts;
 
 	/* If we have some packets from the previous iteration, put them in the frame_packets_list. */
-	for (it=ctx->non_processed_packets_list; it!=NULL; it=it->next) {
-		packet = (Vp8RtpFmtPacket*) it->data;
-		frame_packets_list = bctbx_list_append(frame_packets_list, packet);
+	if (ctx->non_processed_packets_list){
+		frame_packets_list = bctbx_list_concat(ctx->non_processed_packets_list, frame_packets_list);
 	}
-	bctbx_list_free(ctx->non_processed_packets_list);
 	ctx->non_processed_packets_list = NULL;
 
 	/* Process newly received packets. */
@@ -724,7 +733,7 @@ static void output_partitions_of_frame(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *out,
 static int output_valid_partitions(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *out) {
 	Vp8RtpFmtFrame *frame;
 	size_t nb_frames = bctbx_list_size(ctx->frames_list);
-
+	bool_t pli_sent = FALSE;
 	if (nb_frames == 0) return -1;
 	frame = (Vp8RtpFmtFrame *)bctbx_list_nth_data(ctx->frames_list, 0);
 	switch (frame->error) {
@@ -733,6 +742,7 @@ static int output_valid_partitions(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *out) {
 				ctx->valid_keyframe_received = TRUE;
 				ctx->video_size = get_size_from_key_frame(frame);
 				ctx->waiting_for_reference_frame = FALSE;
+				ctx->waiting_for_reference_frame_count = 0;
 				if (ctx->error_notified == TRUE) {
 					if(ctx->filter) ms_filter_notify_no_arg(ctx->filter, MS_VIDEO_DECODER_RECOVERED_FROM_ERRORS);
 					ctx->error_notified = FALSE;
@@ -740,6 +750,8 @@ static int output_valid_partitions(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *out) {
 			}
 			if ((ctx->avpf_enabled == TRUE) && (frame->reference == TRUE)) {
 				ctx->waiting_for_reference_frame = FALSE;
+				ctx->waiting_for_reference_frame_count = 0;
+				ctx->sli_pending = FALSE;
 			}
 			if ((ctx->valid_keyframe_received == TRUE) && (ctx->waiting_for_reference_frame == FALSE)) {
 				/* Output the complete valid frame if a reference frame has been received. */
@@ -752,13 +764,31 @@ static int output_valid_partitions(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *out) {
 				frame->outputted = TRUE;
 			} else {
 				frame->discarded = TRUE;
-				if (!ctx->valid_keyframe_received) send_pli(ctx);
+				if (!ctx->valid_keyframe_received) {
+					/*We send a FIR because:
+					 * in some case the remote encoder thinks that AF and GF are acknoledge and then will create recovery frame based on one of them (or both)
+					 * HOWEVER, the local decoder (if has just been re-instancianted) had no KF at all to start decoding.*/
+					send_fir(ctx);
+					send_pli(ctx);
+					pli_sent = TRUE;
+				}
 				if (ctx->waiting_for_reference_frame == TRUE) {
 					/* Do not decode frames while we are waiting for a reference frame. */
 					if (frame->pictureid_present == TRUE)
 						ms_warning("VP8 decoder: Drop frame because we are waiting for reference frame: pictureID=%i", (int)frame->pictureid);
 					else
 						ms_warning("VP8 decoder: Drop frame because we are waiting for reference frame.");
+					ctx->waiting_for_reference_frame_count++;
+					if (!pli_sent){
+						if (ctx->waiting_for_reference_frame_count > max_non_reference_frame_after_sli_or_pli){
+							/* Workaround/security: if SLI has no effect after a certain number of received frames, use PLI. */
+							ms_warning("VP8 decoder: requesting PLI, reference frame still not received.");
+							if (ctx->filter) ms_filter_notify_no_arg(ctx->filter, MS_VIDEO_DECODER_SEND_PLI);
+						}else if (ctx->sli_pending){
+							/* retransmit last SLI */
+							if (ctx->filter) ms_filter_notify(ctx->filter, MS_VIDEO_DECODER_SEND_SLI, &ctx->current_sli);
+						}
+					}
 				} else {
 					/* Drop frames until the first keyframe is successfully received. */
 					ms_warning("VP8 frame dropped because keyframe has not been received yet.");
@@ -904,6 +934,8 @@ void vp8rtpfmt_unpacker_init(Vp8RtpFmtUnpackerCtx *ctx, MSFilter *f, bool_t avpf
 	ctx->error_notified = FALSE;
 	ctx->initialized_last_ts = FALSE;
 	ctx->initialized_ref_cseq = FALSE;
+	ctx->waiting_for_reference_frame_count = 0;
+	ctx->sli_pending = 0;
 }
 
 void vp8rtpfmt_unpacker_uninit(Vp8RtpFmtUnpackerCtx *ctx) {
@@ -956,7 +988,7 @@ int vp8rtpfmt_unpacker_get_frame(Vp8RtpFmtUnpackerCtx *ctx, MSQueue *out, Vp8Rtp
 		frame_info->pictureid_present = frame->pictureid_present;
 		frame_info->pictureid = frame->pictureid;
 		frame_info->keyframe = frame->keyframe;
-	} else if (bctbx_list_size(ctx->non_processed_packets_list) > 0) {
+	} else if (ctx->non_processed_packets_list != NULL) {
 		ms_debug("VP8 packets are remaining for next iteration of the filter.");
 	}
 	clean_frame(ctx);
