@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006-2013 Belledonne Communications, Grenoble
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/dtmfgen.h"
@@ -97,7 +97,7 @@ typedef struct _stream_manager_t {
 stream_manager_t * stream_manager_new(MSFormatType type) {
 	stream_manager_t * mgr = ms_new0(stream_manager_t,1);
 	mgr->type=type;
-	mgr->local_rtp=(rand() % ((2^16)-1024) + 1024) & ~0x1;
+	mgr->local_rtp=(rand() % ((1<<16)-1024) + 1024) & ~0x1;
 	mgr->local_rtcp=mgr->local_rtp+1;
 	mgr->user_data = 0;
 
@@ -214,7 +214,7 @@ void start_adaptive_stream(MSFormatType type, stream_manager_t ** pmarielle, str
 	char* recorded_file = NULL;
 	char* file = bc_tester_res(HELLO_16K_1S_FILE);
 #if VIDEO_ENABLED
-	MSWebCam * marielle_webcam=mediastreamer2_tester_get_mire_webcam(ms_factory_get_web_cam_manager(_factory));
+	MSWebCam * marielle_webcam=mediastreamer2_tester_get_mire(_factory);
 #endif
 
 	stream_manager_t *marielle=*pmarielle=stream_manager_new(type);
@@ -265,7 +265,7 @@ void start_adaptive_stream(MSFormatType type, stream_manager_t ** pmarielle, str
 		OrtpVideoBandwidthEstimatorParams params = {0};
 		marielle->video_stream->staticimage_webcam_fps_optimization = FALSE;
 		video_manager_start(marielle,payload,margaux->local_rtp,0,marielle_webcam);
-		video_stream_set_direction(margaux->video_stream, MediaStreamRecvOnly);
+		media_stream_set_direction(&margaux->video_stream->ms, MediaStreamRecvOnly);
 		ms_bandwidth_controller_add_stream(margaux->bw_controller, margaux_video_stream);
 		params.packet_count_min = 5;
 		params.packets_size_max = 5;
@@ -426,6 +426,8 @@ static void loss_rate_estimation_bv16(void){
 		LossRateEstimatorCtx ctx;
 		stream_manager_t * marielle, * margaux;
 		int loss_rate = 15;
+		char *plc_filename;
+		char *no_plc_filename;
 		for (plc_disabled = 0; plc_disabled <=1; plc_disabled++){
 			start_adaptive_stream(MSAudio, &marielle, &margaux, BV16_PAYLOAD_TYPE, 8000, 0, (float)loss_rate, 0, 0.0f, (bool_t)plc_disabled);
 
@@ -442,8 +444,12 @@ static void loss_rate_estimation_bv16(void){
 			ortp_ev_queue_destroy(ctx.q);
 		}
 		/*file without plc must be approx 15% smaller in size than with plc */
-		size_with_plc= fsize(bc_tester_file(RECORDED_16K_1S_FILE));
-		size_no_plc = fsize(bc_tester_file(RECORDED_16K_1S_NO_PLC_FILE));
+		plc_filename = bc_tester_file(RECORDED_16K_1S_FILE);
+		size_with_plc= fsize(plc_filename);
+		bctbx_free(plc_filename);
+		no_plc_filename = bc_tester_file(RECORDED_16K_1S_NO_PLC_FILE);
+		size_no_plc = fsize(no_plc_filename);
+		bctbx_free(no_plc_filename);
 		result = (size_with_plc*loss_rate/100 + size_no_plc) ;
 
 		BC_ASSERT_GREATER(result, size_with_plc, int, "%d");
