@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006-2013 Belledonne Communications, Grenoble
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/msrtp.h"
@@ -38,7 +38,6 @@ static MSFactory* _factory = NULL;
 static CodecsManager *_h264_codecs_manager = NULL;
 
 
-
 MSWebCam* mediastreamer2_tester_get_mire_webcam(MSWebCamManager *mgr) {
 	MSWebCam *cam;
 
@@ -74,7 +73,7 @@ static CodecsManager *codecs_manager_new(MSFactory *factory, const char *mime) {
 	for (it=factory->desc_list; it; it=bctbx_list_next(it)) {
 		MSFilterDesc *desc = (MSFilterDesc *)it->data;
 		if (desc->enc_fmt && strcasecmp(desc->enc_fmt, mime) == 0 && (desc->flags & MS_FILTER_IS_ENABLED)) {
-			if (desc->category == MS_FILTER_ENCODER) {
+			if (desc->category == MS_FILTER_ENCODER && desc->category != MS_FILTER_ENCODING_CAPTURER) {
 				cm->encoders = bctbx_list_append(cm->encoders, desc);
 			} else if (desc->category == MS_FILTER_DECODER) {
 				cm->decoders = bctbx_list_append(cm->decoders, desc);
@@ -145,6 +144,8 @@ static int tester_before_all(void) {
 	_h264_codecs_manager = codecs_manager_new(_factory, "h264");
 
 	cam_manager = ms_factory_get_web_cam_manager(_factory);
+	//Disable h264 camera by default because of potential conflict with other h264 encoders
+	ms_factory_enable_filter_from_name(_factory, "h264camera", FALSE);
 	if(mire != NULL) ms_web_cam_manager_add_cam(cam_manager, mire);
 
 	return 0;
@@ -168,12 +169,15 @@ typedef struct _video_stream_tester_stats_t {
 	int number_of_sent_PLI;
 	int number_of_sent_SLI;
 	int number_of_sent_RPSI;
+	int number_of_sent_NACK;
 
 	int number_of_decoder_decoding_error;
 	int number_of_decoder_first_image_decoded;
 	int number_of_decoder_send_request_pli;
 	int number_of_decoder_send_request_sli;
 	int number_of_decoder_send_request_rpsi;
+
+	int number_of_jitter_update_nack;
 } video_stream_tester_stats_t;
 
 typedef struct _video_stream_tester_t {
@@ -279,22 +283,33 @@ static void event_queue_cb(MediaStream *ms, void *user_pointer) {
 						switch (rtcp_PSFB_get_type(d->packet)) {
 							case RTCP_PSFB_PLI:
 								st->number_of_sent_PLI++;
-                                ms_message("event_queue_cb: [%p] sending PLI %d",st,  st->number_of_sent_PLI);
+								ms_message("event_queue_cb: [%p] sending PLI %d", st, st->number_of_sent_PLI);
 								break;
 							case RTCP_PSFB_SLI:
-                                st->number_of_sent_SLI++;
-                                ms_message("event_queue_cb: [%p] sending SLI %d",st,  st->number_of_sent_SLI);
-
+								st->number_of_sent_SLI++;
+								ms_message("event_queue_cb: [%p] sending SLI %d", st, st->number_of_sent_SLI);
 								break;
 							case RTCP_PSFB_RPSI:
 								st->number_of_sent_RPSI++;
-                                 ms_message("event_queue_cb: [%p] sending RPSI %d",st,  st->number_of_sent_RPSI);
+								ms_message("event_queue_cb: [%p] sending RPSI %d", st, st->number_of_sent_RPSI);
+								break;
+							default:
+								break;
+						}
+					} else if (rtcp_is_RTPFB(d->packet)) {
+						switch (rtcp_RTPFB_get_type(d->packet)) {
+							case RTCP_RTPFB_NACK:
+								st->number_of_sent_NACK++;
+								ms_message("event_queue_cb: [%p] sending NACK %d", st, st->number_of_sent_NACK);
 								break;
 							default:
 								break;
 						}
 					}
 				} while (rtcp_next_packet(d->packet));
+			} else if (evt == ORTP_EVENT_JITTER_UPDATE_FOR_NACK) {
+				st->number_of_jitter_update_nack++;
+				ms_message("event_queue_cb: [%p] jitter update NACK %d", st, st->number_of_jitter_update_nack);
 			}
 			ortp_event_destroy(ev);
 		}
@@ -327,7 +342,7 @@ static void destroy_video_stream(video_stream_tester_t *vst) {
 	ortp_ev_queue_destroy(vst->stats.q);
 }
 
-static void init_video_streams(video_stream_tester_t *vst1, video_stream_tester_t *vst2, bool_t avpf, bool_t one_way, OrtpNetworkSimulatorParams *params, int payload_type) {
+static void init_video_streams(video_stream_tester_t *vst1, video_stream_tester_t *vst2, bool_t avpf, bool_t one_way, OrtpNetworkSimulatorParams *params, int payload_type, bool_t nack) {
 	PayloadType *pt;
 
 	create_video_stream(vst1, payload_type);
@@ -350,7 +365,14 @@ static void init_video_streams(video_stream_tester_t *vst1, video_stream_tester_
 	}
 
 	if (one_way == TRUE) {
-		video_stream_set_direction(vst1->vs, MediaStreamRecvOnly);
+		media_stream_set_direction(&vst1->vs->ms, MediaStreamRecvOnly);
+	}
+
+	if (nack == TRUE) {
+		rtp_session_enable_avpf_feature(video_stream_get_rtp_session(vst2->vs), ORTP_AVPF_FEATURE_GENERIC_NACK, TRUE);
+		rtp_session_enable_avpf_feature(video_stream_get_rtp_session(vst2->vs), ORTP_AVPF_FEATURE_IMMEDIATE_NACK, TRUE);
+		video_stream_enable_retransmission_on_nack(vst1->vs, TRUE);
+		video_stream_enable_retransmission_on_nack(vst2->vs, TRUE);
 	}
 
 	BC_ASSERT_EQUAL(video_stream_start(vst1->vs, &rtp_profile, vst2->local_ip, vst2->local_rtp, vst2->local_ip, vst2->local_rtcp, payload_type, 50, vst1->cam), 0,int,"%d");
@@ -393,7 +415,7 @@ static void basic_video_stream_base(int payload_type) {
 	video_stream_tester_t* marielle=video_stream_tester_new();
 	video_stream_tester_t* margaux=video_stream_tester_new();
 
-	init_video_streams(marielle, margaux, FALSE, FALSE, NULL, payload_type);
+	init_video_streams(marielle, margaux, FALSE, FALSE, NULL, payload_type, FALSE);
 
 	BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms,
 													&margaux->vs->ms,
@@ -406,6 +428,9 @@ static void basic_video_stream_base(int payload_type) {
 	video_stream_get_local_rtp_stats(margaux->vs, &margaux->stats.rtp);
 
 	uninit_video_streams(marielle, margaux);
+
+	video_stream_tester_destroy(margaux);
+	video_stream_tester_destroy(marielle);
 }
 
 static void basic_video_stream_vp8(void) {
@@ -434,7 +459,7 @@ static void basic_one_way_video_stream(void) {
 	bool_t supported = ms_factory_codec_supported(_factory, "vp8");
 
 	if (supported) {
-		init_video_streams(marielle, margaux, FALSE, TRUE, NULL,VP8_PAYLOAD_TYPE);
+		init_video_streams(marielle, margaux, FALSE, TRUE, NULL,VP8_PAYLOAD_TYPE, FALSE);
 
 		BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms, &marielle->stats.number_of_RR, 2, 15000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
 		video_stream_get_local_rtp_stats(marielle->vs, &marielle->stats.rtp);
@@ -455,7 +480,7 @@ static void codec_change_for_video_stream(void) {
 	bool_t mp4v_supported = ms_factory_codec_supported(_factory, "mp4v-es");
 
 	if (vp8_supported) {
-		init_video_streams(marielle, margaux, FALSE, FALSE, NULL, VP8_PAYLOAD_TYPE);
+		init_video_streams(marielle, margaux, FALSE, FALSE, NULL, VP8_PAYLOAD_TYPE, FALSE);
 		BC_ASSERT_TRUE(wait_for_until(&marielle->vs->ms, &margaux->vs->ms, &marielle->stats.number_of_decoder_first_image_decoded, 1, 2000));
 		BC_ASSERT_TRUE(wait_for_until(&marielle->vs->ms, &margaux->vs->ms, &margaux->stats.number_of_decoder_first_image_decoded, 1, 2000));
 		BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms, &marielle->stats.number_of_SR, 2, 15000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
@@ -490,10 +515,9 @@ static void multicast_video_stream(void) {
 	marielle->local_rtcp=0; /*no rtcp*/
 	video_stream_tester_set_local_ip(margaux,"0.0.0.0");
 
-
 	if (supported) {
 		int dummy=0;
-		init_video_streams(marielle, margaux, FALSE, TRUE, NULL,VP8_PAYLOAD_TYPE);
+		init_video_streams(marielle, margaux, FALSE, TRUE, NULL,VP8_PAYLOAD_TYPE, FALSE);
 
 		BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms, &margaux->stats.number_of_SR, 2, 15000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
 
@@ -536,7 +560,7 @@ static void avpf_video_stream_base(int payload_type) {
 		params.enabled = TRUE;
 		params.loss_rate = 5.;
 		params.rtp_only = TRUE;
-		init_video_streams(marielle, margaux, TRUE, FALSE, &params,payload_type);
+		init_video_streams(marielle, margaux, TRUE, FALSE, &params,payload_type, FALSE);
 
         BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms, &marielle->stats.number_of_SR, 2, 15000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
 
@@ -618,7 +642,7 @@ static void avpf_rpsi_count(void) {
 	margaux->cam = mediastreamer2_tester_get_mire_webcam(ms_factory_get_web_cam_manager(_factory));
 
 	if (supported) {
-		init_video_streams(marielle, margaux, TRUE, FALSE, &params,VP8_PAYLOAD_TYPE);
+		init_video_streams(marielle, margaux, TRUE, FALSE, &params,VP8_PAYLOAD_TYPE, FALSE);
 		BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms,  &marielle->stats.number_of_decoder_first_image_decoded, 1, 10000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
 		BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms,  &margaux->stats.number_of_decoder_first_image_decoded, 1, 10000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
 
@@ -659,7 +683,7 @@ static void video_stream_first_iframe_lost_base(int payload_type) {
 		/* Make sure first Iframe is lost. */
 		params.enabled = TRUE;
 		params.loss_rate = 100.;
-		init_video_streams(marielle, margaux, FALSE, FALSE, &params, payload_type);
+		init_video_streams(marielle, margaux, FALSE, FALSE, &params, payload_type, FALSE);
 		wait_for_until(&marielle->vs->ms, &margaux->vs->ms,&dummy,1,1000);
 
 		/* Use 10% packet lost to be sure to have decoding errors. */
@@ -729,7 +753,7 @@ static void avpf_video_stream_first_iframe_lost_base(int payload_type) {
 		/* Make sure first Iframe is lost. */
 		params.enabled = TRUE;
 		params.loss_rate = 100.;
-		init_video_streams(marielle, margaux, TRUE, FALSE, &params, payload_type);
+		init_video_streams(marielle, margaux, TRUE, FALSE, &params, payload_type, FALSE);
 		wait_for_until(&marielle->vs->ms, &margaux->vs->ms,&dummy,1,1000);
 
 		/* Remove the lost to be sure that a PLI will be sent and not a SLI. */
@@ -788,7 +812,7 @@ static void avpf_high_loss_video_stream_base(float rate, int payload_type) {
 	if (supported) {
 		params.enabled = TRUE;
 		params.loss_rate = rate;
-		init_video_streams(marielle, margaux, TRUE, FALSE, &params, payload_type);
+		init_video_streams(marielle, margaux, TRUE, FALSE, &params, payload_type, FALSE);
 		BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms,
 			&marielle->stats.number_of_SR, 10, 15000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
 		switch (payload_type) {
@@ -844,7 +868,7 @@ static void video_configuration_stream_base(MSVideoConfiguration* asked, MSVideo
 		margaux->vconf->vsize=asked->vsize;
 		margaux->vconf->fps=asked->fps;
 
-		init_video_streams(marielle, margaux, FALSE, TRUE, NULL,payload_type);
+		init_video_streams(marielle, margaux, FALSE, TRUE, NULL,payload_type, FALSE);
 
 		BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms, &marielle->stats.number_of_RR, 4, 30000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
 
@@ -900,7 +924,7 @@ static void video_configuration_stream_h264(void) {
 	asked.fps=expected.fps=12;
 	asked.vsize.width=expected.vsize.width=MS_VIDEO_SIZE_VGA_W;
 	asked.vsize.height=expected.vsize.height=MS_VIDEO_SIZE_VGA_H;
-	video_configuration_stream_base(&asked,&expected,VP8_PAYLOAD_TYPE);
+	video_configuration_stream_base(&asked,&expected,H264_PAYLOAD_TYPE);
 
 	/*Test video rotation (inverted height <-> width). Not supported on desktop
 	because no real use case yet.*/
@@ -914,28 +938,138 @@ static void video_configuration_stream_h264(void) {
 #endif
 }
 
+video_stream_tester_t* video_stream_tester_elph264_cam_new(void) {
+	video_stream_tester_t* vst = ms_new0(video_stream_tester_t,1);
+	video_stream_tester_set_local_ip(vst,"127.0.0.1");
+	vst->cam = ms_web_cam_manager_get_cam(ms_factory_get_web_cam_manager(_factory), "ELP-USB100W04H: /dev/elp-h264");
+	vst->local_rtp=-1; /*random*/
+	vst->local_rtcp=-1; /*random*/
+	vst->vconf = ms_new0(MSVideoConfiguration, 1);
+	vst->vconf->vsize = MS_VIDEO_SIZE_720P;
+	vst->vconf->fps = 30;
+	vst->vconf->required_bitrate = 500000;
+	return  vst;
+}
+
+static void video_stream_elph264_camera(void) {
+	MSVideoConfiguration asked;
+	MSVideoConfiguration expected;
+	video_stream_tester_t* marielle = NULL;
+	video_stream_tester_t* margaux = NULL;
+	PayloadType* pt = NULL;
+	bool_t supported = FALSE;
+
+	if (bctbx_file_exist("/dev/elp-h264") != 0) {
+		ms_error("Ignoring video_stream_elph264_camera test because required device is not present.");
+		return;
+	} else {
+		ms_factory_enable_filter_from_name(_factory, "h264camera", TRUE);
+	}
+	asked.bitrate_limit=expected.bitrate_limit=1024000;
+	asked.required_bitrate=expected.required_bitrate=1024000;
+	asked.fps=expected.fps=30;
+	asked.vsize.width=expected.vsize.width=MS_VIDEO_SIZE_720P_W;
+	asked.vsize.height=expected.vsize.height=MS_VIDEO_SIZE_720P_H;
+
+	marielle=video_stream_tester_elph264_cam_new();
+	margaux=video_stream_tester_elph264_cam_new();
+	pt = rtp_profile_get_payload(&rtp_profile, H264_PAYLOAD_TYPE);
+	supported = pt ? ms_factory_codec_supported(_factory, pt->mime_type) : FALSE;
+
+	if (supported) {
+		margaux->vconf = ms_new0(MSVideoConfiguration, 1);
+		margaux->vconf->required_bitrate = asked.required_bitrate;
+		margaux->vconf->bitrate_limit = asked.bitrate_limit;
+		margaux->vconf->vsize = asked.vsize;
+		margaux->vconf->fps = asked.fps;
+
+		init_video_streams(marielle, margaux, FALSE, TRUE, NULL, H264_PAYLOAD_TYPE, FALSE);
+
+		BC_ASSERT_TRUE(wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms, &marielle->stats.number_of_RR, 4, 30000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats));
+
+		video_stream_get_local_rtp_stats(marielle->vs, &marielle->stats.rtp);
+		video_stream_get_local_rtp_stats(margaux->vs, &margaux->stats.rtp);
+
+		BC_ASSERT_TRUE(ms_video_size_equal(video_stream_get_received_video_size(marielle->vs), margaux->vconf->vsize));
+		BC_ASSERT_TRUE(fabs(video_stream_get_received_framerate(marielle->vs)-margaux->vconf->fps) < 2);
+
+		uninit_video_streams(marielle, margaux);
+	} else {
+		ms_error("H264 codec is not supported!");
+	}
+	video_stream_tester_destroy(marielle);
+	video_stream_tester_destroy(margaux);
+}
+
 static void video_configuration_stream_all_h264_codec_combinations(void) {
 	codecs_manager_test_all_combinations(_h264_codecs_manager, video_configuration_stream_h264);
 }
 
+static void video_stream_normal_loss_with_retransmission_on_nack(void) {
+	video_stream_tester_t* marielle=video_stream_tester_new();
+	video_stream_tester_t* margaux=video_stream_tester_new();
+	OrtpNetworkSimulatorParams params = { 0 };
+	bool_t supported = ms_factory_codec_supported(_factory, "vp8");
+
+	if (supported) {
+		JBParameters initial_jitter_params, current_jitter_params;
+		int dummy = 0;
+		params.enabled = TRUE;
+		params.loss_rate = 3.;
+
+		init_video_streams(marielle, margaux, TRUE, FALSE, &params, VP8_PAYLOAD_TYPE, TRUE);
+
+		rtp_session_get_jitter_buffer_params(margaux->vs->ms.sessions.rtp_session, &initial_jitter_params);
+
+		wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms,
+			&dummy, 1, 10000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats);
+
+		BC_ASSERT_LOWER(margaux->stats.number_of_sent_SLI, 5, int, "%d");
+
+		/* Since there is some loss_rate the nack context should increase the min_size */
+		rtp_session_get_jitter_buffer_params(margaux->vs->ms.sessions.rtp_session, &current_jitter_params);
+		BC_ASSERT_GREATER(current_jitter_params.min_size, initial_jitter_params.min_size, int, "%d");
+
+		params.loss_rate = 0.;
+		rtp_session_enable_network_simulation(marielle->vs->ms.sessions.rtp_session, &params);
+		rtp_session_enable_network_simulation(margaux->vs->ms.sessions.rtp_session, &params);
+
+		wait_for_until_with_parse_events(&marielle->vs->ms, &margaux->vs->ms,
+			&dummy, 1, 6000, event_queue_cb, &marielle->stats, event_queue_cb, &margaux->stats);
+
+		/* Since no NACK is sent in the last 5 seconds, the jitter min size is set back to default */
+		rtp_session_get_jitter_buffer_params(margaux->vs->ms.sessions.rtp_session, &current_jitter_params);
+		BC_ASSERT_EQUAL(current_jitter_params.min_size, initial_jitter_params.min_size, int, "%d");
+
+		uninit_video_streams(marielle, margaux);
+	} else {
+		ms_error("Codec is not supported!");
+	}
+
+	video_stream_tester_destroy(marielle);
+	video_stream_tester_destroy(margaux);
+}
+
 static test_t tests[] = {
-	TEST_NO_TAG("Basic video stream VP8"                   , basic_video_stream_vp8),
-	TEST_NO_TAG("Basic video stream H264"                  , basic_video_stream_all_h264_codec_combinations),
-	TEST_NO_TAG("Multicast video stream"                   , multicast_video_stream),
-	TEST_NO_TAG("Basic one-way video stream"               , basic_one_way_video_stream),
-	TEST_NO_TAG("Codec change for video stream"            , codec_change_for_video_stream),
-	TEST_NO_TAG("AVPF video stream VP8"                    , avpf_video_stream_vp8),
-	TEST_NO_TAG("AVPF video stream H264"                   , avpf_video_stream_all_h264_codec_combinations),
-	TEST_NO_TAG("AVPF high-loss video stream VP8"          , avpf_high_loss_video_stream_vp8),
-	TEST_NO_TAG("AVPF high-loss video stream H264"         , avpf_high_loss_video_stream_all_h264_codec_conbinations),
-	TEST_NO_TAG("AVPF very high-loss video stream VP8"     , avpf_very_high_loss_video_stream_vp8),
-	TEST_NO_TAG("AVPF video stream first iframe lost VP8"  , avpf_video_stream_first_iframe_lost_vp8),
-	TEST_NO_TAG("AVPF video stream first iframe lost H264" , avpf_video_stream_first_iframe_lost_all_h264_codec_combinations),
-	TEST_NO_TAG("AVP video stream first iframe lost VP8"   , video_stream_first_iframe_lost_vp8),
-	TEST_NO_TAG("AVP video stream first iframe lost H264"  , video_stream_first_iframe_lost_all_h264_codec_combinations),
-	TEST_NO_TAG("Video configuration VP8"                  , video_configuration_stream_vp8),
-	TEST_NO_TAG("Video configuration H264"                 , video_configuration_stream_all_h264_codec_combinations),
-	TEST_NO_TAG("AVPF RPSI count"                          , avpf_rpsi_count)
+	TEST_NO_TAG("Basic video stream VP8"                     , basic_video_stream_vp8),
+	TEST_NO_TAG("Basic video stream H264"                    , basic_video_stream_all_h264_codec_combinations),
+	TEST_NO_TAG("Multicast video stream"                     , multicast_video_stream),
+	TEST_NO_TAG("Basic one-way video stream"                 , basic_one_way_video_stream),
+	TEST_NO_TAG("Codec change for video stream"              , codec_change_for_video_stream),
+	TEST_NO_TAG("AVPF video stream VP8"                      , avpf_video_stream_vp8),
+	TEST_NO_TAG("AVPF video stream H264"                     , avpf_video_stream_all_h264_codec_combinations),
+	TEST_NO_TAG("AVPF high-loss video stream VP8"            , avpf_high_loss_video_stream_vp8),
+	TEST_NO_TAG("AVPF high-loss video stream H264"           , avpf_high_loss_video_stream_all_h264_codec_conbinations),
+	TEST_NO_TAG("AVPF very high-loss video stream VP8"       , avpf_very_high_loss_video_stream_vp8),
+	TEST_NO_TAG("AVPF video stream first iframe lost VP8"    , avpf_video_stream_first_iframe_lost_vp8),
+	TEST_NO_TAG("AVPF video stream first iframe lost H264"   , avpf_video_stream_first_iframe_lost_all_h264_codec_combinations),
+	TEST_NO_TAG("AVP video stream first iframe lost VP8"     , video_stream_first_iframe_lost_vp8),
+	TEST_NO_TAG("AVP video stream first iframe lost H264"    , video_stream_first_iframe_lost_all_h264_codec_combinations),
+	TEST_NO_TAG("Video configuration VP8"                    , video_configuration_stream_vp8),
+	TEST_NO_TAG("Video configuration H264"                   , video_configuration_stream_all_h264_codec_combinations),
+	TEST_NO_TAG("Video steam camera ELPH264"                 , video_stream_elph264_camera),
+	TEST_NO_TAG("AVPF RPSI count"                            , avpf_rpsi_count),
+	TEST_NO_TAG("Video stream normal loss with retransmission on NACK" , video_stream_normal_loss_with_retransmission_on_nack),
 };
 
 test_suite_t video_stream_test_suite = {
