@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include <math.h>
@@ -700,12 +700,12 @@ void setup_media_streams(MediastreamDatas* args) {
 	MSFactory *factory;
 	ortp_init();
 	if (args->logfile)
-		ortp_set_log_file(args->logfile);
+		bctbx_set_log_file(args->logfile);
 
 	if (args->is_verbose) {
-		ortp_set_log_level_mask(ORTP_LOG_DOMAIN, ORTP_DEBUG|ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+		bctbx_set_log_level(BCTBX_LOG_DOMAIN, BCTBX_LOG_DEBUG);
 	} else {
-		ortp_set_log_level_mask(ORTP_LOG_DOMAIN, ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
+		bctbx_set_log_level(BCTBX_LOG_DOMAIN, BCTBX_LOG_MESSAGE);
 	}
 
 	args->factory = factory = ms_factory_new_with_voip();
@@ -980,6 +980,14 @@ void setup_media_streams(MediastreamDatas* args) {
 		args->session=args->video->ms.sessions.rtp_session;
 
 		ms_filter_call_method(args->video->output,MS_VIDEO_DISPLAY_ZOOM, zoom);
+
+		if (args->two_windows) {
+			// Always enable mirroring for output 2
+			bool_t mirroring=TRUE;
+			ms_filter_call_method(args->video->output2, MS_VIDEO_DISPLAY_ENABLE_MIRRORING, &mirroring);
+		}
+
+
 		if (args->enable_srtp) {
 			ms_message("SRTP enabled: %d",
 				video_stream_enable_strp(
@@ -1313,7 +1321,7 @@ static bool_t parse_ice_addr(char *addr, char *type, size_t type_len, char *ip,
 	semicolon=strrchr(addr,':');
 	if (semicolon==NULL) return FALSE;
 	slen=MIN(strlen(semicolon+1),type_len);
-	strncpy(type,semicolon+1,slen);
+	strncpy(type,semicolon+1,type_len);
 	type[slen]='\0';
 	*semicolon='\0';
 	return parse_addr(addr,ip,ip_len,port);
